@@ -31,6 +31,9 @@
 (define-application-frame inspector ()
   ((dico :initform (make-hash-table) :reader dico)
    (cons-cell-dico :initform (make-hash-table) :reader cons-cell-dico)
+   (disassembly-dico :initform (make-hash-table) :reader disassembly-dico
+		     :documentation "A hash table specifying which
+functions should display disassembly")
    (print-length :initform (make-hash-table) :reader print-length
 		 :documentation "A hash table mapping list objects to
 their specific print lengths, if they have one.")
@@ -88,7 +91,7 @@ information is present."))
   (cond ((member object *inspected-objects*)
          (with-output-as-presentation
              (pane object (presentation-type-of object)) 
-           (princ "===")))		; Prevent infinite loops
+           (princ "===" pane)))		; Prevent infinite loops
         ((not (gethash object (dico *application-frame*)))
          (inspect-object-briefly object pane))
         (t
@@ -106,12 +109,12 @@ information is present."))
 (defmethod inspect-object-briefly (object pane)
   (with-output-as-presentation
       (pane object (presentation-type-of object))
-    (princ "...")))
+    (princ "..." pane)))
 
 (defmethod inspect-object (object pane)
   (with-output-as-presentation
       (pane object (presentation-type-of object))  
-    (print object pane)))
+    (prin1 object pane)))
 
 
 (define-presentation-type settable-slot ()
@@ -122,9 +125,9 @@ information is present."))
   :inherit-from t)
 
 (define-presentation-method present (object (type settable-slot) 
-				     stream
-				     (view textual-view)
-				     &key acceptably for-context-type)
+					    stream
+					    (view textual-view)
+					    &key acceptably for-context-type)
   (declare (ignore acceptably for-context-type))
   (format stream "~s" (cdr object)))
 
@@ -132,74 +135,132 @@ information is present."))
   "Cause text output from BODY to be formatted in a heading font. This
 could be boldface, or a different style, or even another font."
   `(with-text-face (,stream :bold)
-     ,@body))
+    ,@body))
 
-(defmacro inspector-table (header &body body)
-  "Present OBJECT (captured from environment) in tabular form, with
-HEADER printed in a box at the top. BODY should output the rows of the
-table using INSPECTOR-TABLE-ROW. Also capured from the macro's
-environment is PANE, which is the pane on which the table will be
-drawn."
-  `(with-output-as-presentation
-       (pane object (presentation-type-of object))
-     (formatting-table (pane)
-       (formatting-column (pane)
-         (formatting-cell (pane)
-           (surrounding-output-with-border (pane)
-	     (with-heading-style (pane)
-	       ,header)))
-         (formatting-cell (pane)
-           (formatting-table (pane)
-             ,@body))))
-    (print-documentation object pane)))
+(defmacro inspector-table ((object pane) header &body body)
+  "Present OBJECT in tabular form on PANE, with HEADER evaluated to
+print a label in a box at the top. BODY should output the rows of the
+table, possibly using INSPECTOR-TABLE-ROW."
+  (let ((evaluated-pane (gensym "pane"))
+	(evaluated-object (gensym "object")))
+    `(let ((,evaluated-pane ,pane)
+	   (,evaluated-object ,object))
+      (with-output-as-presentation
+	  (pane ,evaluated-object
+		(presentation-type-of ,evaluated-object))
+	(formatting-table (,evaluated-pane)
+	  (formatting-column (,evaluated-pane)
+	    (formatting-cell (,evaluated-pane)
+	      (surrounding-output-with-border (,evaluated-pane)
+		(with-heading-style (,evaluated-pane)
+		  ,header)))
+	    (formatting-cell (,evaluated-pane)
+	      (formatting-table (,evaluated-pane)
+		,@body))))
+	(print-documentation (if (eql (class-of ,evaluated-object)
+				      (find-class 'standard-class))
+				 ,evaluated-object
+				 (class-of ,evaluated-object))
+			     ,evaluated-pane)))))
 
-(defmacro inspector-table-row (left right)
-  "Output a table row with two items, LEFT and RIGHT, in the
-environment created by INSPECTOR-TABLE."
-  `(formatting-row (pane)
-     (formatting-cell (pane :align-x :right)
-       (with-heading-style (pane)
-	 ,left))
-     (formatting-cell (pane)
-       ,right)))
+(defmacro inspector-table-row ((pane) left right)
+  "Output a table row with two items, produced by evaluating LEFT and
+RIGHT, on PANE. This should be used only within INSPECTOR-TABLE."
+  (let ((evaluated-pane (gensym "pane")))
+    `(let ((,evaluated-pane ,pane))
+      (formatting-row (,evaluated-pane)
+	(formatting-cell (,evaluated-pane :align-x :right)
+	  (with-heading-style (,evaluated-pane)
+	    ,left))
+	(formatting-cell (,evaluated-pane)
+	  ,right)))))
 
+(defmacro inspector-table-rows ((pane) &body rows)
+  "Output a bunch of rows with INSPECTOR-TABLE-ROW on PANE. Each row
+is a list of a label and a value."
+  (let ((evaluated-pane (gensym "pane")))
+    `(let ((,evaluated-pane ,pane))
+      ,@(loop for row in rows
+	      collect (destructuring-bind (label value) row
+			`(inspector-table-row (,evaluated-pane)
+			  (princ ,label ,evaluated-pane)
+			  (inspect-object ,value ,evaluated-pane)))))))
+
+;; The error handler shouldn't be necessary, but it works around an
+;; ACL bug and shouldn't mess anything up on other lisps. The warning
+;; handler is there in case DOCUMENTATION raises a warning, to tell
+;; lisp that we don't care and it shouldn't go alarming the user.
 (defun print-documentation (object pane)
   "Print OBJECT's documentation, if any, to PANE"
-  (when (handler-bind ((warning #'muffle-warning))
-	  (documentation object t))
+  (when (handler-case (documentation object t)
+	  (error ())
+	  (warning ()))
     (with-heading-style (pane)
       (format pane "~&Documentation: "))
     (princ (documentation object t) pane)))
 
+(defun display-class-superclasses (class pane)
+  "Display the superclasses of CLASS with an INSPECTOR-TABLE-ROW"
+  (when (clim-mop:class-direct-superclasses class)
+    (inspector-table-row (pane)
+	(princ "Superclasses" pane)
+      (inspect-vertical-list (clim-mop:class-direct-superclasses class)
+			     pane))))
+
+(defun display-class-subclasses (class pane)
+  "Display the subclasses of CLASS with an INSPECTOR-TABLE-ROW"
+  (when (clim-mop:class-direct-subclasses class)
+    (inspector-table-row (pane)
+	(princ "Subclasses" pane)
+      (inspect-vertical-list (clim-mop:class-direct-subclasses class)
+			     pane))))
+
+(defun display-object-slot (object slot pane &key display-lists-vertically)
+  "Display a slot of OBJECT onto PANE in the way normally used when
+inspecting standard objects. SLOT must be a MOP SLOT-DEFINITION
+object. If DISPLAY-LISTS-VERTICALLY is t and the slot value is a list,
+it will be displayed with INSPECT-VERTICAL-LIST."
+  (let ((slot-name (clim-mop:slot-definition-name slot)))
+    (inspector-table-row (pane)
+	(with-output-as-presentation
+	    (pane (cons object slot-name) 'settable-slot)
+	  (format pane "~a:" slot-name))
+      (if (slot-boundp object slot-name)
+	  (let ((slot-value (slot-value object slot-name)))
+	    (if (and display-lists-vertically
+		     (listp slot-value))
+		(inspect-vertical-list slot-value pane
+				       :honor-dico t)
+		(inspect-object slot-value pane)))
+	  (format pane "#<unbound slot>")))))
+
 (defun inspect-structure-or-object (object pane)
   "Inspect a structure or an object. Since both can be inspected in
 roughly the same way, the common code is in this function, which is
 called by the INSPECT-OBJECT methods for both standard objects and
 structure objects."
   (let ((class (class-of object)))
-    (inspector-table
-     (print (class-name class) pane)
-     (when (clim-mop:class-direct-superclasses class)
-       (inspector-table-row
-	(princ "Superclasses" pane)
-	(dolist (superclass (clim-mop:class-direct-superclasses class))
-	  (inspect-object superclass pane)
-	  (terpri pane))))
-     (when (clim-mop:class-direct-subclasses class)
-       (inspector-table-row
-	(princ "Subclasses" pane)
-	(dolist (subclass (clim-mop:class-direct-subclasses class))
-	  (inspect-object subclass pane)
-	  (terpri pane))))
-     (loop for slot in (reverse (clim-mop:class-slots class))
-	   do (let ((slot-name (clim-mop:slot-definition-name slot)))
-		(inspector-table-row
-		 (with-output-as-presentation
-		     (pane (cons object slot-name) 'settable-slot)
-		   (format pane "~a:" slot-name))
-		 (if (slot-boundp object slot-name)
-		     (inspect-object (slot-value object slot-name) pane)
-		     (format pane "#<unbound slot>"))))))))
+    (inspector-table (object pane)
+	(print (class-name class) pane)
+      ;; Display superclasses and subclasses
+      (display-class-superclasses class pane)
+      (display-class-subclasses class pane)
+      (dolist (slot (reverse (clim-mop:class-slots class)))
+	(display-object-slot object slot pane)))))
+
+(defun inspect-standard-class (object pane)
+  "Inspect a STANDARD-CLASS. This works almost the same way as
+inspecting a standard object, but with a few differences. This should
+also be used to inspect BUILD-IN-CLASSes."
+  (let ((class (class-of object)))
+    (inspector-table (object pane)
+	(print (class-name class) pane)
+      ;; Display superclasses and subclasses
+      (display-class-superclasses class pane)
+      (display-class-subclasses class pane)
+      (dolist (slot (reverse (clim-mop:class-slots class)))
+	(display-object-slot object slot pane
+			     :display-lists-vertically t)))))
 
 ;; Try to print the normal, textual representation of an object, but
 ;; if that's too long, make an abbreviated "instance of ~S" version.
@@ -211,14 +272,15 @@ that are allowed before abbreviation kicks in")
 (defun inspect-structure-or-object-briefly (object pane)
   (with-output-as-presentation
       (pane object (presentation-type-of object))
-    (handler-case
-	(let ((representation (with-output-to-string (string)
-				(prin1 object string))))
-	  (if (< (length representation) *object-representation-max-length*)
-	      (princ representation pane)
-	      (format pane "#<~S ...>" (class-name (class-of object)))))
-      (error ()
-	(format pane "#<unprintable ~S>" (class-name (class-of object)))))))
+    (with-text-family (pane :fix)
+      (handler-case
+	  (let ((representation (with-output-to-string (string)
+				  (prin1 object string))))
+	    (if (< (length representation) *object-representation-max-length*)
+		(princ representation pane)
+		(format pane "#<~S ...>" (class-name (class-of object)))))
+	(error ()
+	  (format pane "#<unprintable ~S>" (class-name (class-of object))))))))
 
 (defmethod inspect-object-briefly ((object standard-object) pane)
   (inspect-structure-or-object-briefly object pane))
@@ -235,6 +297,12 @@ that are allowed before abbreviation kicks in")
 (defmethod inspect-object ((object structure-object) pane)
   (inspect-structure-or-object object pane))
 
+(defmethod inspect-object ((object standard-class) pane)
+  (inspect-standard-class object pane))
+
+(defmethod inspect-object ((object built-in-class) pane)
+  (inspect-standard-class object pane))
+
 (defmethod inspect-object ((object condition) pane)
   (inspect-structure-or-object object pane))
 
@@ -271,36 +339,76 @@ moderately-sized list."
 	  (formatting-cell (pane)
 	    (inspect-object (cdr object) pane))))))
 
+(defun inspect-vertical-list (object pane &key honor-dico)
+  "Inspect a list without the parentheses, putting each element on a
+new line. This is useful for showing things like direct class
+subclasses, since displaying those as a plain list looks ugly and is
+inconvenient to use. If HONOR-DICO is t, this will respect DICO and
+display '...' if OBJECT is not in DICO."
+  ;; Ordinarily this would be taken care of in the :around method for
+  ;; INSPECT-OBJECT, but since this is not a normal inspection view,
+  ;; we need to do it ourselves. Yes, it would be better if we could
+  ;; find another way to do this.
+  (let ((*print-length* (or (gethash object (print-length
+					     *application-frame*))
+			    *print-length*)))
+    (if (and honor-dico
+	     (not (gethash object (dico *application-frame*))))
+	(inspect-object-briefly object pane)
+	(with-output-as-presentation
+	    (pane object 'cons)
+	  (formatting-table (pane)
+	    (formatting-column (pane)
+	      (do
+	       ((length 0 (1+ length))
+		(cdr (cdr object) (cdr cdr))
+		(car (car object) (car cdr)))
+	       ((cond ((eq nil cdr)
+		       (formatting-cell (pane) (inspect-object car pane))
+		       t)
+		      ((not (consp cdr))
+		       (formatting-cell (pane) (inspect-object car pane))
+		       (formatting-cell (pane) (princ "." pane))
+		       (formatting-cell (pane) (inspect-object cdr pane))
+		       t)
+		      ((and *print-length* (>= length *print-length*))
+		       (with-output-as-presentation
+			   (pane object 'long-list-tail)
+			 (formatting-cell (pane) (princ "..." pane)))
+		       t)
+		      (t nil)))
+		(formatting-cell (pane) (inspect-object car pane)))))))))
+
 (defun inspect-cons-as-list (object pane)
   "Inspect a cons cell in a traditional, plain-text format. The only
 difference between this and simply using the Lisp printer is that this
 code takes advantage of CLIM's tables and presentations to make the
 list as interactive as you would expect."
   (with-output-as-presentation
-    (pane object 'cons)
+      (pane object 'cons)
     (formatting-table (pane)
       (formatting-row (pane)
         (formatting-cell (pane)
           (princ "(" pane))
         (do
-          ((length 0 (1+ length))
-           (cdr (cdr object) (cdr cdr))
-           (car (car object) (car cdr)))
-          ((cond ((eq nil cdr)
-                  (formatting-cell (pane) (inspect-object car pane))
-                  (formatting-cell (pane) (princ ")" pane))
-                  t)
-                 ((not (consp cdr))
-                  (formatting-cell (pane) (inspect-object car pane))
-                  (formatting-cell (pane) (princ "." pane))
-                  (formatting-cell (pane) (inspect-object cdr pane))
-                  (formatting-cell (pane) (princ ")" pane))
-                  t)
-		 ((and *print-length* (>= length *print-length*))
-                  (with-output-as-presentation (pane object 'long-list-tail)
-                    (formatting-cell (pane) (princ "...)" pane)))
-                  t)
-                 (t nil)))
+	 ((length 0 (1+ length))
+	  (cdr (cdr object) (cdr cdr))
+	  (car (car object) (car cdr)))
+	 ((cond ((eq nil cdr)
+		 (formatting-cell (pane) (inspect-object car pane))
+		 (formatting-cell (pane) (princ ")" pane))
+		 t)
+		((not (consp cdr))
+		 (formatting-cell (pane) (inspect-object car pane))
+		 (formatting-cell (pane) (princ "." pane))
+		 (formatting-cell (pane) (inspect-object cdr pane))
+		 (formatting-cell (pane) (princ ")" pane))
+		 t)
+		((and *print-length* (>= length *print-length*))
+		 (with-output-as-presentation (pane object 'long-list-tail)
+		   (formatting-cell (pane) (princ "...)" pane)))
+		 t)
+		(t nil)))
           (formatting-cell (pane) (inspect-object car pane)))))))
 
 (defmethod inspect-object ((object cons) pane)
@@ -315,7 +423,7 @@ list as interactive as you would expect."
       (pane object (presentation-type-of object))
     (princ 'hash-table pane)))
 (defmethod inspect-object ((object hash-table) pane)
-  (inspector-table
+  (inspector-table (object pane)
       (format pane "~A (test: ~A)" 'hash-table (hash-table-test object))
     (loop for key being the hash-keys of object
           do (formatting-row (pane)
@@ -326,7 +434,7 @@ list as interactive as you would expect."
 		 (inspect-object (gethash key object) pane))))))
 
 (defmethod inspect-object ((object generic-function) pane)
-  (inspector-table
+  (inspector-table (object pane)
       (format pane "Generic Function: ~s"
 	      (clim-mop:generic-function-name object))
     (dolist (method (clim-mop:generic-function-methods object))
@@ -334,7 +442,8 @@ list as interactive as you would expect."
 	  (pane method (presentation-type-of method))
 	(formatting-row (pane)
 	  (formatting-cell (pane)
-	    (print (method-qualifiers method)))
+	    (with-text-family (pane :fix)
+	      (print (clim-mop:method-qualifiers method) pane)))
 	  (loop for specializer in (clim-mop:method-specializers method)
 		do (formatting-cell (pane)
 		     (if (typep specializer 'clim-mop:eql-specializer)
@@ -369,19 +478,29 @@ use implementation-specific functions to be more informative."
     ;; please add code for it and send patches.
     #-sbcl (generic-print fun)))
 
+;; This is ugly. I think CLIM requires there to be a presentation type
+;; for every class, so we should use FUNCTION---but I'm not sure how
+;; well that will work.
+(define-presentation-type inspected-function ()
+  :inherit-from t)
+
 (defmethod inspect-object ((object function) pane)
   (with-output-as-presentation
-       (pane object (presentation-type-of object))
+      (pane object 'inspected-function)
     (with-heading-style (pane)
       (princ "Function: " pane))
-    (princ (pretty-print-function object) pane)
+    (with-text-family (pane :fix)
+      (princ (pretty-print-function object) pane))
     #+sbcl
     (unless (typep object 'generic-function)
       (with-heading-style (pane)
 	(format pane "~&Type: "))
-      (princ (sb-kernel:%simple-fun-type (sb-kernel:%closure-fun object))
-	     pane))
-    (print-documentation object pane)))
+      (with-text-family (pane :fix)
+	(princ (sb-kernel:%simple-fun-type (sb-kernel:%closure-fun object))
+	       pane)))
+    (print-documentation object pane)
+    (when (gethash object (disassembly-dico *application-frame*))
+      (display-disassembly object pane))))
 
 (defmethod inspect-object-briefly ((object package) pane)
   ;; Display as 'Package: "PACKAGE-NAME"'. We're doing something a
@@ -389,32 +508,34 @@ use implementation-specific functions to be more informative."
   ;; be a tad inconsistent, but the other way looks very odd.
   (with-output-as-presentation
       (pane object (presentation-type-of object))
-    (format pane "Package: ~S" (package-name object))))
+    (princ "Package: " pane)
+    (with-text-family (pane :fix)
+      (princ (package-name object) pane))))
+
+(defun package-exported-symbols (package)
+  "Return a list of all symbols exported by PACKAGE"
+  (let (symbols)
+    (do-external-symbols (symbol package symbols)
+      (push symbol symbols))))
 
 (defmethod inspect-object ((object package) pane)
-  (inspector-table
-    (format pane "Package: ~S" (package-name object))
-    (inspector-table-row
-      (princ "Name:" pane)
+  (inspector-table (object pane)
+      (format pane "Package: ~S" (package-name object))
+    (inspector-table-row (pane)
+	(princ "Name:" pane)
       (inspect-object (package-name object) pane))
-    (inspector-table-row
-      (princ "Nicknames:" pane)
-      (dolist (nick (package-nicknames object))
-        (inspect-object nick pane)))
-    (inspector-table-row
-      (princ "Used by:")
-      ;; FIXME: This should use some sort of list formatting, so that
-      ;; it can obey conventions about *print-length* and reuse code
-      ;; for modifying it. To support this, list printing should
-      ;; support delimiterless, one-item-per-line display.
-      (dolist (used-by (package-used-by-list object))
-	(fresh-line pane)
-	(inspect-object used-by pane)))
-    (inspector-table-row
-      (princ "Uses:")
-      (dolist (uses (package-use-list object))
-	(fresh-line pane)
-	(inspect-object uses pane)))))
+    (inspector-table-row (pane)
+	(princ "Nicknames:" pane)
+      (inspect-vertical-list (package-nicknames object) pane))
+    (inspector-table-row (pane)
+	(princ "Used by:")
+      (inspect-vertical-list (package-used-by-list object) pane))
+    (inspector-table-row (pane)
+	(princ "Uses:")
+      (inspect-vertical-list (package-use-list object) pane))
+    (inspector-table-row (pane)
+	(princ "Exports:")
+      (inspect-vertical-list (package-exported-symbols object) pane))))
 
 (defmethod inspect-object ((object vector) pane)
   (with-output-as-presentation
@@ -463,56 +584,101 @@ them."
   (inspect-complex object pane))
 
 (defmethod inspect-object ((object float) pane)
-  (inspector-table
-    (format pane "float ~S" object)
+  (inspector-table (object pane)
+      (format pane "Float ~S" object)
     (multiple-value-bind (significand exponent sign)
         (decode-float object)
-      (inspector-table-row
-        (princ "sign:")
-        (inspect-object sign pane))
-      (inspector-table-row
-        (princ "significand:")
-        (inspect-object significand pane))
-      (inspector-table-row
-        (princ "exponent:")
-        (inspect-object exponent pane)))
-    (inspector-table-row
-       (princ "radix:")
-       (inspect-object (float-radix object) pane))))
+      (inspector-table-rows (pane)
+	("sign:" sign)
+	("significand:" significand)
+	("exponent:" exponent)))
+    (inspector-table-rows (pane)
+      ("radix:" (float-radix object)))))
+
+(defun iso-8601-format (time)
+  "Return the given universal time in ISO 8601 format. This will raise
+an error if the given time is not a decodable universal time."
+  (multiple-value-bind (sec min hour date month year)
+      (decode-universal-time time 0)
+    (format nil "~4,'0D-~2,'0D-~2,'0DT~2,'0D:~2,'0D:~2,'0DZ"
+	    year month date hour min sec)))
+
+(defmethod inspect-object ((object integer) pane)
+  (flet ((present-in-base (base &key (radix t) (family :fix))
+	   (with-text-family (pane family)
+	     (formatting-cell (pane)
+	       (with-output-as-presentation
+		   (pane object (presentation-type-of object))
+		 (write object :radix radix :base base :stream pane)))))
+	 (print-equals-cell ()
+	   (formatting-cell (pane)
+	     (princ "=" pane))))
+    (inspector-table (object pane)
+	(format pane "Integer ~S" object)
+      (inspector-table-row (pane)
+	  (princ "value:" pane)
+	(formatting-table (pane)
+	  (formatting-row (pane)
+	    ;; Base 10 should be displayed normally, without the
+	    ;; fixed-width font and without the radix.
+	    (present-in-base 10 :radix nil :family :sans-serif)
+	    (print-equals-cell)		; =
+	    (present-in-base 16)	; Hexadecimal
+	    (print-equals-cell)		; =
+	    (present-in-base 8)		; Octal
+	    (print-equals-cell)		; =
+	    (present-in-base 2))))	; Binary
+      (when (<= 0 object 255)
+	(inspector-table-row (pane)
+	    (princ "character:" pane)
+	  (inspect-object (code-char object) pane)))
+      (inspector-table-row (pane)
+	  (princ "length:" pane)
+	(inspect-object (integer-length object) pane))
+      ;; Sometimes we get numbers that can't be interpreted as a
+      ;; time. Those throw an error, and this just isn't printed.
+      (ignore-errors
+	(inspector-table-row (pane)
+	    (princ "as time:" pane)
+	  (with-text-family (pane :fix)
+	    (with-output-as-presentation
+		(pane object (presentation-type-of object))
+	      (princ (iso-8601-format object) pane))))))))
 
 (defmethod inspect-object-briefly ((object symbol) pane)
   (with-output-as-presentation
       (pane object (presentation-type-of object))
-    (prin1 object)))
+    (with-text-family (pane :fix)
+      (prin1 object))))
 
 (defmethod inspect-object ((object symbol) pane)
-  (inspector-table
-    (format pane "Symbol ~S" (symbol-name object))
-    (inspector-table-row
-       (princ "value:")
-       (if (boundp object)
-         (inspect-object (symbol-value object) pane)
-         (princ "unbound")))
-    (inspector-table-row
-       (princ "function:")
-       (if (fboundp object)
-         (inspect-object (symbol-function object) pane)
-         (princ "unbound")))
+  (inspector-table (object pane)
+      (format pane "Symbol ~S" (symbol-name object))
+    (inspector-table-row (pane)
+	(princ "value:" pane)
+      (if (boundp object)
+	  (inspect-object (symbol-value object) pane)
+	  (princ "unbound" pane)))
+    (inspector-table-row (pane)
+	(princ "function:" pane)
+      (if (fboundp object)
+	  (inspect-object (symbol-function object) pane)
+	  (princ "unbound" pane)))
     ;; This is not, strictly speaking, a property of the
     ;; symbol. However, this is useful enough that I think it's worth
     ;; including here, since it can eliminate some minor annoyances.
-    (inspector-table-row
-       (princ "class:")
-       (if (find-class object nil)
-         (inspect-object (find-class object) pane)
-         (princ "unbound")))
-    (inspector-table-row
-       (princ "package:")
-       (inspect-object (symbol-package object) pane))
-    (inspector-table-row
-       (princ "propery list:")
-       (dolist (property (symbol-plist object))
-         (inspect-object property pane)))))
+    (inspector-table-row (pane)
+	(princ "class:" pane)
+      (if (find-class object nil)
+	  (inspect-object (find-class object) pane)
+	  (princ "unbound" pane)))
+    (inspector-table-row (pane)
+	(princ "package:" pane)
+      (inspect-object (symbol-package object) pane))
+    (inspector-table-row (pane)
+	(princ "propery list:" pane)
+      (dolist (property (symbol-plist object))
+	(inspect-object property pane)))))
 
 ;; Characters are so short that displaying them as "..."  takes almost
 ;; as much space as just showing them, and this way is more
@@ -522,17 +688,31 @@ them."
       (pane object (presentation-type-of object))
     (print object pane)))
 (defmethod inspect-object ((object character) pane)
-  (inspector-table
-    (format pane "Character ~S" object)
-    (inspector-table-row
-       (princ "code:" pane)
-       (inspect-object (char-code object) pane))
-    (inspector-table-row
-       (princ "int:" pane)
-       (inspect-object (char-int object) pane))
-    (inspector-table-row
-       (princ "name:" pane)
-       (inspect-object (char-name object) pane))))
+  (inspector-table (object pane)
+      (format pane "Character ~S" object)
+    (inspector-table-rows (pane)
+      ("code:" (char-code object))
+      ("int:"  (char-int object))
+      ("name:" (char-name object)))))
+
+(defmethod inspect-object ((object pathname) pane)
+  (inspector-table (object pane)
+      (princ (if (wild-pathname-p object)
+		 "Wild pathname"
+		 "Pathname"))
+    (inspector-table-rows (pane)
+      ("namestring:" (namestring object))
+      ("host:"       (pathname-host object))
+      ("device:"     (pathname-device object))
+      ("directory:"  (pathname-directory object))
+      ("name:"       (pathname-name object))
+      ("type:"       (pathname-type object))
+      ("version:"    (pathname-version object)))
+    (unless (or (wild-pathname-p object)
+		(not (probe-file object)))
+      (inspector-table-row (pane)
+	  (princ "truename:" pane)
+	(inspect-object (truename object) pane)))))
 
 (defun display-app (frame pane)
   "Display the APP frame of the inspector"
@@ -624,3 +804,55 @@ available in direct slots."
           (with-heading-style (stream)
 	    (format stream "~&Writers: "))
           (present writers (presentation-type-of writers) :stream stream))))))
+
+(define-inspector-command (com-disassemble :name t)
+    ((obj 'inspected-function
+	  :prompt "Select a function"))
+  (when (typep obj 'function)
+    (togglef (gethash obj (disassembly-dico *application-frame*)))))
+
+(define-presentation-to-command-translator disassemble-function
+    (inspected-function com-disassemble inspector
+			:documentation "Toggle Disassembly"
+			:gesture :menu
+			:menu t)
+    (object)
+  (list object))
+
+(defun tracedp (symbol)
+  "Is SYMBOL currently traced?"
+  (member symbol (trace)))
+
+(define-inspector-command (com-trace :name t)
+    ((obj 'symbol
+	  :prompt "Select an fbound symbol"))
+  (when (fboundp obj)
+    (eval `(trace ,obj))))
+
+(define-inspector-command (com-untrace :name t)
+    ((obj 'symbol
+	  :prompt "Select an fbound symbol"))
+  (when (fboundp obj)
+    (eval `(untrace ,obj))))
+
+(define-presentation-to-command-translator trace-symbol
+    (symbol com-trace inspector
+	    :documentation "Trace"
+	    :gesture :menu
+	    :menu t
+	    :tester ((object) (and object
+				   (fboundp object)
+				   (not (tracedp object)))))
+    (object)
+  (list object))
+
+(define-presentation-to-command-translator untrace-symbol
+    (symbol com-untrace inspector
+	    :documentation "Untrace"
+	    :gesture :menu
+	    :menu t
+	    :tester ((object) (and object
+				   (fboundp object)
+				   (tracedp object))))
+    (object)
+  (list object))
\ No newline at end of file
