@@ -434,30 +434,35 @@
 (defparameter *graph-edge-ink* (make-rgb-color 0.72 0.72 0.72))
 (defparameter *graph-text-style* (make-text-style :fix :roman :normal))
 
-(defun class-grapher (stream class inferior-fun)
+(defun class-grapher (stream class inferior-fun &key (orientation :horizontal))
   "Does the graphing for Show Class Superclasses and Subclasses commands"
   (let ((normal-ink +foreground-ink+)
         (arrow-ink  *graph-edge-ink*)
 	(text-style *graph-text-style*))
     (with-drawing-options (stream :text-style text-style)
-      (format-graph-from-roots (list class)
-                               #'(lambda (class stream)
-                                   (with-drawing-options (stream :ink normal-ink
-                                                                 :text-style text-style)
-                                     ;; Present class name rather than class here because the printing of the
-                                     ;; class object itself is rather long and freaks out the pointer doc pane.
-                                     (with-output-as-presentation (stream (clim-mop:class-name class) 'class-name)
-                                        ; (surrounding-output-with-border (stream :shape :drop-shadow)
-				       (princ (clim-mop:class-name class) stream)))) ;)
-                               inferior-fun
-                               :stream stream
-                               :merge-duplicates T
-                               :graph-type :tree
-                               :orientation :horizontal
-                               :arc-drawer
-                               #'(lambda (stream foo bar x1 y1 x2 y2)
-                                   (declare (ignore foo bar))
-                                   (draw-arrow* stream x1 y1 x2 y2 :ink arrow-ink))))))
+      (prog1
+	;; not sure whether anyone wants the return value...
+	(format-graph-from-roots (list class)
+				 #'(lambda (class stream)
+				     (with-drawing-options (stream :ink normal-ink
+								   :text-style text-style)
+				       ;; Present class name rather than class here because the printing of the
+				       ;; class object itself is rather long and freaks out the pointer doc pane.
+				       (with-output-as-presentation (stream (clim-mop:class-name class) 'class-name)
+					; (surrounding-output-with-border (stream :shape :drop-shadow)
+					 (princ (clim-mop:class-name class) stream)))) ;)
+				 inferior-fun
+				 :stream stream
+				 :merge-duplicates T
+				 :graph-type :tree
+				 :orientation orientation
+				 :arc-drawer
+				 #'(lambda (stream foo bar x1 y1 x2 y2)
+				     (declare (ignore foo bar))
+				     (draw-arrow* stream x1 y1 x2 y2 :ink arrow-ink)))
+	;; format-graph-from-roots doesn't do this by default...
+	(when (typep stream 'pane)
+	  (change-space-requirements stream))))))
 
 (defun frob-to-class (spec)
   (if (typep spec 'class)
@@ -468,20 +473,26 @@
                                              :command-table show-commands
                                              :menu "Class Superclasses"
 					     :provide-output-destination-keyword t)
-    ((class-spec 'class-name :prompt "class"))
+    ((class-spec 'class-name :prompt "class")
+     &key
+     (orientation 'keyword :prompt "orientation" :default :horizontal))
   (let ((class (frob-to-class class-spec)))
     (if (null class)
 	(note "~A is not a defined class." class-spec)
-        (class-grapher *standard-output* class #'clim-mop:class-direct-superclasses))))
+        (class-grapher *standard-output* class #'clim-mop:class-direct-superclasses
+                       :orientation orientation))))
 
 (define-command (com-show-class-subclasses :name "Show Class Subclasses"
                                            :command-table show-commands
                                            :menu "Class Subclasses"
 					   :provide-output-destination-keyword t)
-    ((class-spec 'class-name :prompt "class"))
+    ((class-spec 'class-name :prompt "class")
+     &key
+     (orientation 'keyword :prompt "orientation" :default :horizontal))     
   (let ((class (frob-to-class class-spec)))
     (if (not (null class))
-        (class-grapher *standard-output* class #'clim-mop:class-direct-subclasses)
+        (class-grapher *standard-output* class #'clim-mop:class-direct-subclasses
+                       :orientation orientation)
       (note "~A is not a defined class." class-spec))))
 
 
@@ -661,7 +672,8 @@
   #+clisp (clos:specializer-direct-generic-functions specializer)
   #+openmcl-partial-mop
   (openmcl-mop:specializer-direct-generic-functions specializer)
-  #-(or PCL SBCL clisp openmcl-partial-mop)
+  #+scl (clos:specializer-direct-generic-functions specializer)
+  #-(or PCL SBCL scl clisp openmcl-partial-mop)
   (error "Sorry, not supported in your CL implementation. See the function X-SPECIALIZER-DIRECT-GENERIC-FUNCTION if you are interested in fixing this."))
 
 (defun class-funcs (class)
@@ -930,10 +942,10 @@
   "Return the number of internal symbols in PACKAGE."
   ;; We take only the first value, the symbol count, and discard the second, the
   ;; hash table capacity
-  #+cmu  (values (lisp::internal-symbol-count package))
+  #+(or cmu scl)  (values (lisp::internal-symbol-count package))
   #+sbcl (values (sb-int:package-internal-symbol-count package))
   #+clisp (svref (sys::%record-ref *package* 1) 2)
-  #-(or cmu sbcl clisp) (portable-internal-symbol-count package))
+  #-(or cmu scl sbcl clisp) (portable-internal-symbol-count package))
 
 (defun portable-external-symbol-count (package)
   (let ((n 0))
@@ -944,10 +956,10 @@
 
 (defun count-external-symbols (package)
   "Return the number of external symbols in PACKAGE."
-  #+cmu  (values (lisp::external-symbol-count package))
+  #+(or cmu scl)  (values (lisp::external-symbol-count package))
   #+sbcl (values (sb-int:package-external-symbol-count package))
   #+clisp (svref (sys::%record-ref *package* 0) 2)
-  #-(or cmu sbcl clisp) (portable-external-symbol-count package))
+  #-(or cmu scl sbcl clisp) (portable-external-symbol-count package))
 
 (defun package-grapher (stream package inferior-fun)
   "Draw package hierarchy graphs for `Show Package Users' and `Show Used Packages'."
@@ -1022,7 +1034,12 @@
   (terpri stream))
 
 (defun sort-pathnames (list sort-by)
-  list)                 ; <--- FIXME
+  (case sort-by            ; <--- FIXME
+    ('name  (sort list #'string-lessp
+                  :key (lambda (pathname)
+                         (or (file-namestring pathname)
+                             (first (last (pathname-directory pathname)))))))
+    (t list)))
 
 (defun split-sort-pathnames (list group-dirs sort-by)
   (mapcar (lambda (x) (sort-pathnames x sort-by))
@@ -1058,31 +1075,37 @@
 				    :provide-output-destination-keyword t)
     ((pathname 'pathname #+nil(or 'string 'pathname) :prompt "pathname")
      &key
-     #+NIL (sort-by '(member name size modify none) :default 'name)
+     (sort-by '(member name size modify none) :default 'name)
      (show-hidden  'boolean :default nil :prompt "show hidden")
      (hide-garbage 'boolean :default T   :prompt "hide garbage")
      (show-all     'boolean :default nil :prompt "show all")
      (style '(member items list) :default 'items :prompt "listing style")
      (group-directories 'boolean :default T :prompt "group directories?")
-     (full-names 'boolean :default nil :prompt "show full name?"))
+     (full-names 'boolean :default nil :prompt "show full name?")
+     (list-all-direct-subdirectories 'boolean :default nil :prompt "list all direct subdirectories?"))
 
   (let* ((pathname (if (wild-pathname-p pathname) ; Forgot why I did this..
                        (merge-pathnames pathname)
                      pathname))
-         (dir (list-directory (gen-wild-pathname pathname))))
+         (wild-pathname (gen-wild-pathname pathname))
+         (dir (if list-all-direct-subdirectories
+                  (list-directory-with-all-direct-subdirectories wild-pathname)
+                  (list-directory wild-pathname))))
 
     (with-text-family (T :sans-serif)      
       (invoke-as-heading
         (lambda ()
           (format T "Directory contents of ")
-          (present pathname)))
+          (present (directory-namestring pathname) 'pathname)
+          (when (pathname-type pathname)
+            (format T " (only files of type ~a)" (pathname-type pathname)))))
     
       (when (parent-directory pathname)
-        (with-output-as-presentation (T (parent-directory pathname) 'clim:pathname)
+        (with-output-as-presentation (T (strip-filespec (parent-directory pathname)) 'clim:pathname)
           (draw-icon T (standard-icon "up-folder.xpm") :extra-spacing 3)
           (format T "Parent Directory~%")))
 
-      (dolist (group (split-sort-pathnames dir group-directories :none #+NIL sort-by))
+      (dolist (group (split-sort-pathnames dir group-directories sort-by))
         (unless show-all
           (setf group (filter-garbage-pathnames group show-hidden hide-garbage)))
         (ecase style
@@ -1099,7 +1122,8 @@
                  (goatee::reposition-stream-cursor *standard-output*)                 
                  (vertical-gap T))
           (list (dolist (ent group)
-                  (let ((ent (merge-pathnames ent pathname))) ; This is for CMUCL, see above. (fixme!)
+                  (let ((ent (merge-pathnames ent pathname))) ;; This is for CMUCL, see above. (fixme!)
+                                                              ;; And breaks some things for SBCL.. (mgr) 
                     (pretty-pretty-pathname ent *standard-output* :long-name full-names)))))))))
 
 #+nil   ; OBSOLETE
