@@ -133,7 +133,8 @@
 ;; ICON-OF is measurably slow here in CMUCL. Interesting..
 
 (defmethod icon-of ((pathname pathname))
-  (cond ((not (probe-file pathname)) (standard-icon "invalid.xpm"))
+  (cond ((wild-pathname-p pathname) (standard-icon "wild.xpm"))
+        ((not (probe-file pathname)) (standard-icon "invalid.xpm"))
         ((directoryp pathname) *folder-icon*) ;; FIXME: use inode mime types                              
         (T (let ((mime-class (find-class (pathname-mime-type pathname) nil)))
              (if mime-class
@@ -180,7 +181,8 @@
   (:icon (standard-icon "design.xpm")))
 
 (define-mime-type (application x-lisp-fasl)
-  (:extensions "x86f" "fasl" "ibin" "dfsl" "ufsl") ; MORE!
+  (:extensions "x86f" "amd64f" "sparcf" "sparc64f" "hpf" "hp64f" "lbytef"
+	       "fasl" "ibin" "dfsl" "ufsl") ; MORE!
   (:icon (standard-icon "object.xpm")))
 
 (define-mime-type (text x-shellscript)
