@@ -34,15 +34,9 @@
 ;;; CLX-MEDIUM class
 
 (defclass clx-medium (basic-medium)
-  ((gc
-      :initform nil)
-   (picture
-    :initform nil)
-   #+unicode
-   (fontset
-      :initform nil
-      :accessor medium-fontset)
-   ))
+  ((gc :initform nil)
+   (picture :initform nil)
+   (buffer :initform nil :accessor medium-buffer)))
 
 #+CLX-EXT-RENDER
 (defun clx-medium-picture (clx-medium)
@@ -54,7 +48,6 @@
 
 ;;; secondary methods for changing text styles and line styles
 
-#-unicode
 (defmethod (setf medium-text-style) :before (text-style (medium clx-medium))
   (with-slots (gc) medium
     (when gc
@@ -63,13 +56,6 @@
 	  (setf (xlib:gcontext-font gc)
 		(text-style-to-X-font (port medium) (medium-text-style medium))))))))
 
-#+unicode
-(defmethod (setf medium-text-style) :before (text-style (medium clx-medium))
-  (with-slots (fontset) medium
-    (let ((old-text-style (medium-text-style medium)))
-      (unless (eq text-style old-text-style)
-        (setf fontset (text-style-to-X-fontset (port medium) (medium-text-style medium)))))))
-
 ;;; Translate from CLIM styles to CLX styles.
 (defconstant +cap-shape-map+ '((:butt . :butt)
 			       (:square . :projecting)
@@ -164,10 +150,7 @@
 		  (xlib:gcontext-dashes gc) (if (eq dashes t) 3
 						dashes)))))
       (setf (xlib:gcontext-function gc) boole-1)
-      #-unicode
       (setf (xlib:gcontext-font gc) (text-style-to-X-font port (medium-text-style medium)))
-      #+unicode
-      (setf (medium-fontset medium) (text-style-to-X-fontset port (medium-text-style medium)))
       (setf (xlib:gcontext-foreground gc) (X-pixel port ink)
 	    (xlib:gcontext-background gc) (X-pixel port (medium-background medium)))
       ;; Here is a bug with regard to clipping ... ;-( --GB )
@@ -338,19 +321,15 @@
 
 (defmacro with-clx-graphics ((medium) &body body)
   `(let* ((port (port ,medium))
-          (mirror (port-lookup-mirror port (medium-sheet ,medium))))
+	  (mirror (or (medium-buffer medium) (port-lookup-mirror port (medium-sheet ,medium)))))
     (when mirror
       (let* ((line-style (medium-line-style ,medium))
-             (ink        (medium-ink ,medium))
-             (gc         (medium-gcontext ,medium ink))
-             #+unicode
-             (*fontset*  (or (medium-fontset ,medium)
-                             (setf (medium-fontset ,medium)
-                                   (text-style-to-X-fontset (port ,medium) *default-text-style*)))))
-        line-style ink
-        (unwind-protect
-             (progn ,@body)
-          #+ignore(xlib:free-gcontext gc))))))
+	     (ink        (medium-ink ,medium))
+	     (gc         (medium-gcontext ,medium ink)))
+	line-style ink
+	(unwind-protect
+	     (progn ,@body)
+	  #+ignore(xlib:free-gcontext gc))))))
 
 
 ;;; Pixmaps
@@ -363,12 +342,14 @@
                               from-x from-y)
     (with-transformed-position ((sheet-native-transformation (medium-sheet to-drawable))
                                 to-x to-y)
-      (xlib:copy-area (sheet-direct-mirror (medium-sheet from-drawable))
-                      (medium-gcontext from-drawable +background-ink+)
-                      (round-coordinate from-x) (round-coordinate from-y)
-		      (round width) (round height)
-                      (sheet-direct-mirror (medium-sheet to-drawable))
-                      (round-coordinate to-x) (round-coordinate to-y)))))
+      (multiple-value-bind (width height) (transform-distance (medium-transformation from-drawable)
+                                                              width height)
+        (xlib:copy-area (sheet-direct-mirror (medium-sheet from-drawable))
+                        (medium-gcontext from-drawable +background-ink+)
+                        (round-coordinate from-x) (round-coordinate from-y)
+                        (round width) (round height)
+                        (or (medium-buffer to-drawable) (sheet-direct-mirror (medium-sheet to-drawable)))
+                        (round-coordinate to-x) (round-coordinate to-y))))))
 
 (defmethod medium-copy-area ((from-drawable clx-medium) from-x from-y width height
                              (to-drawable pixmap) to-x to-y)
@@ -389,7 +370,7 @@
                     (medium-gcontext to-drawable +background-ink+)
                     (round-coordinate from-x) (round-coordinate from-y)
 		    (round width) (round height)
-                    (sheet-direct-mirror (medium-sheet to-drawable))
+                    (or (medium-buffer to-drawable) (sheet-direct-mirror (medium-sheet to-drawable)))
                     (round-coordinate to-x) (round-coordinate to-y))))
 
 (defmethod medium-copy-area ((from-drawable pixmap) from-x from-y width height
@@ -583,7 +564,7 @@
   (with-transformed-position ((sheet-native-transformation (medium-sheet medium))
                               center-x center-y)
     (let* ((arc-angle (- end-angle start-angle))
-           (arc-angle (if (< end-angle 0)
+           (arc-angle (if (< arc-angle 0)
                           (+ (* pi 2) arc-angle)
                           arc-angle)))
       (with-clx-graphics (medium)
@@ -607,7 +588,7 @@
 							    medium))
                               center-x center-y)
     (let* ((arc-angle (- end-angle start-angle))
-           (arc-angle (if (< end-angle 0)
+           (arc-angle (if (< arc-angle 0)
                           (+ (* pi 2) arc-angle)
                           arc-angle))
 	   (min-x (round-coordinate (- center-x radius)))
@@ -626,170 +607,97 @@
 ;;;
 ;;; Methods for text styles
 
-#-unicode
 (defmethod text-style-ascent (text-style (medium clx-medium))
   (let ((font (text-style-to-X-font (port medium) text-style)))
     (xlib:font-ascent font)))
 
-#+unicode
-(defmethod text-style-ascent (text-style (medium clx-medium))
-  (let ((fontset (text-style-to-X-fontset (port medium) text-style)))
-    (fontset-ascent fontset)))
-
-#-unicode
 (defmethod text-style-descent (text-style (medium clx-medium))
   (let ((font (text-style-to-X-font (port medium) text-style)))
     (xlib:font-descent font)))
 
-#+unicode
-(defmethod text-style-descent (text-style (medium clx-medium))
-  (let ((fontset (text-style-to-X-fontset (port medium) text-style)))
-    (fontset-descent fontset)))
-
-#-unicode
 (defmethod text-style-height (text-style (medium clx-medium))
   (let ((font (text-style-to-X-font (port medium) text-style)))
     (+ (xlib:font-ascent font) (xlib:font-descent font))))
 
-#+unicode
-(defmethod text-style-height (text-style (medium clx-medium))
-  (let ((fontset (text-style-to-X-fontset (port medium) text-style)))
-    (fontset-height fontset)))
-
-#-unicode
 (defmethod text-style-character-width (text-style (medium clx-medium) char)
   (xlib:char-width (text-style-to-X-font (port medium) text-style) (char-code char)))
 
-#+unicode
-(defmethod text-style-character-width (text-style (medium clx-medium) char)
-  (fontset-point-width (char-code char) (text-style-to-X-fontset (port medium) text-style)))
-
 (defmethod text-style-width (text-style (medium clx-medium))
   (text-style-character-width text-style medium #\m))
 
-#-unicode
+(eval-when (:compile-toplevel :execute)
+  ;; ASCII / CHAR-CODE compatibility checking
+  (unless (equal (mapcar #'char-code '(#\Backspace #\Tab #\Linefeed
+                                       #\Page #\Return #\Rubout))
+                 '(8 9 10 12 13 127))
+    (error "~S not ASCII-compatible for semi-standard characters: ~
+           implement a CLX translate function for this implementation."
+           'code-char))
+  (let ((standard-chars " !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~"))
+    (dotimes (i 95)
+      (unless (eql (char standard-chars i) (code-char (+ i 32)))
+        (error "~S not ASCII-compatible for standard character ~S: ~
+                implement a CLX translate function for this implementation."
+               'code-char (code-char (+ i 32)))))))
+
+;;; The default CLX translation function is defined to work only for
+;;; ASCII characters; quoting from the documentation,
+;;;
+;;;   The default :translate function handles all characters that
+;;;   satisfy graphic-char-p by converting each character into its
+;;;   ASCII code.
+;;;
+;;; We provide our own translation function which is essentially the
+;;; same as that of CLX, but with the ASCII restriction relaxed.  This
+;;; is by no means a proper solution to the problem of
+;;; internationalization, because fonts tend not to have a complete
+;;; coverage of the entirety of the Unicode space, even assuming that
+;;; the underlying lisp supports it (as of 2006-02-06, only the case
+;;; for SBCL and CLISP); instead, the translation function is meant to
+;;; handle font sets by requesting the X server change fonts in the
+;;; middle of rendering strings.  However, the below stands a chance
+;;; of working when using ISO-8859-1-encoded fonts, and will tend to
+;;; lose in other cases.
 (defun translate (src src-start src-end afont dst dst-start)
-  ;; This is for replacing the clx-translate-default-function
-  ;; who does'nt know about accentated characters because
-  ;; of a call to cl:graphic-char-p that return nil with accentated characters.
-  ;; For further informations, on a clx-translate-function, see the clx-man.
   (declare (type sequence src)
 	   (type xlib:array-index src-start src-end dst-start)
 	   (type (or null xlib:font) afont)
 	   (type vector dst))
-  #+cmucl(declare (xlib::clx-values integer
-				    (or null integer xlib:font)
-				    (or null integer)))
+  ;; FIXME: what if AFONT is null?
   (let ((min-char-index (xlib:font-min-char afont))
 	(max-char-index (xlib:font-max-char afont)))
-    afont
     (if (stringp src)
 	(do ((i src-start (xlib::index+ i 1))
 	     (j dst-start (xlib::index+ j 1))
 	     (char))
 	    ((xlib::index>= i src-end)
 	     i)
-	    (declare (type xlib:array-index i j))
-	    (setq char (xlib:char->card8 (char src i)))
-	    (if (or (< char min-char-index) (> char max-char-index))
-		(return i)
-	        (setf (aref dst j) char)))
+          (declare (type xlib:array-index i j))
+          (setq char (char-code (char src i)))
+          (if (or (< char min-char-index) (> char max-char-index))
+              (progn
+                (warn "Character ~S not representable in font ~S" 
+                      (char src i) afont)
+                (return i))
+              (setf (aref dst j) char)))
         (do ((i src-start (xlib::index+ i 1))
 	     (j dst-start (xlib::index+ j 1))
 	     (elt))
 	    ((xlib::index>= i src-end)
 	     i)
-	    (declare (type xlib:array-index i j))
-	    (setq elt (elt src i))
-	    (when (characterp elt) (setq elt (xlib:char->card8 elt)))
-	    (if (or (not (integerp elt)) 
-		    (< elt min-char-index)
-		    (> elt max-char-index))
-		(return i)
-	        (setf (aref dst j) elt))))))
+          (declare (type xlib:array-index i j))
+          (setq elt (elt src i))
+          (when (characterp elt) 
+            (setq elt (char-code elt)))
+          (if (or (not (integerp elt)) 
+                  (< elt min-char-index)
+                  (> elt max-char-index))
+              (progn
+                (warn "Thing ~S not representable in font ~S"
+                      (elt src i) afont)
+                (return i))
+              (setf (aref dst j) elt))))))
 
-; Yes, the following is a nasty hack.
-; It's just a proof of concept, I'll try not to commit it :]
-; If it does get committed, it shouldn't affect anyone much...
-
-#+unicode
-(defun translate (source source-start source-end initial-font destination destination-start)
-  ; do the first character especially
-  (let* ((code   (char-code (char source source-start)))
-         (result (fontset-point code)))
-    (if result
-        (destructuring-bind ((range-start . range-stop) font translator) result
-          (if (not (eq font initial-font))
-            ; may need to change fonts immediately:
-            (values source-start font)
-            ; otherwise, lets finish the job...
-            (multiple-value-bind (result success) (funcall translator code)
-              (setf (elt destination destination-start) result)
-              (do ((src  (+ source-start 1)      (+ src 1))
-                   (dst  (+ destination-start 1) (+ dst 1)))
-                  ((>= src source-end)
-                   ; we finished
-                   (values src nil))
-                (let* ((code (char-code (char source src))))
-                  (if (<= range-start code range-stop)
-                      (multiple-value-bind (result success) (funcall translator code)
-                        (setf (elt destination dst) result))
-                      ; wasn't in the range... need to switch
-                      (let ((new (fontset-point code)))
-                        (if new
-                            (destructuring-bind ((range-start . range-stop) font translator) new
-                              (return (values src font)))
-                            (return (values src nil))))))))))
-        (values source-start nil))))
-
-#+unicode
-(in-package :external-format)
-
-#+unicode
-(defun ascii-code-to-font-index (code)
-  (values code (<= #x00 code #x7f)))
-
-#+unicode
-(defun ksc5601-code-to-font-index (wc)
-  (labels ((illegal-sequence ()
-             (error "ksc5601-wctomb"))
-           (summary-of (array index)
-             (values (aref array index 0)
-                     (aref array index 1))))
-
-    (multiple-value-bind (indx used)
-        (cond
-          ((<= #x0000 wc #x045f)
-           (summary-of ksc5601-uni2indx-page00 (ash wc -4)))
-          ((<= #x2000 wc #x266f)
-           (summary-of ksc5601-uni2indx-page20 (- (ash wc -4) #x200)))
-          ((<= #x3000 wc #x33df)
-           (summary-of ksc5601-uni2indx-page30 (- (ash wc -4) #x300)))
-          ((<= #x4e00 wc #x9f9f)
-           (summary-of ksc5601-uni2indx-page4e (- (ash wc -4) #x4e0)))
-          ((<= #xac00 wc #xd79f)
-           (summary-of ksc5601-uni2indx-pageac (- (ash wc -4) #xac0)))
-          ((<= #xf900 wc #xfa0f)
-           (summary-of ksc5601-uni2indx-pagef9 (- (ash wc -4) #xf90)))
-          ((<= #xff00 wc #xffef)
-           (summary-of ksc5601-uni2indx-pageff (- (ash wc -4) #xff0)))
-          (t
-           (illegal-sequence)))
-      (let ((i (logand wc #x0f)))
-        (if (/= 0 (logand used (ash 1 i)))
-            (let* ((used (logand used (- (ash 1 i) 1)))
-                   (used (+ (logand used #x5555) (ash (logand used #xaaaa) -1)))
-                   (used (+ (logand used #x3333) (ash (logand used #xcccc) -2)))
-                   (used (+ (logand used #x0f0f) (ash (logand used #xf0f0) -4)))
-                   (used (+ (logand used #x00ff) (ash used -8)))
-                   (c    (aref ksc5601-2charset (+ indx used))))
-              c)
-            (illegal-sequence))))))
-
-#+unicode
-(in-package :clim-clx)
-
-#-unicode
 (defmethod text-size ((medium clx-medium) string &key text-style (start 0) end)
   (when (characterp string)
     (setf string (make-string 1 :initial-element string)))
@@ -799,7 +707,7 @@
     (cond ((= start end)
            (values 0 0 0 0 0))
           (t
-           (let ((position-newline (position #\newline string :start start)))
+           (let ((position-newline (position #\newline string :start start :end end)))
              (cond ((not (null position-newline))
                     (multiple-value-bind (width ascent descent left right
                                                 font-ascent font-descent direction
@@ -827,48 +735,48 @@
                                           direction first-not-done))
                       (values width (+ ascent descent) width 0 ascent)) )))))) )
 
-#+unicode
-(defmethod text-size ((medium clx-medium) string &key text-style (start 0) end)
+(defmethod climi::text-bounding-rectangle*
+    ((medium clx-medium) string &key text-style (start 0) end)
   (when (characterp string)
     (setf string (make-string 1 :initial-element string)))
   (unless end (setf end (length string)))
   (unless text-style (setf text-style (medium-text-style medium)))
-  (let* ((xfontset     (text-style-to-X-fontset (port medium) text-style))
-         (default-font (fontset-default-font xfontset)))
+  (let ((xfont (text-style-to-X-font (port medium) text-style)))
     (cond ((= start end)
-           (values 0 0 0 0 0))
+           (values 0 0 0 0))
           (t
-           (let ((position-newline (position #\newline string :start start)))
+           (let ((position-newline (position #\newline string :start start :end end)))
              (cond ((not (null position-newline))
                     (multiple-value-bind (width ascent descent left right
                                                 font-ascent font-descent direction
                                                 first-not-done)
-                        (let ((*fontset* xfontset))
-                          (xlib:text-extents default-font string
-                                             :start start :end position-newline
-                                             :translate #'translate))
+                        (xlib:text-extents xfont string
+                                           :start start :end position-newline
+                                           :translate #'translate)
                       (declare (ignorable left right
                                           font-ascent font-descent
                                           direction first-not-done))
-                      (multiple-value-bind (w h x y baseline)
-                          (text-size medium string :text-style text-style
-                                     :start (1+ position-newline) :end end)
-                        (values (max w width) (+ ascent descent h)
-                                x (+ ascent descent y) (+ ascent descent baseline)))))
+                      (multiple-value-bind (minx miny maxx maxy)
+                          (text-bounding-rectangle*
+                           medium string :text-style text-style
+                           :start (1+ position-newline) :end end)
+                        (values (min minx left) (- ascent)
+                                (max maxx right) (+ descent maxy)))))
                    (t
                     (multiple-value-bind (width ascent descent left right
                                                 font-ascent font-descent direction
                                                 first-not-done)
-                        (let ((*fontset* xfontset))
-                          (xlib:text-extents default-font string
-                                     :start start :end end
-                                     :translate #'translate))
-                      (declare (ignorable left right
-                                          font-ascent font-descent
-                                          direction first-not-done))
-                      (values width (+ ascent descent) width 0 ascent)) )))))) )
+                        (xlib:text-extents xfont string
+                                   :start start :end end
+                                   :translate #'translate)
+                      (declare (ignore width direction first-not-done))
+                      ;; FIXME: Potential style points:
+                      ;; * (min 0 left), (max width right)
+                      ;; * font-ascent / ascent
+                      (values left (- font-ascent) right font-descent)))))))))
+
+
 
-#-unicode
 (defmethod medium-draw-text* ((medium clx-medium) string x y
                               start end
                               align-x align-y
@@ -902,41 +810,6 @@
                                 :start start :end end
                                 :translate #'translate)))))))
 
-#+unicode
-(defmethod medium-draw-text* ((medium clx-medium) string x y
-                              start end
-                              align-x align-y
-                              toward-x toward-y transform-glyphs)
-  (declare (ignore toward-x toward-y transform-glyphs))
-  (with-transformed-position ((sheet-native-transformation (medium-sheet medium))
-                              x y)
-    (with-clx-graphics (medium)
-      (when (characterp string)
-        (setq string (make-string 1 :initial-element string)))
-      (when (null end) (setq end (length string)))
-      (multiple-value-bind (text-width text-height x-cursor y-cursor baseline) 
-          (text-size medium string :start start :end end)
-        (declare (ignore x-cursor y-cursor))
-        (unless (and (eq align-x :left) (eq align-y :baseline))	    
-          (setq x (- x (ecase align-x
-                         (:left 0)
-                         (:center (round text-width 2))
-                         (:right text-width))))
-          (setq y (ecase align-y
-                    (:top (+ y baseline))
-                    (:center (+ y baseline (- (floor text-height 2))))
-                    (:baseline y)
-                    (:bottom (+ y baseline (- text-height)))))))
-      (let ((x (round-coordinate x))
-            (y (round-coordinate y)))
-        (when (and (<= #x-8000 x #x7FFF)
-                   (<= #x-8000 y #x7FFF))
-          (multiple-value-bind (halt width)
-              (xlib:draw-glyphs mirror gc x y string
-                                :start start :end end
-                                :size 16
-                                :translate #'translate)))))))
-
 (defmethod medium-buffering-output-p ((medium clx-medium))
   t)
 
@@ -972,13 +845,16 @@
 	      (min-y (round-coordinate (min top bottom)))
 	      (max-x (round-coordinate (max left right)))
 	      (max-y (round-coordinate (max top bottom))))
-	  (xlib:clear-area (port-lookup-mirror (port medium)
-					       (medium-sheet medium))
-			   :x (max #x-8000 (min #x7fff min-x))
-			   :y (max #x-8000 (min #x7fff min-y))
-			   :width (max 0 (min #xffff (- max-x min-x)))
-			   :height (max 0 (min #xffff (- max-y min-y)))))))))
-
+	  (xlib:draw-rectangle (or (medium-buffer medium)
+				   (port-lookup-mirror (port medium)
+						       (medium-sheet medium)))
+			       (medium-gcontext medium (medium-background medium))
+			       (max #x-8000 (min #x7fff min-x))
+			       (max #x-8000 (min #x7fff min-y))
+			       (max 0 (min #xffff (- max-x min-x)))
+			       (max 0 (min #xffff (- max-y min-y)))
+			       t))))))
+  
 (defmethod medium-beep ((medium clx-medium))
   (xlib:bell (clx-port-display (port medium))))
 
@@ -999,3 +875,18 @@
 
 (defmethod medium-miter-limit ((medium clx-medium))
   #.(* pi (/ 11 180)))
+
+(defmethod climi::medium-invoke-with-possible-double-buffering (frame pane (medium clx-medium) continuation)
+  (if (climi::pane-double-buffering pane)
+      (let* ((mirror (sheet-direct-mirror pane))
+	     (width (xlib:drawable-width mirror))
+	     (height (xlib:drawable-height mirror))
+	     (depth (xlib:drawable-depth mirror))
+	     (pixmap (xlib:create-pixmap :width width :height height :depth depth :drawable mirror)))
+	(setf (medium-buffer medium) pixmap)
+	(unwind-protect (funcall continuation)
+	  (xlib:copy-area pixmap (medium-gcontext medium (medium-foreground medium)) 0 0 width height mirror 0 0)
+	  (xlib:free-pixmap pixmap)
+	  (setf (medium-buffer medium) nil)))
+      (funcall continuation)))
+
