@@ -42,12 +42,9 @@
 
 ;;; CLX-PORT class
 
-(defclass clx-pointer (pointer)
+(defclass clx-pointer (standard-pointer)
   ((cursor :accessor pointer-cursor :initform :upper-left)))
 
-(defclass standard-pointer (clx-pointer)
-  ())
-
 #|
  Perhaps this belongs elsewhere
 
@@ -165,20 +162,38 @@
    (design-cache :initform (make-hash-table :test #'eq))
    (pointer :reader port-pointer)
    (pointer-grab-sheet :accessor pointer-grab-sheet :initform nil)
-   (selection-owner :initform nil :accessor selection-owner)))
+   (selection-owner :initform nil :accessor selection-owner)
+   (selection-timestamp :initform nil :accessor selection-timestamp)))
 
 (defun parse-clx-server-path (path)
   (pop path)
-  (let* ((s (get-environment-variable "DISPLAY"))
-	 (colon (position #\: s))
-	 (dot (position #\. s :start colon))
-	 (host-name (subseq s 0 colon))
-	 (display-number (parse-integer s :start (1+ colon) :end dot))
-	 (screen-number (if dot (parse-integer s :start (1+ dot)) 0)))
+  (let* ((name (get-environment-variable "DISPLAY"))
+	 ;; this code courtesy telent-clx.
+         (slash-i (or (position #\/ name) -1))
+         (colon-i (position #\: name :start (1+ slash-i)))
+         (decnet-colon-p (eql (elt name (1+ colon-i)) #\:))
+         (host (subseq name (1+ slash-i) colon-i))
+         (dot-i (and colon-i (position #\. name :start colon-i)))
+         (display (when colon-i
+                    (parse-integer name
+                                   :start (if decnet-colon-p
+                                              (+ colon-i 2)
+					      (1+ colon-i))
+                                   :end dot-i)))
+         (screen (when dot-i
+                   (parse-integer name :start (1+ dot-i))))
+         (protocol
+          (cond ((or (string= host "") (string-equal host "unix")) :local)
+                (decnet-colon-p :decnet)
+                ((> slash-i -1) (intern
+                                 (string-upcase (subseq name 0 slash-i))
+                                 :keyword))
+                (t :internet))))
     (list :clx
-	  :host (getf path :host host-name)
-	  :display-id (getf path :display-id display-number)
-	  :screen-id (getf path :screen-id screen-number))))
+	  :host (getf path :host host)
+	  :display-id (getf path :display-id (or display 0))
+	  :screen-id (getf path :screen-id (or screen 0))
+	  :protocol protocol)))
 
 (setf (get :x11 :port-type) 'clx-port)
 (setf (get :x11 :server-path-parser) 'parse-clx-server-path)
@@ -189,15 +204,17 @@
   (declare (ignore args))
   (push (make-instance 'clx-frame-manager :port port) (slot-value port 'frame-managers))
   (setf (slot-value port 'pointer)
-	(make-instance 'standard-pointer :port port))
+	(make-instance 'clx-pointer :port port))
   (initialize-clx port))
 
 (defmethod print-object ((object clx-port) stream)
   (print-unreadable-object (object stream :identity t :type t)
     (when (slot-boundp object 'display)
-      (format stream "~S ~S ~S ~S"
-              :host (xlib:display-host (slot-value object 'display))
-              :display-id (xlib:display-display (slot-value object 'display))))))
+      (let ((display (slot-value object 'display)))
+	(when display
+	  (format stream "~S ~S ~S ~S"
+		  :host (xlib:display-host display)
+		  :display-id (xlib:display-display display)))))))
 
 (defun clx-error-handler (display error-name &rest args &key major &allow-other-keys)
   (unless (and (eql major 42)  ; 42 is SetInputFocus, we ignore match-errors from that
@@ -250,7 +267,9 @@
 (defmethod initialize-clx ((port clx-port))
   (let ((options (cdr (port-server-path port))))
     (setf (clx-port-display port)
-	  (xlib:open-display (getf options :host "") :display (getf options :display-id 0)))
+	  (xlib:open-display (getf options :host) 
+			     :display (getf options :display-id) 
+			     :protocol (getf options :protocol)))
     (progn
       (setf (xlib:display-error-handler (clx-port-display port))
         #'clx-error-handler)
@@ -259,7 +278,7 @@
       (setf (xlib:display-after-function (clx-port-display port)) #'xlib:display-force-output))
       
     
-    (setf (clx-port-screen port) (nth (getf options :screen-id 0)
+    (setf (clx-port-screen port) (nth (getf options :screen-id)
 				      (xlib:display-roots (clx-port-display port))))
     (setf (clx-port-window port) (xlib:screen-root (clx-port-screen port)))
     (make-cursor-table port)    
@@ -328,8 +347,8 @@
 					      :button-press :button-release
 					      :enter-window :leave-window
 					      :structure-notify
-					     ;:pointer-motion
-                                              :button-motion)))
+					      :pointer-motion
+					      :button-motion)))
   (when (null (port-lookup-mirror port sheet))
     (update-mirror-geometry sheet)
     (let* ((desired-color (typecase sheet
@@ -450,6 +469,18 @@
     (xlib:destroy-window (port-lookup-mirror port sheet))
     (port-unregister-mirror port sheet (sheet-mirror sheet))))
 
+(defmethod raise-mirror ((port clx-port) (sheet basic-sheet))
+  (let ((mirror (sheet-mirror sheet)))
+    (when (and mirror
+	       (typep mirror 'xlib:window))
+      (xlib:circulate-window-up mirror))))
+
+(defmethod bury-mirror ((port clx-port) (sheet basic-sheet))
+  (let ((mirror (sheet-mirror sheet)))
+    (when (and mirror
+	       (typep mirror 'xlib:window))
+      (xlib:circulate-window-down mirror))))
+
 (defmethod mirror-transformation ((port clx-port) mirror)
   (make-translation-transformation (xlib:drawable-x mirror)
                                    (xlib:drawable-y mirror)))
@@ -519,7 +550,10 @@
   (xlib:unmap-window (sheet-direct-mirror mirror)) )
 
 (defmethod destroy-port :before ((port clx-port))
-  (xlib:close-display (clx-port-display port)))
+  (handler-case
+      (xlib:close-display (clx-port-display port))
+    (stream-error ()
+      (xlib:close-display (clx-port-display port) :abort t))))
 
 (defmethod port-motion-hints ((port clx-port) (sheet mirrored-sheet-mixin))
   (let ((event-mask (xlib:window-event-mask (sheet-direct-mirror sheet))))
@@ -554,11 +588,12 @@
                                   +pointer-middle-button+
                                   +pointer-right-button+
                                   +pointer-wheel-up+
-                                  +pointer-wheel-down+)))
-    (if (and (> code 0)
-             (<= code (1+ (length button-mapping))))
-        (aref button-mapping (1- code))
-        nil)))
+                                  +pointer-wheel-down+
+                                  +pointer-wheel-left+
+                                  +pointer-wheel-right+)))
+    (and (> code 0)
+         (<= code (1+ (length button-mapping)))
+         (aref button-mapping (1- code)))))
 
 ;; From "Inter-Client Communication Conventions Manual", Version 2.0.xf86.1,
 ;; section 4.1.5:
@@ -623,13 +658,11 @@
 ;; pointer-event-buttons slot to pointer events. -- moore
 ;; 
 
-(defun event-handler (&rest event-slots
-                      &key display window event-key code state mode time
+(defun event-handler (&key display window event-key code state mode time
 		      type width height x y root-x root-y
 		      data override-redirect-p send-event-p hint-p
                       target property requestor selection
                       &allow-other-keys)
-  (declare (ignorable event-slots))
   (declare (special *clx-port*))
   (let ((sheet (and window
 		    (port-lookup-sheet *clx-port* window))))
@@ -749,7 +782,7 @@
 				:modifier-state modifier-state
 				:timestamp time)))))
         ;;
-	((:exposure :display)
+	((:exposure :display :graphics-exposure)
          ;; Notes:
          ;; . Do not compare count with 0 here, last rectangle in an
          ;;   :exposure event sequence does not cover the whole region. 
@@ -918,7 +951,6 @@
 
 (defvar *fontset* nil)
 
-#-unicode
 (defmethod text-style-mapping ((port clx-port) text-style
                                &optional character-set)
   (declare (ignore character-set))
@@ -953,96 +985,6 @@
                           (open-font (clx-port-display port) font-name)))
               font-name))))))
 
-#+unicode
-(defun build-english-font-name (text-style)
-  (multiple-value-bind (family face size language)
-      (text-style-components text-style)
-    (destructuring-bind (family-name face-table)
-         (if (stringp family)
-             (list family *clx-text-faces*)
-             (or (getf *clx-text-family+face-map* family)
-                 (getf *clx-text-family+face-map* :fix)))
-       (let* ((face-name (if (stringp face)
-                             face
-                             (or (getf face-table
-                                       (if (listp face)
-                                           (intern (format nil "~A-~A"
-                                                           (symbol-name (first face))
-                                                           (symbol-name (second face)))
-                                                   :keyword)
-                                           face))
-                                 (getf *clx-text-faces* :roman))))
-              (size-number (if (numberp size)
-                               (round size)
-                               (or (getf *clx-text-sizes* size)
-                                   (getf *clx-text-sizes* :normal))))
-              (font-name (format nil "-~A-~A-*-*-~D-*-*-*-*-*-*-*"
-                                 family-name face-name size-number)))
-          font-name))))
-
-#+unicode
-(defun build-korean-font-name (text-style)
-  (multiple-value-bind (family face size language)
-      (text-style-components text-style)
-    (let* ((face (if (equal face '(:bold :italic)) :bold-italic face))
-           (font (case family
-                   ((:fix nil)
-                    (case face
-                      ((:roman nil)          "baekmuk-dotum-medium-r")
-                      ((:bold)               "baekmuk-dotum-bold-r")
-                      ((:italic)             "baekmuk-dotum-medium-r")
-                      ((:bold-italic)        "baekmuk-dotum-bold-r")))
-                   ((:serif)
-                    (case face
-                      ((:roman nil)          "baekmuk-batang-medium-r")
-                      ((:bold)               "baekmuk-batang-bold-r")
-                      ((:italic)             "baekmuk-batang-medium-r")
-                      ((:bold-italic)        "baekmuk-batang-bold-r")))
-                   ((:sans-serif)
-                    (case face
-                      ((:roman nil)          "baekmuk-gulim-medium-r")
-                      ((:bold)               "baekmuk-gulim-bold-r")
-                      ((:italic)             "baekmuk-gulim-medium-r")
-                      ((:bold-italic)        "baekmuk-gulim-bold-r")))))
-            (size-number (if (numberp size)
-                             (round size)
-                             (or (getf *clx-text-sizes* size)
-                                 (getf *clx-text-sizes* :normal)))))
-      (format nil "-~A-*-*-~D-*-*-*-*-*-ksx1001.1997-*" font size-number))))
-
-; this needs much refactoring... FIXME
-#+unicode
-(defmethod text-style-mapping ((port clx-port) text-style
-                               &optional character-set)
-  (declare (ignore character-set))
-
-  (let ((table (port-text-style-mappings port)))
-    (or (car (gethash text-style table))
-        (multiple-value-bind (family face size language)
-            (text-style-components text-style)
-          (let* ((display (clx-port-display port))
-                 (fontset (case language
-                            ((nil :english)
-                             (let* ((font-name (build-english-font-name text-style))
-                                    (font      (xlib:open-font display  font-name)))
-                               (make-fontset font-name
-                                 (0 255 font #'external-format::ascii-code-to-font-index))))
-                            ((:korean)
-                             (let* ((english-font-name (build-english-font-name text-style))
-                                    (english-font      (xlib:open-font display  english-font-name))
-                                    (korean-font-name  (build-korean-font-name  text-style))
-                                    (korean-font       (xlib:open-font display  korean-font-name)))
-                               (make-fontset korean-font-name
-                                 (0      255    english-font
-                                                #'external-format::ascii-code-to-font-index)
-                                 (#xAC00 #xD7A3 korean-font
-                                                #'external-format::ksc5601-code-to-font-index)
-                                 (#x4E00 #x9FA5 korean-font
-                                                #'external-format::ksc5601-code-to-font-index)))))))
-            (setf (gethash text-style table)
-                  (cons (fontset-name fontset) fontset))
-            (fontset-name fontset))))))
-
 (defmethod (setf text-style-mapping) (font-name (port clx-port)
                                       (text-style text-style)
                                       &optional character-set)
@@ -1051,38 +993,20 @@
         (cons font-name (open-font (clx-port-display port) font-name)))
   font-name)
 
-#-unicode
 (defun text-style-to-X-font (port text-style)
   (let ((text-style (parse-text-style text-style)))
     (text-style-mapping port text-style)
     (cdr (gethash text-style (port-text-style-mappings port)))))
 
-#+unicode
-(defun text-style-to-X-fontset (port text-style)
-  (let ((text-style (parse-text-style text-style)))
-    (text-style-mapping port text-style)
-    (cdr (gethash text-style (port-text-style-mappings port)))))
-
-#-unicode
 (defmethod port-character-width ((port clx-port) text-style char)
   (let* ((font (text-style-to-X-font port text-style))
 	 (width (xlib:char-width font (char-code char))))
     width))
 
-#+unicode
-(defmethod port-character-width ((port clx-port) text-style char)
-  (fontset-point-width (char-code char) (text-style-to-X-fontset port text-style)))
-
-#-unicode
 (defmethod port-string-width ((port clx-port) text-style string &key (start 0) end)
   (xlib:text-width (text-style-to-X-font port text-style)
 		   string :start start :end end))
 
-#+unicode ; this requires a translator and so on.
-(defmethod port-string-width ((port clx-port) text-style string &key (start 0) end)
-  (let ((*fontset* (text-style-to-X-fontset port text-style)))
-    (xlib:text-width nil string :start start :end end :translator #'translate)))
-
 (defmethod X-pixel ((port clx-port) color)
   (let ((table (slot-value port 'color-table)))
     (or (gethash color table)
@@ -1314,19 +1238,23 @@
 
 ;; we at least want to support:
 
-;; :TEXT, :STRING
-;;
-;; :UTF8_STRING
-;;     As seen from xterm [make that the prefered encoding]
-;;
-;; :COMPOUND_TEXT
-;;    Perhaps relatively easy to produce, hard to grok.
-;;
-
+;;; :TEXT, :STRING
+;;;
+;;; :UTF8_STRING
+;;;    As seen from xterm [make that the preferred encoding]
+;;;
+;;; :COMPOUND_TEXT
+;;;    Perhaps relatively easy to produce, hard to grok.
+;;;
+;;; :TARGETS
+;;;    Clients want legitimately to find out what we support.
+;;;
+;;; :TIMESTAMP
+;;;    Clients want to know when we took ownership of the selection.
 
 ;;; Utilities
 
-(defun utf-8-encode (code-points)
+(defun utf8-string-encode (code-points)
   (let ((res (make-array (length code-points)
                          :adjustable t
                          :fill-pointer 0)))
@@ -1379,7 +1307,8 @@
   (xlib:set-selection-owner
    (clx-port-display port)
    :primary nil time)
-  (setf (selection-owner port) nil))
+  (setf (selection-owner port) nil)
+  (setf (selection-timestamp port) nil))
 
 (defmethod request-selection ((port clx-port) requestor time)
   (xlib:convert-selection :primary :STRING requestor :bounce time))
@@ -1399,50 +1328,108 @@
 
 ;; Incredibly crappy broken unportable Latin 1 encoder which should be
 ;; replaced by various implementation-specific versions.
-(defun latin1-encode (string)
-  (delete-if (lambda (x) (or (< x 0)
-                             (> x 255)))
-             (map 'vector #'char-code string)))
+(flet ((latin1-code-p (x)
+	 (not (or (< x 9) (< 10 x 32) (< #x7f x #xa0) (> x 255)))))
+  (defun string-encode (string)
+    (delete-if-not #'latin1-code-p (map 'vector #'char-code string)))
+  (defun exactly-encodable-as-string-p (string)
+    (every #'latin1-code-p (map 'vector #'char-code string))))
 
-;; TODO: INCR property?
+;;; TODO: INCR property?
+;;;
+;;; FIXME: per ICCCM we MUST support :MULTIPLE
 (defmethod send-selection ((port clx-port) (event clx-selection-request-event) string)
   (let ((requestor (selection-event-requestor event))
         (property  (selection-event-property event))
         (target    (selection-event-target event))
         (time      (event-timestamp event)))
     (when (null property)
-      (format *trace-output* "~&* Requestor property is null! *~%"))    
-    (describe event *trace-output*)
-    (finish-output *trace-output*)
+      (format *trace-output* "~&* Requestor property is null! *~%"))
+    #+nil ; debugging output
+    (progn
+      (describe event *trace-output*)
+      (finish-output *trace-output*))
     (flet ((send-event (&key target (property property))
+	     ;; debugging output, but the KDE Klipper client turns out
+	     ;; to poll other clients for selection, which means it
+	     ;; would be bad to print at every request.
+             #+nil
              (format *trace-output*
-                     "~&;; clim-clx::send-selection - Requested target ~A, sent ~A to property ~A.~%"
+                     "~&;; clim-clx::send-selection - Requested target ~A, sent ~A to property ~A. time ~S~%"
                      (selection-event-target event)
                      target
-                     property)                     
+                     property time)
              (xlib:send-event requestor
-                            :selection-notify nil
-                            :window requestor
-                            :selection :primary
-                            :target target
-                            :property property
-                            :time time)))
-      (cond ((member target '(:UTF8_STRING :TEXT))
-             (xlib:change-property requestor property
-                                   (utf-8-encode
-                                    (concatenate 'vector (map 'vector #'char-code string)))
-                                   :UTF8_STRING
-                                   8)
-             (send-event :target :UTF8_STRING))
-            ((member target '(:STRING :COMPOUND_TEXT))
-             (xlib:change-property requestor property                                   
-                                   (latin1-encode string)
-                                   :COMPOUND_TEXT
-                                   8)            
-             (send-event :target :COMPOUND_TEXT))
-            (t
-             (format *trace-output*
-                     "~&;; Warning, unhandled type \"~A\". Trying to send as UTF8_STRING.~%"
-                     target)
-             (send-event :target :UTF8_STRING :property nil)))) ;; ...
+			      :selection-notify nil
+			      :window requestor
+			      :event-window requestor
+			      :selection (climi::selection-event-selection event)
+			      :target target
+			      :property property
+			      :time time)))
+      (case target
+	((:UTF8_STRING)
+	 (xlib:change-property requestor property
+			       (utf8-string-encode
+				(map 'vector #'char-code string))
+			       :UTF8_STRING 8)
+	 (send-event :target :UTF8_STRING))
+	((:STRING :COMPOUND_TEXT)
+	 (xlib:change-property requestor property
+			       (string-encode string)
+			       target 8)            
+	 (send-event :target target))
+	((:TEXT)
+	 (cond
+	   ((exactly-encodable-as-string-p string)
+	    (xlib:change-property requestor property
+				  (string-encode string)
+				  :STRING 8)
+	    (send-event :target :STRING))
+	   (t 
+	    (xlib:change-property requestor property
+				  (utf8-string-encode
+				   (map 'vector #'char-code string))
+				  :UTF8_STRING 8)
+	    (send-event :target :UTF8_STRING))))
+	((:TARGETS)
+	 (let* ((display (clx-port-display port))
+		(targets (mapcar (lambda (x) (xlib:intern-atom display x))
+				 '(:TARGETS :STRING :TEXT :UTF8_STRING
+				   :COMPOUND_TEXT :TIMESTAMP))))
+	   (xlib:change-property requestor property targets target 32))
+	 (send-event :target :TARGETS))
+	((:TIMESTAMP)
+	 (when (null (selection-timestamp port))
+	   (format *trace-output* "~&;; selection-timestamp is null!~%"))
+	 (xlib:change-property requestor property
+			       (list (selection-timestamp port))
+			       target 32)
+	 (send-event :target :TIMESTAMP))
+	(t
+	 (format *trace-output*
+		 "~&;; Warning, unhandled type \"~A\". ~
+                  Sending property NIL to target.~%" target)
+	 (send-event :target target :property nil))))
     (xlib:display-force-output (xlib:window-display requestor))))
+
+;;; XXX CLX in ACL doesn't use local sockets, so here's a fix. This is gross
+;;; and should obviously be included in Franz' clx and portable clx, but I
+;;; believe that enough users will find that their X servers don't listen for
+;;; TCP connections that it is worthwhile to include this code here
+;;; temporarily.
+
+#+allegro
+(defun xlib::open-x-stream (host display protocol)
+  (declare (ignore protocol)) ;; Derive from host
+  (let ((stream (if (or (string= host "") (string= host "unix"))
+		    (socket:make-socket
+		     :address-family :file
+		     :remote-filename (format nil "/tmp/.X11-unix/X~D" display)
+		     :format :binary)
+		    (socket:make-socket :remote-host (string host)
+					:remote-port (+ *x-tcp-port* display)
+					:format :binary))))
+    (if (streamp stream)
+	stream
+      (error "Cannot connect to server: ~A:~D" host display))))
