@@ -68,7 +68,7 @@
 
 (defvar *extra-entries* 0)
 
-(defun write-postcript-dictionary (stream)
+(defun write-postscript-dictionary (stream)
   ;;; FIXME: DSC
   (format stream "~&%%BeginProlog~%")
   (format stream "/~A ~D dict def ~2:*~A begin~%"
@@ -147,8 +147,15 @@ setmatrix")
            (cy (point-y center))
            (tr (make-transformation ndx2 ndx1 ndy2 ndy1 cx cy))
            (circle (untransform-region tr ellipse))
-           (start-angle (or (ellipse-start-angle circle) 0))
-           (end-angle (or (ellipse-end-angle circle) (* 2 pi))))
+           ;; we need an extra minus sign because the rotation
+           ;; convention for Postscript differs in chirality from the
+           ;; abstract CLIM convention; we do a reflection
+           ;; transformation to move the coordinates to the right
+           ;; handedness, but then the sense of positive rotation is
+           ;; backwards, so we need this reflection for angles.  --
+           ;; CSR, 2005-08-01
+           (start-angle (- (or (ellipse-end-angle circle) 0)))
+           (end-angle (- (or (ellipse-start-angle circle) (* -2 pi)))))
       (write-string (if filled "true " "false ") stream)
       (write-angle stream (if (< end-angle start-angle)
                               (+ end-angle (* 2 pi))
@@ -162,23 +169,25 @@ setmatrix")
   "Native transformation")
 
 ;;; Postscript output utilities
-(defmacro with-graphics-state ((medium) &body body)
-  `(invoke-with-graphics-state ,medium
+(defmacro with-graphics-state ((stream) &body body)
+  `(invoke-with-graphics-state ,stream
     (lambda () ,@body)))
 
-(defun postscript-save-graphics-state (medium)
-  (push (copy-list (postscript-medium-graphics-state medium))
-        (slot-value (medium-sheet medium) 'graphics-state-stack))
-  (format (postscript-medium-file-stream medium) "gsave~%"))
+(defun postscript-save-graphics-state (stream)
+  (push (copy-list (first (slot-value stream 'graphics-state-stack)))
+        (slot-value stream 'graphics-state-stack))
+  (when (stream-drawing-p stream)
+    (format (postscript-stream-file-stream stream) "gsave~%")))
 
-(defun postscript-restore-graphics-state (medium)
-  (pop (slot-value (medium-sheet medium) 'graphics-state-stack))
-  (format (postscript-medium-file-stream medium) "grestore~%"))
+(defun postscript-restore-graphics-state (stream)
+  (pop (slot-value stream 'graphics-state-stack))
+  (when (stream-drawing-p stream)
+    (format (postscript-stream-file-stream stream) "grestore~%")))
 
-(defun invoke-with-graphics-state (medium continuation)
-  (postscript-save-graphics-state medium)
+(defun invoke-with-graphics-state (stream continuation)
+  (postscript-save-graphics-state stream)
   (funcall continuation)
-  (postscript-restore-graphics-state medium))
+  (postscript-restore-graphics-state stream))
 
 
 ;;; Postscript path functions
@@ -339,8 +348,8 @@ setmatrix")
   ;; does only one level of saving graphics state, so we can restore
   ;; and save again GS to obtain an initial CP. It is ugly, but I see
   ;; no other way now. -- APD, 2002-02-11
-  (postscript-restore-graphics-state medium)
-  (postscript-save-graphics-state medium)
+  (postscript-restore-graphics-state (medium-sheet medium))
+  (postscript-save-graphics-state (medium-sheet medium))
   (postscript-set-clipping-region stream
                                   (medium-clipping-region medium)))
 
@@ -453,11 +462,14 @@ setmatrix")
 
 (defmethod postscript-set-graphics-state (stream medium
                                           (kind (eql :text-style)))
-  (destructuring-bind (font . size)
-      (medium-font medium)
+  (let* ((font-name (medium-font medium))
+         (font (%font-name-postscript-name font-name))
+         (size (%font-name-size font-name)))
     (pushnew font (slot-value (medium-sheet medium) 'document-fonts)
              :test #'string=)
-    (format stream "/~A-iso findfont ~D scalefont setfont~%" font size))) ;### evil hack.
+    (format stream "/~A findfont ~D scalefont setfont~%"
+	    font
+	    size))) ;### evil hack.
 
 (defun postscript-escape-char (char)
   (case char
@@ -487,7 +499,7 @@ setmatrix")
   (let ((*transformation* (sheet-native-transformation (medium-sheet medium))))
     (let ((file-stream (postscript-medium-file-stream medium)))
       (postscript-actualize-graphics-state file-stream medium :color :text-style)
-      (with-graphics-state (medium)
+      (with-graphics-state ((medium-sheet medium))
         #+ignore
         (when transform-glyphs
           ;;
@@ -513,7 +525,9 @@ setmatrix")
                     (format-postscript-number ty))))
         (multiple-value-bind (total-width total-height
                               final-x final-y baseline)
-            (destructuring-bind (font . size) (medium-font medium)
+            (let* ((font-name (medium-font medium))
+                   (font (%font-name-metrics-key font-name))
+                   (size (%font-name-size font-name)))
               (text-size-in-font font size string 0 nil))
           (declare (ignore final-x final-y))
           ;; Only one line?
