@@ -24,6 +24,8 @@
 
 (in-package :MCCLIM-FREETYPE)
 
+(declaim (optimize (speed 3) (safety 3) (debug 1) (space 3)))
+
 ;;;; Notes
 
 ;; You might need to tweak mcclim-freetype::*families/faces* to point
@@ -33,71 +35,77 @@
   ((lib :initarg :lib)
    (filename :initarg :filename)))
 
-(defclass concrete-font (climi::device-font-text-style)
-  ((lib :initarg :lib)
-   (face :initarg :face)
-   (im-cache :initform (make-hash-table :test #'equal))
-   (ascent :initform nil)
-   (descent :initform nil)
-   ))
+(defparameter *vague-font-hash* (make-hash-table :test #'equal))
 
 (defun make-vague-font (filename)
-  (make-instance 'vague-font
-                 :lib (let ((libf (alien:make-alien freetype:library)))
-                       (freetype:init-free-type libf)
-                       (alien:deref libf))
-                 :filename filename))
+  (let ((val (gethash filename *vague-font-hash*)))
+    (or val
+        (setf (gethash filename *vague-font-hash*)
+              (make-instance 'vague-font
+                             :lib (let ((libf (make-alien freetype:library)))
+                                    (declare (type (alien (* freetype:library)) libf))
+                                    (freetype:init-free-type libf)
+                                    (deref libf))
+                             :filename filename)))))
 
 (defparameter *dpi* 72)
 
-(defmethod make-concrete-font ((vague-font vague-font) size &key (dpi *dpi*))
-  (let (face)
-    (with-slots (lib filename) vague-font
-      (setf face
-            (let ((facef (alien:make-alien freetype:face)))
-             (freetype:new-face lib filename 0 facef)
-             (alien:deref facef)))
-      (freetype:set-char-size face 0 (round (* size 64)) (round dpi) (round dpi))
-      (make-instance 'concrete-font :lib lib :face face))))
+(defparameter *concrete-font-hash* (make-hash-table :test #'equal))
 
-(defmethod glyph-pixarray ((font concrete-font) char)
-  (declare (optimize (speed 3) (safety 0))
-           (inline freetype:load-glyph freetype:render-glyph))
-  (with-slots (face im-cache) font
-    (declare (type (alien:alien freetype:face) face))
-    (freetype:load-glyph face (freetype:get-char-index face (char-code char)) 0)
-    (freetype:render-glyph (alien:slot face 'freetype:glyph) 0)
-    (let* ((glyph (alien:slot face 'freetype:glyph))
-           (bm (alien:slot glyph 'freetype:bitmap)))
-      (declare (type (alien:alien freetype:bitmap) bm)
-               (type (alien:alien freetype:glyph-slot) glyph))
-      (let* ((width  (alien:slot bm 'freetype:width))
-             (pitch  (alien:slot bm 'freetype:pitch))
-             (height (alien:slot bm 'freetype:rows))
-             (res    (make-array (list height width) :element-type '(unsigned-byte 8))))
-        (declare (type (simple-array (unsigned-byte 8) (* *)) res))
-        (let ((m (* width height)))
-          (locally
-              (declare (optimize (speed 3) (safety 0)))
-            (loop for y*width of-type fixnum below m by width 
-                  for y*pitch of-type fixnum from 0 by pitch do
-                  (loop for x of-type fixnum below width do
-                        (setf (row-major-aref res (+ x y*width))
-                              (alien:deref (alien:slot bm 'freetype:buffer) (+ x y*pitch)))))))
-        (values
-         res
-         (alien:slot glyph 'freetype:bitmap-left)
-         (alien:slot glyph 'freetype:bitmap-top)
-         (/ (alien:slot (alien:slot glyph 'freetype:advance) 'freetype:x) 64)
-         (/ (alien:slot (alien:slot glyph 'freetype:advance) 'freetype:y) 64))))))
+(defun make-concrete-font (vague-font size &key (dpi *dpi*))
+  (with-slots (lib filename) vague-font
+    (let* ((key (cons lib filename))
+           (val (gethash key *concrete-font-hash*)))
+      (unless val
+        (let ((facef (make-alien freetype:face)))
+          (declare (type (alien (* freetype:face)) facef))
+          (if (zerop (freetype:new-face lib filename 0 facef))
+              (setf val (setf (gethash key *concrete-font-hash*)
+                              (deref facef)))
+              (error "Freetype error in make-concrete-font"))))
+      (let ((face val))
+        (declare (type (alien freetype:face) face))
+        (freetype:set-char-size face 0 (round (* size 64)) (round dpi) (round dpi))
+        face))))
 
-(defmethod glyph-advance ((font concrete-font) char)
-  (with-slots (face) font
-    (freetype:load-glyph face (freetype:get-char-index face (char-code char)) 0)
-    (let* ((glyph (alien:slot face 'freetype:glyph)))
+(declaim (inline make-concrete-font))
+
+(defun glyph-pixarray (face char)
+  (declare (optimize (speed 3) (debug 1))
+           (inline freetype:load-glyph freetype:render-glyph)
+           (type (alien freetype:face) face))
+  (freetype:load-glyph face (freetype:get-char-index face (char-code char)) 0)
+  (freetype:render-glyph (slot face 'freetype:glyph) 0)
+  (symbol-macrolet
+      ((glyph (slot face 'freetype:glyph))
+       (bm (slot glyph 'freetype:bitmap)))
+    (let* ((width  (slot bm 'freetype:width))
+           (pitch  (slot bm 'freetype:pitch))
+           (height (slot bm 'freetype:rows))
+           (buffer (slot bm 'freetype:buffer))
+           (res    (make-array (list height width) :element-type '(unsigned-byte 8))))
+      (declare (type (simple-array (unsigned-byte 8) (* *)) res))
+      (let ((m (* width height)))
+        (locally
+            (declare (optimize (speed 3) (safety 0)))
+          (loop for y*width of-type fixnum below m by width 
+                for y*pitch of-type fixnum from 0 by pitch do
+                (loop for x of-type fixnum below width do
+                      (setf (row-major-aref res (+ x y*width))
+                            (deref buffer (+ x y*pitch)))))))
       (values
-       (/ (alien:slot (alien:slot glyph 'freetype:advance) 'freetype:x) 64)
-       (/ (alien:slot (alien:slot glyph 'freetype:advance) 'freetype:y) 64)))))
+       res
+       (slot glyph 'freetype:bitmap-left)
+       (slot glyph 'freetype:bitmap-top)
+       (/ (slot (slot glyph 'freetype:advance) 'freetype:x) 64)
+       (/ (slot (slot glyph 'freetype:advance) 'freetype:y) 64)))))
+
+(defun glyph-advance (face char)
+  (freetype:load-glyph face (freetype:get-char-index face (char-code char)) 0)
+  (let* ((glyph (slot face 'freetype:glyph)))
+    (values
+     (/ (slot (slot glyph 'freetype:advance) 'freetype:x) 64)
+     (/ (slot (slot glyph 'freetype:advance) 'freetype:y) 64))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -140,8 +148,8 @@
 (defun display-generate-glyph (display font matrix glyph-index)
   (let* ((glyph-id (display-draw-glyph-id display))
          (font (or (gethash font *font-hash*)
-                    (setf (gethash font *font-hash*)
-                          (make-vague-font font))))
+		   (setf (gethash font *font-hash*)
+			 (make-vague-font font))))
          (face (make-concrete-font font matrix)))
     (multiple-value-bind (arr left top dx dy) (glyph-pixarray face (code-char glyph-index))
       (when (= (array-dimension arr 0) 0)
@@ -152,7 +160,8 @@
                               :y-origin top
                               :x-advance dx
                               :y-advance dy)
-      (list glyph-id dx dy))))
+      (let ((right (+ left (array-dimension arr 1))))
+        (list glyph-id dx dy left right top)))))
 
 ;;;;;;; mcclim interface
 
@@ -174,22 +183,32 @@
 (defmethod clim-clx::font-glyph-width ((font freetype-face) char)
   (with-slots (display font matrix) font
     (nth 1 (display-get-glyph display font matrix char))))
+(defmethod clim-clx::font-glyph-left ((font freetype-face) char)
+  (with-slots (display font matrix) font
+    (nth 3 (display-get-glyph display font matrix char))))
+(defmethod clim-clx::font-glyph-right ((font freetype-face) char)
+  (with-slots (display font matrix) font
+    (nth 4 (display-get-glyph display font matrix char))))
 
+;;; this is a hacky copy of XLIB:TEXT-EXTENTS
 (defmethod clim-clx::font-text-extents ((font freetype-face) string
                                         &key (start 0) (end (length string)) translate)
   ;; -> (width ascent descent left right
   ;; font-ascent font-descent direction
   ;; first-not-done)
   translate
-  (values
-   (loop for i from start below end 
-         sum (clim-clx::font-glyph-width font (char-code (aref string i))))
-   (clim-clx::font-ascent font)
-   (clim-clx::font-descent font)
-   0 0 
-   (clim-clx::font-ascent font)
-   (clim-clx::font-descent font)
-   0 end))
+  (let ((width (loop for i from start below end 
+                     sum (clim-clx::font-glyph-width font (char-code (aref string i))))))
+    (values
+     width
+     (clim-clx::font-ascent font)
+     (clim-clx::font-descent font)
+     (clim-clx::font-glyph-left font (char-code (char string start)))
+     (- width (- (clim-clx::font-glyph-width font (char-code (char string (1- end))))
+                 (clim-clx::font-glyph-right font (char-code (char string (1- end))))))
+     (clim-clx::font-ascent font)
+     (clim-clx::font-descent font)
+     0 end)))
 
 (defun drawable-picture (drawable)
   (or (getf (xlib:drawable-plist drawable) 'picture)
@@ -238,20 +257,21 @@
 
 (let ((cache (make-hash-table :test #'equal)))
   (defun make-free-type-face (display font size)
-    (or (gethash (list font size) cache)
-        (setf (gethash (list font size) cache)
+    (or (gethash (list display font size) cache)
+        (setf (gethash (list display font size) cache)
               (let* ((f.font (or (gethash font *font-hash*)
                                  (setf (gethash font *font-hash*)
                                        (make-vague-font font))))
-                     (f (slot-value (make-concrete-font f.font size) 'face)))
+                     (f (make-concrete-font f.font size)))
+                (declare (type (alien freetype:face) f))
                 (make-instance 'freetype-face
                                :display display
                                :font font
                                :matrix size
-                               :ascent  (/ (alien:slot (alien:slot (alien:slot f 'freetype:size) 'freetype:metrics) 'freetype:ascender) 64)
-                               :descent (/ (alien:slot (alien:slot (alien:slot f 'freetype:size) 'freetype:metrics) 'freetype:descender) -64)))))))
+                               :ascent  (/ (slot (slot (slot f 'freetype:size_s) 'freetype:metrics) 'freetype:ascender) 64)
+                               :descent (/ (slot (slot (slot f 'freetype:size_s) 'freetype:metrics) 'freetype:descender) -64)))))))
 
-(defparameter mcclim-freetype::*sizes*
+(defparameter *sizes*
   '(:normal 12
     :small 10
     :very-small 8
@@ -260,38 +280,71 @@
     :very-large 18
     :huge 24))
 
-(defparameter mcclim-freetype::*families/faces*
-  '(((:fix :roman) . "/usr/local/lib/fonts/microsoft/cour.ttf")
-    ((:fix :italic) . "/usr/local/lib/fonts/microsoft/couri.ttf")
-    ((:fix :bold-italic) . "/usr/local/lib/fonts/microsoft/courbi.ttf")
-    ((:fix :italic-bold) . "/usr/local/lib/fonts/microsoft/courbi.ttf")
-    ((:fix :bold) . "/usr/local/lib/fonts/microsoft/courbd.ttf")
-    ((:serif :roman) . "/usr/local/lib/fonts/microsoft/times.ttf")
-    ((:serif :italic) . "/usr/local/lib/fonts/microsoft/timesi.ttf")
-    ((:serif :bold-italic) . "/usr/local/lib/fonts/microsoft/timesbi.ttf")
-    ((:serif :italic-bold) . "/usr/local/lib/fonts/microsoft/timesbi.ttf")
-    ((:serif :bold) . "/usr/local/lib/fonts/microsoft/timesbd.ttf")
-    ((:sans-serif :roman) . "/usr/local/lib/fonts/microsoft/verdana.ttf")
-    ((:sans-serif :italic) . "/usr/local/lib/fonts/microsoft/verdanai.ttf")
-    ((:sans-serif :bold-italic) . "/usr/local/lib/fonts/microsoft/verdanaz.ttf")
-    ((:sans-serif :italic-bold) . "/usr/local/lib/fonts/microsoft/verdanaz.ttf")
-    ((:sans-serif :bold) . "/usr/local/lib/fonts/microsoft/verdanab.ttf")))
+(defparameter *families/faces*
+  '(((:fix :roman) . "VeraMono.ttf")
+    ((:fix :italic) . "VeraMoIt.ttf")
+    ((:fix (:bold :italic)) . "VeraMoBI.ttf")
+    ((:fix (:italic :bold)) . "VeraMoBI.ttf")
+    ((:fix :bold) . "VeraMoBd.ttf")
+    ((:serif :roman) . "VeraSe.ttf")
+    ((:serif :italic) . "VeraSe.ttf")
+    ((:serif (:bold :italic)) . "VeraSeBd.ttf")
+    ((:serif (:italic :bold)) . "VeraSeBd.ttf")
+    ((:serif :bold) . "VeraSeBd.ttf")
+    ((:sans-serif :roman) . "Vera.ttf")
+    ((:sans-serif :italic) . "VeraIt.ttf")
+    ((:sans-serif (:bold :italic)) . "VeraBI.ttf")
+    ((:sans-serif (:italic :bold)) . "VeraBI.ttf")
+    ((:sans-serif :bold) . "VeraBd.ttf")))
 
-(defmethod clim-clx::text-style-to-X-font :around ((port clim-clx::clx-port) text-style)
-  (multiple-value-bind (family face size) (clim:text-style-components text-style)
-    (setf face (or face :roman))
-    (setf size (or size :normal))
-    (cond (size
-           (setf size (getf *sizes* size size))
-           (let ((q (cdr (assoc (list family face) *families/faces*
-                                :test #'equal))))
-             (if q
-                 (make-free-type-face (slot-value port 'clim-clx::display)
-                                      q
-                                      size)
-                 (call-next-method))))
-          (t
-           (call-next-method)))))
+(defvar *freetype-font-path*)
+
+(fmakunbound 'clim-clx::text-style-to-x-font)
+
+(defstruct freetype-device-font-name
+  (font-file (error "missing argument"))
+  (size (error "missing argument")))
+
+(defmethod clim-clx::text-style-to-X-font :around 
+    ((port clim-clx::clx-port) (text-style climi::device-font-text-style))
+  (let ((display (slot-value port 'clim-clx::display))
+        (font-name (climi::device-font-name text-style)))
+    (make-free-type-face display
+                         (freetype-device-font-name-font-file font-name)
+                         (freetype-device-font-name-size font-name))))
+
+(defmethod text-style-mapping :around
+    ((port clim-clx::clx-port) (text-style climi::device-font-text-style)
+     &optional character-set)
+  (values (gethash text-style (clim-clx::port-text-style-mappings port))))
+(defmethod (setf text-style-mapping) :around
+    (value 
+     (port clim-clx::clx-port) 
+     (text-style climi::device-font-text-style)
+     &optional character-set)
+  (setf (gethash text-style (clim-clx::port-text-style-mappings port)) value))
+
+(defparameter *free-type-face-hash* (make-hash-table :test #'equal))
+
+(defmethod clim-clx::text-style-to-X-font :around ((port clim-clx::clx-port) (text-style standard-text-style))
+  (multiple-value-bind (family face size) 
+      (clim:text-style-components text-style)
+    (let ((display (clim-clx::clx-port-display port)))
+      (setf face (or face :roman))
+      (setf size (or size :normal))
+      (cond (size
+	     (setf size (getf *sizes* size size))
+	     (let ((val (gethash (list display family face size) *free-type-face-hash*)))
+	       (if val val
+		   (setf (gethash (list display family face size) *free-type-face-hash*)
+			 (let* ((font-path-relative (cdr (assoc (list family face) *families/faces*
+								:test #'equal)))
+				(font-path (namestring (merge-pathnames font-path-relative *freetype-font-path*))))
+			   (if (and font-path (probe-file font-path))
+			       (make-free-type-face display font-path size)
+			       (call-next-method)))))))
+	    (t
+	     (call-next-method))))))
 
 (defmethod clim-clx::text-style-to-X-font ((port clim-clx::clx-port) text-style)
   (error "You lost: ~S." text-style))
@@ -355,6 +408,47 @@
                                           direction first-not-done))
                       (values width (+ ascent descent) width 0 ascent)) )))))) )
 
+(defmethod climi::text-bounding-rectangle*
+    ((medium clx-medium) string &key text-style (start 0) end)
+  (when (characterp string)
+    (setf string (make-string 1 :initial-element string)))
+  (unless end (setf end (length string)))
+  (unless text-style (setf text-style (medium-text-style medium)))
+  (let ((xfont (text-style-to-X-font (port medium) text-style)))
+    (cond ((= start end)
+           (values 0 0 0 0))
+          (t
+           (let ((position-newline (position #\newline string :start start)))
+             (cond ((not (null position-newline))
+                    (multiple-value-bind (width ascent descent left right
+                                                font-ascent font-descent direction
+                                                first-not-done)
+                        (font-text-extents xfont string
+                                           :start start :end position-newline
+                                           :translate #'translate)
+                      (declare (ignorable left right
+                                          font-ascent font-descent
+                                          direction first-not-done))
+                      (multiple-value-bind (minx miny maxx maxy)
+                          (climi::text-bounding-rectangle*
+                           medium string :text-style text-style
+                           :start (1+ position-newline) :end end)
+                        (values (min minx left) (- ascent)
+                                (max maxx right) (+ descent maxy)))))
+                   (t
+                    (multiple-value-bind (width ascent descent left right
+                                                font-ascent font-descent direction
+                                                first-not-done)
+                        (font-text-extents xfont string
+                                   :start start :end end
+                                   :translate #'translate)
+                      (declare (ignore width direction first-not-done))
+                      ;; FIXME: Potential style points:
+                      ;; * (min 0 left), (max width right)
+                      ;; * font-ascent / ascent
+                      (values left (- font-ascent) right font-descent)))))))))
+
+
 (defmethod make-medium-gcontext* (medium foreground background line-style text-style (ink color) clipping-region)
   (let* ((drawable (sheet-mirror (medium-sheet medium)))
          (port (port medium)))
@@ -467,4 +561,4 @@
     (unless (eql r clim:+nowhere+)
       (clim:with-drawing-options (m :clipping-region r)
         (clim:draw-design m r :ink clim:+background-ink+)
-        (call-next-method s r)))))
\ No newline at end of file
+        (call-next-method s r)))))
