@@ -66,7 +66,7 @@
   (with-slots (documentation text-style) menu-item
     (list ':documentation documentation ':text-style text-style)))
 
-(define-protocol-class command-table ()
+(defclass standard-command-table (command-table)
   ((name :initarg :name :reader command-table-name)
    (inherit-from :initarg :inherit-from
 		 :initform '()
@@ -81,14 +81,18 @@
    (keystroke-accelerators :initform nil)
    (keystroke-items :initform nil)))
 
-
-(defmethod print-object ((table command-table) stream)
+(defmethod print-object ((table standard-command-table) stream)
   (print-unreadable-object (table stream :identity t :type t)
     (format stream "~S" (command-table-name table))))
 
-(defclass standard-command-table (command-table)
-  ())
-   
+;;; Franz user manual says that this slot is setf-able
+(defgeneric (setf command-table-inherit-from) (inherit-from table))
+
+(defmethod (setf command-table-inherit-from)
+    (inherit (table standard-command-table))
+  (invalidate-translator-caches)
+  (setf (slot-value table 'inherit-from) inherit))
+
 (defparameter *command-tables* (make-hash-table :test #'eq))
 
 (define-condition command-table-error (error)
@@ -182,19 +186,6 @@
 			     :menu ',menu
 			     :errorp nil))))
 
-(defun command-name-from-symbol (symbol)
-  (let ((name (symbol-name symbol)))
-    (string-capitalize
-     (substitute
-      #\Space #\-
-      (subseq name (if (string= "COM-" name :end2 (min (length name) 4))
-		       4
-		       0))))))
-
-(defun keyword-arg-name-from-symbol (symbol)
-  (let ((name (symbol-name symbol)))
-    (string-capitalize (substitute #\Space #\- name))))
-
 (defun remove-command-from-command-table (command-name
 					  command-table
 					  &key (errorp t))
@@ -207,11 +198,11 @@
 	  (when (typep item 'menu-item)
 	    (remove-menu-item-from-command-table table
 						 (command-menu-item-name item)
-						 :errorp nil)
-	    
-	    (when (command-item-name item)
-	      (remhash (command-item-name item) (command-line-names table)))
-	    (remhash command-name (commands table)))))))
+                                                 :errorp nil))
+
+          (when (command-item-name item)
+            (remhash (command-item-name item) (command-line-names table)))
+          (remhash command-name (commands table))))))
 
 (defun add-command-to-command-table (command-name
 				     command-table
@@ -316,20 +307,17 @@
 
 (defun command-line-name-for-command (command-name command-table
 				      &key (errorp t))
-  (block exit				; save typing
-    (do-command-table-inheritance (table command-table)
-      (let* ((command-item (gethash command-name (slot-value table 'commands)))
-	     (command-line-name (and command-item
-				     (command-line-name command-item))))
-	(cond ((stringp command-line-name)
-	       (return-from exit command-line-name))
-	      ((eq errorp :create)
-	       (return-from exit (command-name-from-symbol command-name)))
-	      (errorp
-	       (error 'command-not-accessible))
-	      (t nil))))
-    nil))
-
+  (do-command-table-inheritance (table command-table)
+    (let* ((command-item (gethash command-name (slot-value table 'commands)))
+	   (command-line-name (and command-item
+				   (command-line-name command-item))))
+      (when (stringp command-line-name)
+	(return-from command-line-name-for-command command-line-name))))
+  (cond ((eq errorp :create)
+	 (command-name-from-symbol command-name))
+	(errorp
+	 (error 'command-not-accessible))
+	(t nil)))
 
 (defun find-menu-item (menu-name command-table &key (errorp t))
   (let* ((table (find-command-table command-table))
@@ -474,7 +462,8 @@
       (loop for gesture in keystroke-accelerators
 	    for item in keystroke-items
 	    do (funcall function
-			(command-menu-item-name item)
+                        (and (slot-boundp item 'menu-name)
+                             (command-menu-item-name item))
 			gesture
 			item)))))
 
@@ -641,6 +630,7 @@
 					     ,command-arg))
 		     else if (member key accept-keys :test #'eq)
 		       append `(,key ,val))))
+	(setq args (append args `(:query-identifier ',(gensym "COMMAND-PROMPT-ID"))))
 	(if (member :default args :test #'eq)
 	    `(accept ,ptype :stream ,stream ,@args)
 	    `(if (eq ,original-command-arg *unsupplied-argument-marker*)
@@ -749,30 +739,36 @@
       ;; We don't need fresh gensyms of these variables for each accept form.
       (with-gensyms (value ptype changedp)
 	`(defun ,name (,command-table ,stream ,partial-command)
-	   (destructuring-bind (,command-name ,@original-args)
-	       ,partial-command
-	     (let ((,command-line-name (command-line-name-for-command
-					,command-name
-					,command-table
-					:errorp nil))
-		   ,@(mapcar #'list required-arg-names original-args))
-	       (accepting-values (,stream)
-		 (format ,stream
-			 "You are being prompted for arguments to ~S~%"
-			 ,command-line-name)
-		 ,@(loop
-		      for var in required-arg-names
-		      for original-var in original-args
-		      for parameter in required-args
-		      append `((multiple-value-bind (,value ,ptype ,changedp)
-				   ,(accept-form-for-argument-partial
-				     stream parameter var original-var)
-				 (declare (ignore ,ptype))
-				 (terpri ,stream)
-				 (when ,changedp
-				   (setq ,var ,value))))))
-	       (list ,command-name ,@required-arg-names))))))))
-
+           (do ((still-missing nil t))
+               (nil)
+             (destructuring-bind (,command-name ,@original-args)
+                 ,partial-command
+               (let ((,command-line-name (command-line-name-for-command
+                                          ,command-name
+                                          ,command-table
+                                          :errorp nil))
+                     ,@(mapcar #'list required-arg-names original-args))
+                 (accepting-values (,stream)
+                   (format ,stream
+                           "You are being prompted for arguments to ~S~%~%"
+                           ,command-line-name)
+                   ,@(loop
+                        for var in required-arg-names
+                        for original-var in original-args
+                        for parameter in required-args
+                        append `((multiple-value-bind (,value ,ptype ,changedp)
+                                     ,(accept-form-for-argument-partial
+                                       stream parameter var original-var)
+                                   (declare (ignore ,ptype))
+                                   (terpri ,stream)
+                                   (when ,changedp
+                                     (setq ,var ,value)))))
+                   (when still-missing
+                     (format ,stream
+                             "~&Please supply all arguments.")))
+                 (setf ,partial-command (list ,command-name ,@required-arg-names))
+                 (unless (partial-command-p ,partial-command)
+                   (return ,partial-command))))))))))
 
 ;;; XXX What do to about :acceptably? Probably need to wait for Goatee "buffer
 ;;; streams" so we can insert an accept-result-extent in the buffer for
@@ -807,7 +803,7 @@
 	   into key-clauses
 	 finally (setq key-case-clauses key-clauses))
       `(defun ,name (,command ,stream)
-	 (declare (ignorable ,stream))
+	 ,(declare-ignorable-form* stream)
 	 (let* ((,seperator #\Space) (,command-args (cdr ,command))
 		,@required-arg-bindings)
 	   (declare (ignorable ,seperator ,command-args
@@ -1081,11 +1077,12 @@
 (define-presentation-method present (object (type command-name)
 				     stream
 				     (view textual-view)
-				     &key acceptably for-context-type)
-  (declare (ignore acceptably for-context-type))
-  (princ (command-line-name-for-command object command-table :errorp :create)
-	 stream))
-
+				     &key)
+  (let ((command-line-name (command-line-name-for-command object command-table
+							  :errorp nil)))
+    (if command-line-name
+	(write-string command-line-name stream)
+	(prin1 object stream))))
 
 (define-presentation-method accept ((type command-name) stream
 				    (view textual-view)
@@ -1216,6 +1213,23 @@
             (position *unsupplied-argument-marker* command)))
 	  (t (values command type)))))
 
+;;; A presentation type for empty input at the command line; something for
+;;; read-command to supply as a default.  The command is defined in
+;;; builtin-commands.lisp.
+
+(define-presentation-type null-command
+    ()
+  :inherit-from '(command :command-table global-command-table))
+
+(define-presentation-method presentation-typep (object (type null-command))
+  (and (consp object) (eq (car object) 'com-null-command)))
+
+(define-presentation-method present
+    (object (type null-command) stream (view textual-view) &key)
+  (declare (ignore object stream view)))
+
+(defparameter +null-command+ '(com-null-command))
+
 (defclass presentation-command-translator (presentation-translator)
   ()
   (:documentation "Wraps the tester function with a test that
@@ -1308,16 +1322,20 @@
 	  ((or (typep stream 'interactor-pane)
 	       (typep stream 'input-editing-stream))
 	   (handler-case
-	       (let ((command (accept `(command :command-table ,command-table)
-				      :stream stream
-				      :prompt nil)))
-		 (if (partial-command-p command)
-		     (progn
-		       (beep)
-		       (format *query-io* "~&Argument ~D not supplied.~&"
-			       (position *unsupplied-argument-marker* command))
-		       nil)
-		     command))
+	       (multiple-value-bind (command ptype)
+		   (accept `(command :command-table ,command-table)
+			   :stream stream
+			   :prompt nil
+			   :default +null-command+
+			   :default-type 'null-command)
+		 (cond ((eq ptype 'null-command)
+			nil)
+		       ((partial-command-p command)
+			(beep)
+			(format *query-io* "~&Argument ~D not supplied.~&"
+				(position *unsupplied-argument-marker* command))
+			nil)
+		       (t command)))
 	     ((or simple-parse-error input-not-of-required-type)  (c)
 	       (beep)
 	       (fresh-line *query-io*)
