@@ -79,15 +79,8 @@
 (in-package :clim-internals)
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
-(define-protocol-class design ())
 
-;; Some internal superclasses:
-
-(define-protocol-class uniform-design ())
-
-;;; Color class
-
-(define-protocol-class color (design uniform-design))
+(defgeneric color-rgb (color))
 
 (defmethod print-object ((color color) stream)
   (print-unreadable-object (color stream :identity nil :type t)
@@ -239,6 +232,8 @@
              (saturation (atan f2 f1)))
         (values intensity hue saturation)))))
 
+(defgeneric color-ihs (color))
+
 (defmethod color-ihs ((color color))
   (multiple-value-call #'rgb-to-ihs (color-rgb color)))
 
@@ -307,8 +302,6 @@
 ;;;; 13.4 Opacity
 ;;;;
 
-(define-protocol-class opacity (design))
-
 (defmethod print-object ((object opacity) stream)
   (print-unreadable-object (object stream :identity nil :type t)
     (format stream "~S" (opacity-value object))))
@@ -354,6 +347,8 @@
     (print-unreadable-object (flipper stream :identity nil :type t)
       (format stream "~S ~S" design1 design2))))
 
+(defgeneric make-flipping-ink (design1 design2))
+
 (defmethod make-flipping-ink ((design1 design) (design2 design))
   (make-instance 'standard-flipping-ink :design1 design1 :design2 design2))
 
@@ -396,10 +391,14 @@
 (defun make-pattern (array designs)
   (make-instance 'indexed-pattern :array array :designs designs))
 
+(defgeneric pattern-width (pattern))
+
 (defmethod pattern-width ((pattern indexed-pattern))
   (with-slots (array) pattern
     (array-dimension array 1)))
 
+(defgeneric pattern-height (pattern))
+
 (defmethod pattern-height ((pattern indexed-pattern))
   (with-slots (array) pattern
     (array-dimension array 0)))
