@@ -30,12 +30,6 @@
 
 ;;; Frame-Manager class
 
-(define-protocol-class frame-manager ()
-  ((port :initarg :port
-	 :reader frame-manager-port)
-   (frames :initform nil
-	   :reader frame-manager-frames)))
-
 (defun find-frame-manager (&rest options &key port &allow-other-keys)
   (declare (special *frame-manager*))
   (if (boundp '*frame-manager*)
@@ -142,68 +136,6 @@ FRAME-EXIT condition."))
   (:documentation "Called when a pane receives or loses the keyboard
 input focus. This is a McCLIM extension."))
 
-(define-protocol-class  application-frame ()
-  ((port :initform nil
-	 :initarg :port
-	 :accessor port)
-   (graft :initform nil
-	  :initarg :graft
-	  :accessor graft)
-   (name :initarg :name
-	 :reader frame-name)
-   (pretty-name :initarg :pretty-name
-		:accessor frame-pretty-name)
-   (command-table :initarg :command-table
-		  :initform nil
-		  :accessor frame-command-table)
-   (disabled-commands :initarg :disabled-commands
-		      :initform nil
-		      :accessor frame-disabled-commands)
-   (named-panes :accessor frame-named-panes :initform nil)
-   (panes :initform nil :reader frame-panes
-	  :documentation "The tree of panes in the current layout.")
-   (layouts :initform nil
-	    :initarg :layouts
-	    :reader frame-layouts)
-   (current-layout :initform nil
-		   :initarg :current-layout
-		   :accessor frame-current-layout)
-   (panes-for-layout :initform nil :accessor frame-panes-for-layout
-		     :documentation "alist of names and panes (as returned by make-pane)")
-   (top-level-sheet :initform nil
-		    :reader frame-top-level-sheet)
-   (menu-bar :initarg :menu-bar
-	     :initform nil)
-   (calling-frame :initarg :calling-frame
-		  :initform nil)
-   (state :initarg :state
-	  :initform :disowned
-	  :reader frame-state)
-   (manager :initform nil
-	    :reader frame-manager
-            :accessor %frame-manager)
-   (keyboard-input-focus :initform nil
-                         :accessor keyboard-input-focus)
-   (properties :accessor %frame-properties
-	       :initarg :properties
-	       :initform nil)
-   (top-level :initform '(default-frame-top-level)
-	      :initarg :top-level
-	      :reader frame-top-level)
-   (top-level-lambda :initarg :top-level-lambda
-		     :reader frame-top-level-lambda)
-   (hilited-presentation :initform nil
-			 :initarg :hilited-presentation
-			 :accessor frame-hilited-presentation)
-   (user-supplied-geometry :initform nil
-			   :initarg :user-supplied-geometry
-                           :documentation "plist of defaulted :left, :top, :bottom, :right, :width and :height options.")
-   (process :reader frame-process :initform (current-process))
-   (client-settings :accessor client-settings :initform nil)))
-
-(defmethod frame-geometry ((frame application-frame))
-  (slot-value frame 'user-supplied-geometry))
-
 (defmethod frame-geometry* ((frame application-frame))
   "-> width height &optional top left"
   (let ((pane (frame-top-level-sheet frame)))
@@ -226,7 +158,60 @@ input focus. This is a McCLIM extension."))
     
 (defclass standard-application-frame (application-frame
 				      presentation-history-mixin)
-  ((event-queue :initarg :frame-event-queue
+  ((port :initform nil
+	 :initarg :port
+	 :accessor port)
+   (graft :initform nil
+	  :initarg :graft
+	  :accessor graft)
+   (name :initarg :name
+	 :reader frame-name)
+   (pretty-name :initarg :pretty-name
+		:accessor frame-pretty-name)
+   (command-table :initarg :command-table
+		  :initform nil
+		  :accessor frame-command-table)
+   (named-panes :accessor frame-named-panes :initform nil)
+   (panes :initform nil :reader frame-panes
+	  :documentation "The tree of panes in the current layout.")
+   (layouts :initform nil
+	    :initarg :layouts
+	    :reader frame-layouts)
+   (current-layout :initform nil
+		   :initarg :current-layout
+		   :accessor frame-current-layout)
+   (panes-for-layout :initform nil :accessor frame-panes-for-layout
+		     :documentation "alist of names and panes (as returned by make-pane)")
+   (top-level-sheet :initform nil
+		    :reader frame-top-level-sheet)
+   (menu-bar :initarg :menu-bar
+	     :initform nil)
+   (state :initarg :state
+	  :initform :disowned
+	  :reader frame-state)
+   (manager :initform nil
+	    :reader frame-manager
+            :accessor %frame-manager)
+   (keyboard-input-focus :initform nil
+                         :accessor keyboard-input-focus)
+   (properties :accessor %frame-properties
+	       :initarg :properties
+	       :initform nil)
+   (top-level :initform '(default-frame-top-level)
+	      :initarg :top-level
+	      :reader frame-top-level)
+   (top-level-lambda :initarg :top-level-lambda
+		     :reader frame-top-level-lambda)
+   (hilited-presentation :initform nil
+			 :initarg :hilited-presentation
+			 :accessor frame-hilited-presentation)
+   (user-supplied-geometry :initform nil
+			   :initarg :user-supplied-geometry
+			   :reader frame-geometry
+                           :documentation "plist of defaulted :left, :top, :bottom, :right, :width and :height options.")
+   (process :accessor frame-process :initform nil)
+   (client-settings :accessor client-settings :initform nil)
+   (event-queue :initarg :frame-event-queue
                 :initarg :input-buffer
                 :initform nil
 		:accessor frame-event-queue
@@ -242,9 +227,15 @@ input focus. This is a McCLIM extension."))
 		  :documentation "The frame that is the parent of this
 frame, if any")
    (disabled-commands :accessor disabled-commands
+		      :accessor frame-disabled-commands
+		      :initarg :disabled-commands
 		      :initform nil
 		      :documentation "A list of command names that have been
-				      disabled in this frame")))
+				      disabled in this frame")
+   (documentation-record :accessor documentation-record
+			 :initform nil
+			 :documentation "updating output record for pointer
+documentation produced by presentations.")))
 
 ;;; Support the :input-buffer initarg for compatibility with "real CLIM"
 
@@ -272,8 +263,11 @@ frame, if any")
 
 (defmethod (setf frame-current-layout) :after (name (frame application-frame))
   (declare (ignore name))
-  (generate-panes (frame-manager frame) frame)
-  (signal 'frame-layout-changed :frame frame))
+  (when (frame-manager frame)
+    (generate-panes (frame-manager frame) frame)
+    (multiple-value-bind (w h) (frame-geometry* frame)
+      (layout-frame frame w h))  
+    (signal 'frame-layout-changed :frame frame)))
 
 (defmethod generate-panes :before (fm  (frame application-frame))
   (declare (ignore fm))
@@ -424,6 +418,23 @@ frame, if any")
   (declare (ignore pane force-p))
   nil)
 
+(defgeneric medium-invoke-with-possible-double-buffering (frame pane medium continuation))
+
+(defmethod medium-invoke-with-possible-double-buffering (frame pane medium continuation)
+  (funcall continuation))
+
+(defgeneric invoke-with-possible-double-buffering (frame pane continuation))
+
+(defmethod invoke-with-possible-double-buffering (frame pane continuation)
+  (declare (ignore frame pane))
+  (funcall continuation))
+
+(defmethod invoke-with-possible-double-buffering (frame (pane sheet-with-medium-mixin) continuation)
+  (medium-invoke-with-possible-double-buffering frame pane (sheet-medium pane) continuation))
+
+(defmacro with-possible-double-buffering ((frame pane) &body body)
+  `(invoke-with-possible-double-buffering ,frame ,pane (lambda () ,@body)))
+
 (defmethod redisplay-frame-pane :around ((frame application-frame) pane
 					 &key force-p)
   (multiple-value-bind (redisplayp clearp)
@@ -436,18 +447,21 @@ frame, if any")
 	(when hilited
 	  (highlight-presentation-1 (car hilited) (cdr hilited) :unhighlight)
 	  (setf (frame-hilited-presentation frame) nil)))
-      (when clearp
-	(window-clear pane))
-      (call-next-method)
+      (with-possible-double-buffering (frame pane)
+	(when clearp
+	  (window-clear pane))
+	(call-next-method))
       (unless (or (eq redisplayp :command-loop) (eq redisplayp :no-clear))
 	(setf (pane-needs-redisplay pane) nil)))))
 
 (defmethod run-frame-top-level ((frame application-frame)
 				&key &allow-other-keys)
-  (handler-case
-      (funcall (frame-top-level-lambda frame) frame)
-    (frame-exit ()
-      nil)))
+  (letf (((frame-process frame) (current-process)))
+    (handler-case
+	(funcall (frame-top-level-lambda frame) frame)
+      (frame-exit ()
+	nil))))
+
 
 (defmethod run-frame-top-level :around ((frame application-frame) &key)
   (let ((*application-frame* frame)
@@ -481,25 +495,6 @@ frame, if any")
 	  (:disowned
 	   (disown-frame fm frame)))))))
 
-;;; Defined in incremental-redisplay.lisp
-(defvar *enable-updating-output*)
-
-#+nil
-(defun redisplay-changed-panes (frame)
-  (map-over-sheets #'(lambda (pane)
-                       (multiple-value-bind (redisplayp clearp)
-                           (pane-needs-redisplay pane)
-                         (when redisplayp
-                           (when (and clearp
-                                      (or (not (pane-incremental-redisplay
-                                                pane))
-                                          (not *enable-updating-output*)))
-                             (window-clear pane))
-                           (redisplay-frame-pane frame pane)
-                           (unless (eq redisplayp :command-loop)
-                             (setf (pane-needs-redisplay pane) nil)))))
-                   (frame-top-level-sheet frame)))
-
 (defparameter +default-prompt-style+ (make-text-style :fix :italic :normal))
 
 (defmethod default-frame-top-level
@@ -578,10 +573,33 @@ frame, if any")
 
 (defmethod read-frame-command ((frame application-frame)
 			       &key (stream *standard-input*))
+  ;; The following is the correct interpretation according to the spec.
+  ;; I think it is terribly counterintuitive and want to look into
+  ;; what existing CLIMs do before giving in to it.
+  ;; If we do things as the spec says, command accelerators will
+  ;; appear to not work, confusing new users.
+  #+NIL (read-command (frame-command-table frame) :use-keystrokes nil :stream stream)
   (read-command (frame-command-table frame) :use-keystrokes t :stream stream))
 
+(defclass execute-command-event (window-manager-event)
+  ((sheet :initarg :sheet :reader event-sheet)
+   (command :initarg :command :reader execute-command-event-command)))
+
 (defmethod execute-frame-command ((frame application-frame) command)
-  (apply (command-name command) (command-arguments command)))
+  ;; ### FIXME: I'd like a different method than checking for
+  ;; *application-frame* to decide, which process processes which
+  ;; frames command loop. Perhaps looking ath the process slot?
+  ;; --GB 2005-11-28
+  (cond ((eq *application-frame* frame)
+         (apply (command-name command) (command-arguments command)))
+        (t
+         (let ((eq (sheet-event-queue (frame-top-level-sheet frame))))
+           (event-queue-append eq (make-instance 'execute-command-event
+                                                  :sheet frame
+                                                  :command command))))))
+
+(defmethod handle-event ((frame application-frame) (event execute-command-event))
+  (execute-frame-command frame (execute-command-event-command event)))
 
 (defmethod command-enabled (command-name (frame standard-application-frame))
   (and (command-accessible-in-command-table-p command-name
@@ -661,6 +679,9 @@ frame, if any")
 
 (defgeneric enable-frame (frame))
 (defgeneric disable-frame (frame))
+(defgeneric destroy-frame (frame))
+(defgeneric raise-frame (frame))
+(defgeneric bury-frame (frame))
 
 (defgeneric note-frame-enabled (frame-manager frame))
 (defgeneric note-frame-disbled (frame-manager frame))
@@ -675,6 +696,17 @@ frame, if any")
   (setf (slot-value frame 'state) :disabled)
   (note-frame-disabled (frame-manager frame) frame))
 
+(defmethod destroy-frame ((frame application-frame))
+  (when (eq (frame-state frame) :enabled)
+    (disable-frame frame))
+  (disown-frame (frame-manager frame) frame))
+
+(defmethod raise-frame ((frame application-frame))
+  (raise-sheet (frame-top-level-sheet frame)))
+
+(defmethod bury-frame ((frame application-frame))
+  (bury-sheet (frame-top-level-sheet frame)))
+
 (defmethod note-frame-enabled ((fm frame-manager) frame)
   (declare (ignore frame))
   t)
@@ -683,6 +715,15 @@ frame, if any")
   (declare (ignore frame))
   t)
 
+(defun map-over-frames (function &key port frame-manager)
+  (cond (frame-manager
+	 (mapc function (frame-manager-frames frame-manager)))
+	(port
+	 (loop for manager in (frame-managers port)
+	       do (map-over-frames function :frame-manager manager)))
+	(t (loop for p in *all-ports*
+		 do (map-over-frames function :port p)))))
+
 (defvar *pane-realizer* nil)
 
 (defmacro with-look-and-feel-realization ((frame-manager frame) &body body)
@@ -876,11 +917,11 @@ frame, if any")
       ,@(if command-table
             `((define-command-table ,@command-table)))
       ,@(if command-definer
-            `((defmacro ,command-definer (name-and-options arguements &rest body)
+            `((defmacro ,command-definer (name-and-options arguments &rest body)
                 (let ((name (if (listp name-and-options) (first name-and-options) name-and-options))
                       (options (if (listp name-and-options) (cdr name-and-options) nil))
                       (command-table ',(first command-table)))
-                  `(define-command (,name :command-table ,command-table ,@options) ,arguements ,@body))))))))
+                  `(define-command (,name :command-table ,command-table ,@options) ,arguments ,@body))))))))
 
 (defun get-application-frame-class-geometry (name indicator)
   (getf (get name 'application-frame-geometry) indicator nil))
@@ -923,6 +964,40 @@ frame, if any")
 	     (warn ":state ~S not supported yet." state)))
       frame)))
 
+;;; From Franz Users Guide
+
+(defun find-application-frame (frame-name &rest initargs
+			       &key (create t) (activate t)
+			       (own-process *multiprocessing-p*) port
+			       frame-manager frame-class
+			       &allow-other-keys)
+  (declare (ignorable frame-class))
+  (let ((frame (unless (eq create :force)
+		 (block
+		     found-frame
+		   (map-over-frames
+		    #'(lambda (frame)
+			(when (eq (frame-name frame) frame-name)
+			  (return-from found-frame frame)))
+		    :port port
+		    :frame-manager frame-manager)))))
+    (unless (or frame create)
+      (return-from find-application-frame nil))
+    (unless frame
+      (with-keywords-removed (initargs (:create :activate :own-process))
+	(setq frame (apply #'make-application-frame frame-name initargs))))
+    (when (and frame activate)
+      (cond ((frame-process frame)
+	     (raise-frame frame))
+	    (own-process
+	     (clim-sys:make-process #'(lambda ()
+					(run-frame-top-level frame))
+				    :name (format nil "~A" frame-name)))
+	    (t (run-frame-top-level frame))))
+    frame))
+
+
+
 ;;; Menu frame class
 
 (defclass menu-frame ()
@@ -1049,7 +1124,8 @@ frame, if any")
 							  long))
 		 (setq trailing t)))))
 
-
+;;; XXX Warning: Changing rapidly!
+;;;
 ;;; We don't actually want to print out the translator documentation and redraw
 ;;; the pointer documentation window on every motion event.  So, we compute a
 ;;; state object (basically modifier state and a list of the applicable
@@ -1166,6 +1242,43 @@ frame, if any")
 		     (print-modifiers pstream modifier :long)))
 	  (write-char #\. pstream))))))
 
+(defmethod frame-update-pointer-documentation
+    ((frame standard-application-frame) input-context stream event)
+  (when *pointer-documentation-output*
+    (with-accessors ((frame-documentation-state frame-documentation-state)
+		     (documentation-record documentation-record))
+      frame
+      (setf frame-documentation-state
+	    (frame-compute-pointer-documentation-state frame
+						       input-context
+						       stream
+						       event))
+      ;; These ugly special bindings work around the fact that the outer
+      ;; updating-output form closes over its body and allow the inner
+      ;; form to see the correct, current values of those variables.
+      (let ((%input-context% input-context)
+	    (%stream% stream)
+	    (%doc-state% frame-documentation-state)
+	    (%event% event))
+	(declare (special %input-context% %stream% %doc-state% %event&))
+	(if (and documentation-record
+		 (output-record-parent documentation-record))
+	    (redisplay documentation-record *pointer-documentation-output*)
+	    (progn
+	      (window-clear *pointer-documentation-output*)
+	      (setf documentation-record
+		    (updating-output (*pointer-documentation-output*)
+		      (updating-output (*pointer-documentation-output*
+					:cache-value %doc-state%
+					:cache-test
+					#'equal)
+			(frame-print-pointer-documentation frame
+							   %input-context%
+							   %stream%
+							   %doc-state%
+							   %event%))))))))))
+    
+#-(and)
 (defmethod frame-update-pointer-documentation
     ((frame standard-application-frame) input-context stream event)
   (when *pointer-documentation-output*
@@ -1312,13 +1425,16 @@ frame, if any")
      (highlight nil highlightp) context-type
      multiple-window)
   (declare (ignore multiple-window))
-  (setf (slot-value obj 'highlight) (if highlightp
-					highlight
-					(or presentation
-					    presentation-button-press
-					    presentation-button-release)))
+  (let ((presentation-clauses-p (or presentation
+				    presentation-button-press
+				    presentation-button-release)))
+    (setf (slot-value obj 'highlight) (if highlightp
+					  highlight
+					  presentation-clauses-p))
   (setf (slot-value obj 'input-context)
-	(make-fake-input-context context-type)))
+	(if (or presentation-clauses-p highlight)
+	    (make-fake-input-context context-type)
+	    nil))))
 
 (defmethod make-tracking-pointer-state
     ((frame standard-application-frame)	sheet args)
@@ -1339,44 +1455,43 @@ frame, if any")
 
 ;;; Poor man's find-applicable-translators. tracking-pointer doesn't want to
 ;;; see any results from presentation translators.
-
-(defun highlight-for-tracking-pointer (frame stream x y input-context)
-  (let ((context-ptype (input-context-type (car input-context)))
-	(presentation nil)
+;;;
+;;; XXX I don't see why not (even though I wrote the above comment :) and
+;;; Classic CLIM seems to agree. -- moore
+(defun highlight-for-tracking-pointer (frame stream event input-context
+				       highlight)
+  (let ((presentation nil)
 	(current-hilited (frame-hilited-presentation frame)))
-    (if (output-recording-stream-p stream)
-	(progn
-	  (block found-presentation
-	    (flet ((do-presentation (p)
-		     (when (presentation-subtypep (presentation-type p)
-						  context-ptype)
-		       (setq presentation p)
-		       (return-from found-presentation nil))))
-	      (declare (dynamic-extent #'do-presentation))
-	      (map-over-presentations-containing-position
-	       #'do-presentation (stream-output-history stream) x y)))
-	  (when (and current-hilited
-		     (not (eq (car current-hilited) presentation)))
-	    (highlight-presentation-1 (car current-hilited)
-				      (cdr current-hilited)
-				      :unhighlight))
-	  (if presentation
-	      (progn
-		(setf (frame-hilited-presentation frame)
-		      (cons presentation stream))
-		(highlight-presentation-1 presentation stream :highlight)))
-	  presentation))))
+    (when (output-recording-stream-p stream)
+      ;; XXX Massive hack to prevent the presentation action for completions
+      ;; from being applicable. After the .9.2.2 release that action will have
+      ;; a more restrictive context type.
+      (let ((*completion-possibilities-continuation* nil))
+	(setq presentation (find-innermost-applicable-presentation
+			    input-context
+			    stream
+			    (device-event-x event)
+			    (device-event-y event)
+			    :frame frame))))
+    (when (and current-hilited (not (eq (car current-hilited) presentation)))
+      (highlight-presentation-1 (car current-hilited)
+				(cdr current-hilited)
+				:unhighlight))
+    (when (and presentation highlight)
+      (setf (frame-hilited-presentation frame) (cons presentation stream))
+      (highlight-presentation-1 presentation stream :highlight))
+    presentation))
 
 (defmethod tracking-pointer-loop-step :before
     ((state frame-tracking-pointer-state) (event pointer-event) x y)
   (declare (ignore x y))
-  (when (highlight state)
+  (when (input-context state)
     (let ((stream (event-sheet event)))
       (setf (applicable-presentation state)
 	    (highlight-for-tracking-pointer *application-frame* stream
-					    (device-event-x event)
-					    (device-event-y event)
-					    (input-context state))))))
+					    event
+					    (input-context state)
+					    (highlight state))))))
 
 
 (macrolet ((frob (event handler)
@@ -1426,26 +1541,196 @@ frame, if any")
 	    (null
 	     ())))))))
 
-(defun frame-drag (translator-name command-table object presentation
-		   context-type frame event window x y)
-  (let* ((translator (gethash translator-name
-			      (translators (presentation-translators
-					    (find-command-table
-					     command-table)))))
-	 (tester (tester translator))
-	 (drag-type (from-type translator))
-	 (feedback-fn (feedback translator))
-	 (hilite-fn (highlighting translator))
-	 (drag-c-type `(drag-over ))
-	 (drag-context (make-fake-input-context drag-c-type))
-	 (*dragged-object* object)
-	 (destination-object nil))
-    (multiple-value-bind (x0 y0)
-	(stream-pointer-position window)
-      (funcall feedback-fn *application-frame* object window
-	       x0 y0 x0 y0 :highlight)
-      (tracking-pointer (window :context-type drag-c-type :highlight nil)
-       (:pointer-motion (&key event x y)
-	 (multiple-value-bind (presentation translator)
-	     (find-innermost-presentation-match drag-context window
-						x y :event event)))))))
+(defgeneric frame-drag-and-drop-feedback
+    (frame from-presentation stream initial-x initial-y x y state))
+
+(defmethod frame-drag-and-drop-feedback
+    ((frame standard-application-frame) from-presentation stream
+     initial-x initial-y x y state)
+  (with-bounding-rectangle* (fp-x1 fp-y1 fp-x2 fp-y2)
+      from-presentation
+    ;; Offset from origin of presentation is preserved throughout
+    (let* ((x-off (-  fp-x1 initial-x))
+	   (y-off (-  fp-y1 initial-y))
+	   (hilite-x1 (+ x-off x))
+	   (hilite-y1 (+ y-off y))
+	   (hilite-x2 (+ hilite-x1 (- fp-x2 fp-x1)))
+	   (hilite-y2 (+ hilite-y1 (- fp-y2 fp-y1))))
+      (with-identity-transformation (stream)
+	(ecase state
+	  (:highlight
+	   (with-output-recording-options (stream :record nil)
+	     (draw-rectangle* stream hilite-x1 hilite-y1 hilite-x2 hilite-y2
+			      :filled nil :line-dashes #(4 4))))
+	  (:unhighlight
+	   (with-double-buffering
+	       ((stream hilite-x1 hilite-y1 (1+ hilite-x2) (1+ hilite-y2))
+		(buffer-rectangle))
+	     (stream-replay stream buffer-rectangle))))))))
+
+(defgeneric frame-drag-and-drop-highlighting
+    (frame to-presentation stream state))
+
+(defmethod frame-drag-and-drop-highlighting
+    ((frame standard-application-frame) to-presentation stream state)
+  (highlight-presentation-1 to-presentation stream state))
+
+(defun frame-drag-and-drop (translator-name command-table
+			    from-presentation context-type frame event window
+			    x y)
+  (declare (ignore command-table))
+  (let* ((*dragged-presentation* from-presentation)
+	 (*dragged-object* (presentation-object from-presentation))
+	 (translators (mapcan (lambda (trans)
+				(and (typep trans 'drag-n-drop-translator)
+				     (funcall (tester trans)
+					      (presentation-object
+					       from-presentation)
+					      :presentation from-presentation
+					      :context-type context-type
+					      :frame frame
+					      :window window
+					      :x x
+					      :y y
+					      :event event)
+				     (list trans)))
+			      (find-presentation-translators
+			       (presentation-type from-presentation)
+			       context-type
+			       (frame-command-table frame))))
+	 ;; Try to run the feedback and highlight functions of the translator
+	 ;; that got us here.
+	 (translator (or (find translator-name translators :key #'name)
+			 (car translators)))
+	 (initial-feedback-fn (feedback translator))
+	 (initial-hilite-fn (highlighting translator))
+	 (destination-presentation nil)
+	 (initial-x x)
+	 (initial-y y)
+	 (last-presentation nil)
+	 (feedback-activated nil)
+	 (feedback-fn initial-feedback-fn)
+	 (hilite-fn initial-hilite-fn)
+	 (last-event nil))
+    ;; We shouldn't need to use find-innermost-presentation-match
+    ;; This repeats what tracking-pointer has already done, but what are you
+    ;; gonna do?
+    (flet ((find-dest-translator (presentation window x y)
+	     (loop for translator in translators
+		   when (and (presentation-subtypep
+			      (presentation-type presentation)
+			      (destination-ptype translator))
+			     (test-presentation-translator translator
+							   presentation
+							   context-type frame
+							   window x y))
+		   do (return-from find-dest-translator translator))
+	     nil)
+	   (do-feedback (window x y state)
+	     (funcall feedback-fn frame from-presentation window
+			initial-x initial-y x y state))
+	   (do-hilite (presentation window state)
+	     (funcall hilite-fn frame presentation window state))
+	   (last-window ()
+	     (event-sheet last-event))
+	   (last-x ()
+	     (pointer-event-x last-event))
+	   (last-y ()
+	     (pointer-event-y last-event)))
+      ;; :highlight nil will cause the presentation that is the source of the
+      ;; dragged object to be unhighlighted initially.
+      (block do-tracking
+	(tracking-pointer (window :context-type `(or ,(mapcar #'from-type
+							      translators))
+				  :highlight nil
+				  :multiple-window nil)	;XXX
+	  (:presentation (&key presentation window event x y)
+	    (let ((dest-translator (find-dest-translator presentation window
+							 x y)))
+	      (when feedback-activated
+		(do-feedback (last-window) (last-x) (last-y) :unhighlight))
+	      (setq feedback-activated t
+		    last-event event)
+	      (when last-presentation
+		(do-hilite last-presentation (last-window) :unhighlight))
+	      (setq last-presentation presentation
+		    feedback-fn (feedback dest-translator)
+		    hilite-fn (highlighting dest-translator))
+	      (do-hilite presentation window :highlight)
+	      (do-feedback window x y :highlight)
+	      (document-drag-n-drop dest-translator presentation
+				    context-type frame event window
+				    x y)))
+	  (:pointer-motion (&key event window x y)
+	    (when feedback-activated
+	      (do-feedback (last-window) (last-x) (last-y) :unhighlight))
+	    (setq feedback-activated t
+		  last-event event)
+	    (when last-presentation
+	      (do-hilite last-presentation (last-window) :unhighlight))
+	    (setq last-presentation nil)
+	    (do-feedback window x y :highlight)
+	    (document-drag-n-drop translator nil
+				  context-type frame event window
+				  x y))
+	  ;; XXX only support finish-on-release for now.
+	  #-(and)(:presentation-button-press ())
+	  (:presentation-button-release (&key presentation event)
+	    (setq destination-presentation presentation
+		  last-event event)
+	    (return-from do-tracking nil))
+	  #-(and)(:button-press ())
+	  (:button-release (&key event)
+	    (setq last-event event)
+	    (return-from do-tracking nil))))
+      ;;
+      ;; XXX Assumes x y from :button-release are the same as for the preceding
+      ;; button-motion; is that correct?
+      (when feedback-activated
+	(do-feedback (last-window) (last-x) (last-y) :unhighlight))
+      (when last-presentation
+	(do-hilite last-presentation (last-window) :unhighlight))
+      (if destination-presentation
+	  (let ((final-translator (find-dest-translator destination-presentation
+							(last-window)
+							(last-x)
+							(last-y))))
+	    (if final-translator
+		(funcall (destination-translator final-translator)
+			 *dragged-object*
+			 :presentation *dragged-presentation*
+			 :destination-object (presentation-object
+					      destination-presentation)
+			 :destination-presentation destination-presentation
+			 :context-type context-type
+			 :frame frame
+			 :event event
+			 :window window
+			 :x x
+			 :y y)
+		(values nil nil)))
+	  (values nil nil)))))
+
+(defun document-drag-n-drop
+    (translator presentation context-type frame event window x y)
+  (when *pointer-documentation-output*
+    (let ((s *pointer-documentation-output*))
+      (window-clear s)
+      (with-end-of-page-action (s :allow)
+	(with-end-of-line-action (s :allow)
+	  (funcall (pointer-documentation translator)
+	   *dragged-object*
+	   :presentation *dragged-presentation*
+	   :destination-object (and presentation
+				    (presentation-object presentation))
+	   :destination-presentation presentation
+	   :context-type context-type
+	   :frame frame
+	   :event event
+	   :window window
+	   :x x
+	   :y y
+	   :stream s))))))
+
+
+
