@@ -3,10 +3,11 @@
 ;;;     Title: Graph Formatting
 ;;;   Created: 2002-08-13
 ;;;   License: LGPL (See file COPYING for details).
-;;;       $Id: graph-formatting.lisp,v 1.11 2004/12/05 19:37:52 hefner1 Exp $
+;;;       $Id: graph-formatting.lisp,v 1.17 2006/03/10 21:58:13 tmoore Exp $
 ;;; ---------------------------------------------------------------------------
 
 ;;;  (c) copyright 2002 by Gilbert Baumann
+;;;  (c) copyright 2005 by Robert P. Goldman
 
 ;;; This library is free software; you can redistribute it and/or
 ;;; modify it under the terms of the GNU Library General Public
@@ -53,8 +54,6 @@
 
 ;; format-graph-from-roots
 
-(define-protocol-class graph-output-record (output-record))
-
 (defgeneric graph-root-nodes (graph-output-record))
 (defgeneric (setf graph-root-nodes) (new-value graph-output-record))
 (defgeneric generate-graph-nodes (graph-output-record stream root-objects
@@ -64,8 +63,6 @@
 (defgeneric layout-graph-edges (graph-output-record stream arc-drawer arc-drawing-options))
 ;;; NOTE: Which calls which? --GB 2002-08-13
 
-(define-protocol-class graph-node-output-record (output-record))
-
 (defgeneric graph-node-parents (graph-node-record))
 (defgeneric (setf graph-node-parents) (new-value graph-node-record))
 (defgeneric graph-node-children (graph-node-record))
@@ -125,16 +122,28 @@
                                      merge-duplicates duplicate-key duplicate-test
                                      generation-separation
                                      within-generation-separation
-                                     center-nodes arc-drawer arc-drawing-options
+                                     center-nodes
+                                     (arc-drawer #'clim-internals::standard-arc-drawer)
+                                     arc-drawing-options
                                      graph-type (move-cursor t)
                                 &allow-other-keys)
   (declare (ignore orientation generation-separation within-generation-separation center-nodes))
   ;; Mungle some arguments
   (check-type cutoff-depth (or null integer))
+  (check-type root-objects sequence)
   (setf stream (or stream *standard-output*)
         graph-type (or graph-type (if merge-duplicates :digraph :tree))
         duplicate-key (or duplicate-key #'identity)
         duplicate-test (or duplicate-test #'eql) )
+
+  ;; I'm not sure what to do here.  Saying you want a tree, but want
+  ;; duplicates merged seems wrong.  OTOH, if you go out of your way
+  ;; to do it, at your own risk, is it our place to say "no"?
+  ;; [2005/08/11:rpg]
+;;;  (when (and (eq graph-type :tree) merge-duplicates)
+;;;    (cerror "Substitute NIL for merge-duplicates"
+;;;	    "Merge duplicates specified to be true when using :tree layout.")
+;;;    (setf merge-duplicates nil))
   
   ;; clean the options
   (remf graph-options :stream)
@@ -160,8 +169,10 @@
                     #'cont
                     (find-graph-type graph-type)
 		    nil
-                    :hash-table (make-hash-table :test duplicate-test)
-                    graph-options))))
+		    ;; moved to local variable... [2005/07/25:rpg]
+                    ;; :hash-table (make-hash-table :test duplicate-test)
+		    graph-options 
+		    ))))
       (setf (output-record-position graph-output-record)
             (values cursor-old-x cursor-old-y))
       (with-output-recording-options (stream :draw t :record nil)
@@ -179,35 +190,40 @@
 
 (defclass standard-graph-output-record (graph-output-record
                                         standard-sequence-output-record)
-  ((orientation
-    :initarg :orientation
-    :initform :horizontal)
-   (center-nodes
-    :initarg :center-nodes
-    :initform nil)
-   (cutoff-depth
-    :initarg :cutoff-depth
-    :initform nil)
-   (merge-duplicates
-    :initarg :merge-duplicates
-    :initform nil)
-   (generation-separation
-    :initarg :generation-separation
-    :initform '(4 :character))
-   (within-generation-separation
-    :initarg :within-generation-separation
-    :initform '(1/2 :line))
-   (hash-table
-    :initarg :hash-table
-    :initform nil)
-   (root-nodes
-    :accessor graph-root-nodes) ))
+     ((orientation
+       :initarg :orientation
+       :initform :horizontal)
+      (center-nodes
+       :initarg :center-nodes
+       :initform nil)
+      (cutoff-depth
+       :initarg :cutoff-depth
+       :initform nil)
+      (merge-duplicates
+       :initarg :merge-duplicates
+       :initform nil)
+      (generation-separation
+       :initarg :generation-separation
+       :initform '(4 :character))
+      (within-generation-separation
+       :initarg :within-generation-separation
+       :initform '(1/2 :line))
+      ;; removed HASH-TABLE slot and stuffed it into
+      ;; GENERATE-GRAPH-NODES method definition [2005/07/25:rpg]
+      (root-nodes
+       :accessor graph-root-nodes)
+      ))
 
 (defclass tree-graph-output-record (standard-graph-output-record)
-  ())
+     ())
+
+;;;(defmethod initialize-instance :after ((obj tree-graph-output-record) &key merge-duplicates)
+;;;  (when merge-duplicates
+;;;    (warn "Cannot use a TREE layout for graphs while merging duplicates.")))
 
 (defclass dag-graph-output-record (standard-graph-output-record)
-  ())
+  (
+   ))
 
 (defclass digraph-graph-output-record (standard-graph-output-record)
   ())
@@ -235,45 +251,64 @@
 
 ;;;;
 
+;;; Modified to make this obey the spec better by using a hash-table
+;;; for detecting previous nodes only when the duplicate-test argument
+;;; permits it.  [2005/08/10:rpg]
 (defmethod generate-graph-nodes ((graph-output-record standard-graph-output-record)
                                  stream root-objects
                                  object-printer inferior-producer
                                  &key duplicate-key duplicate-test)
-  (declare (ignore duplicate-test))
-  (with-slots (cutoff-depth merge-duplicates hash-table) graph-output-record
-    (labels
-        ((traverse-objects (node objects depth)
-           (unless (and cutoff-depth (>= depth cutoff-depth))
-             (remove nil
-                     (map 'list
-                          (lambda (child)
-                            (let* ((key (funcall duplicate-key child))
-                                   (child-node (and merge-duplicates
-                                                    (gethash key hash-table))))
-                              (cond (child-node
-                                     (when node
-                                       (push node (graph-node-parents child-node)))
-                                     child-node)
-                                    (t
-                                     (let ((child-node
-                                            (with-output-to-output-record
-                                                (stream 'standard-graph-node-output-record new-node
-                                                        :object child)
-                                              (funcall object-printer child stream))))
-                                       (when merge-duplicates
-                                         (setf (gethash key hash-table) child-node))
-                                       (when node
-                                         (push node (graph-node-parents child-node)))
-                                       (setf (graph-node-children child-node)
-                                             (traverse-objects child-node
-                                                              (funcall inferior-producer child)
-                                                              (+ depth 1)))
-                                       child-node)))))
-                          objects)))))
-      ;;
-      (setf (graph-root-nodes graph-output-record)
-            (traverse-objects nil root-objects 0))
-      (values))))
+  (with-slots (cutoff-depth merge-duplicates) graph-output-record
+    (let* ((hash-table (when (and merge-duplicates (member duplicate-test (list #'eq #'eql #'equal #'equalp)))
+			 (make-hash-table :test duplicate-test)))
+	   node-list 
+	   (hashed hash-table))
+      (labels
+	  ((previous-node (obj)
+	     ;; is there a previous node for obj?  if so, return it.
+	     (when merge-duplicates
+	       (if hashed
+		   (locally (declare (type hash-table hash-table))
+		     (gethash obj hash-table))
+		 (cdr (assoc obj node-list :test duplicate-test)))))
+	   ((setf previous-node) (val obj)
+	     (if hashed
+		 (locally (declare (type hash-table hash-table))
+		   (setf (gethash obj hash-table) val))
+		 (setf node-list (push (cons obj val) node-list))))
+	   (traverse-objects (node objects depth)
+	     (unless (and cutoff-depth (>= depth cutoff-depth))
+	       (remove nil
+		       (map 'list
+			 (lambda (child)
+			   (let* ((key (funcall duplicate-key child))
+				  (child-node (previous-node key)))
+			     (cond (child-node
+				    (when node
+				      (push node (graph-node-parents child-node)))
+				    child-node)
+				   (t
+				    (let ((child-node
+					   (with-output-to-output-record
+					       (stream 'standard-graph-node-output-record new-node
+						       :object child)
+					     (funcall object-printer child stream))))
+				      (when merge-duplicates
+					(setf (previous-node key) child-node)
+					;; (setf (gethash key hash-table) child-node)
+					)
+				      (when node
+					(push node (graph-node-parents child-node)))
+				      (setf (graph-node-children child-node)
+					(traverse-objects child-node
+							  (funcall inferior-producer child)
+							  (+ depth 1)))
+				      child-node)))))
+			 objects)))))
+	;;
+	(setf (graph-root-nodes graph-output-record)
+	  (traverse-objects nil root-objects 0))
+	(values)))))
 
 (defun traverse-graph-nodes (graph continuation)
   ;; continuation: node x children x cont -> some value
@@ -297,6 +332,8 @@
                                                        (:horizontal :vertical)
                                                        (:vertical :horizontal))))
           (generation-separation (parse-space stream generation-separation orientation)))
+      ;; generation sizes is an adjustable array that tracks the major
+      ;; dimension of each of the generations [2005/07/18:rpg]
       (let ((generation-sizes (make-array 10 :adjustable t :initial-element 0)))
         (labels ((node-major-dimension (node)
                    (if (eq orientation :vertical)
@@ -306,10 +343,13 @@
                    (if (eq orientation :vertical)
                        (bounding-rectangle-width node)
                        (bounding-rectangle-height node)))
+		 ;; WALK returns a node minor dimension for the node,
+		 ;; AFAICT, allowing space for that node's children
+		 ;; along the minor dimension. [2005/07/18:rpg]
                  (walk (node depth)
                    (unless (graph-node-minor-size node)
                      (when (>= depth (length generation-sizes))
-                       (setf generation-sizes (adjust-array generation-sizes (ceiling (* depth 1.2)))))
+                       (setf generation-sizes (adjust-array generation-sizes (ceiling (* depth 1.2)) :initial-element 0)))
                      (setf (aref generation-sizes depth)
                            (max (aref generation-sizes depth) (node-major-dimension node)))
                      (setf (graph-node-minor-size node) 0)
@@ -336,8 +376,8 @@
                                 (let ((v (+ v0 (/ (min 0 d) -2))))
                                   (setf (output-record-position node)
                                         (if (eq orientation :vertical)
-                                            (values v u0)
-                                            (values u0 v)))
+                                            (transform-position (medium-transformation stream) v u0)
+                                            (transform-position (medium-transformation stream) u0 v)))
                                   (add-output-record node graph-output-record))
                                 ;;
                                 (let ((u (+ u0 (car majors)))
@@ -365,6 +405,121 @@
                                (incf v within-generation-separation)))
                            (graph-root-nodes graph-output-record)))))))))))
 
+
+(defmethod layout-graph-nodes ((graph-output-record dag-graph-output-record)
+                               stream arc-drawer arc-drawing-options)
+  "This is a first shot at a DAG layout.  First does a TOPO sort that associates 
+each node with a depth, then lays out by depth.  Tries to reuse a maximum of the
+tree graph layout code.
+PRECONDITION:  This code assumes that we have generated only nodes up to the 
+cutoff-depth.  GENERATE-GRAPH-NODES seems to obey this precondition."
+  (declare (ignore arc-drawer arc-drawing-options))
+  (with-slots (orientation center-nodes generation-separation within-generation-separation root-nodes
+			   merge-duplicates) graph-output-record
+    ;; this code is snarly enough, handling merge-duplicates.  If
+    ;; you're not merging duplicates, you're out of luck, at least for
+    ;; now... [2005/07/18:rpg]
+    (unless merge-duplicates
+      (cerror "Set to T and continue?" "DAG graph-layout type only supports merge-duplicates to be T")
+      (setf merge-duplicates t))
+	      
+    (check-type orientation (member :horizontal :vertical)) ;xxx move to init.-inst.
+
+    ;; here major dimension is the dimension in which we grow the
+    ;; tree.
+    (let ((within-generation-separation (parse-space stream within-generation-separation
+                                                     (case orientation
+                                                       (:horizontal :vertical)
+                                                       (:vertical :horizontal))))
+          (generation-separation (parse-space stream generation-separation orientation)))
+      ;; generation sizes is an adjustable array that tracks the major
+      ;; dimension of each of the generations [2005/07/18:rpg]
+      (let ((generation-sizes (make-array 10 :adjustable t :initial-element 0))
+	    (visited (make-hash-table :test #'eq))
+	    (parent-hash (make-hash-table :test #'eq)))
+        (labels ((node-major-dimension (node)
+                   (if (eq orientation :vertical)
+                       (bounding-rectangle-height node)
+                       (bounding-rectangle-width node)))
+                 (node-minor-dimension (node)
+                   (if (eq orientation :vertical)
+                       (bounding-rectangle-width node)
+                       (bounding-rectangle-height node)))
+		 ;; WALK returns a node minor dimension for the node,
+		 ;; AFAICT, allowing space for that node's children
+		 ;; along the minor dimension. [2005/07/18:rpg]
+                 (walk (node depth &optional parent)
+		   (unless (gethash node visited)
+		     (setf (gethash node visited) depth)
+		     (when parent
+		       (setf (gethash node parent-hash) parent))
+		     (unless (graph-node-minor-size node)
+		       (when (>= depth (length generation-sizes))
+			 (setf generation-sizes (adjust-array generation-sizes (ceiling (* depth 1.2))
+							      :initial-element 0)))
+		       (setf (aref generation-sizes depth)
+			     (max (aref generation-sizes depth) (node-major-dimension node)))
+		       (setf (graph-node-minor-size node) 0)
+		       (max (node-minor-dimension node)
+			    (setf (graph-node-minor-size node)
+				  (let ((sum 0) (n 0))
+				    (map nil (lambda (child)
+					       (let ((x (walk child (+ depth 1) node)))
+						 (when x
+						   (incf sum x)
+						   (incf n))))
+					 (graph-node-children node))
+				    (+ sum
+				       (* (max 0 (- n 1)) within-generation-separation)))))))))
+          (map nil #'(lambda (x) (walk x 0)) root-nodes)
+          (let ((hash (make-hash-table :test #'eq)))
+            (labels ((foo (node majors u0 v0)
+                       (cond ((gethash node hash)
+                              v0)
+                             (t
+                              (setf (gethash node hash) t)
+                              (let ((d (- (node-minor-dimension node)
+                                          (graph-node-minor-size node))))
+                                (let ((v (+ v0 (/ (min 0 d) -2))))
+                                  (setf (output-record-position node)
+                                        (if (eq orientation :vertical)
+                                            (transform-position (medium-transformation stream) v u0)
+                                            (transform-position (medium-transformation stream) u0 v)))
+                                  (add-output-record node graph-output-record))
+                                ;;
+                                (let ((u (+ u0 (car majors)))
+                                      (v (+ v0 (max 0 (/ d 2))))
+                                      (firstp t))
+                                  (map nil (lambda (q)
+                                             (unless (gethash q hash)
+                                               (if firstp
+                                                   (setf firstp nil)
+                                                   (incf v within-generation-separation))
+                                               (setf v (foo q (cdr majors)
+                                                            u v))))
+				       ;; when computing the sizes, to
+				       ;; make the tree-style layout
+				       ;; work, we have to have each
+				       ;; node have a unique
+				       ;; parent. [2005/07/18:rpg]
+				       (remove-if-not #'(lambda (x) (eq (gethash x parent-hash) node))
+						      (graph-node-children node))))
+                                ;;
+                                (+ v0 (max (node-minor-dimension node)
+                                           (graph-node-minor-size node))))))))
+              ;;
+              (let ((majors (mapcar (lambda (x) (+ x generation-separation))
+                                    (coerce generation-sizes 'list))))
+                (let ((u (+ 0 (car majors)))
+                      (v 0))
+                  (maplist (lambda (rest)
+                             (setf v (foo (car rest) majors u v))
+                             (unless (null rest)
+                               (incf v within-generation-separation)))
+                           (graph-root-nodes graph-output-record)))))))))))
+
+
+
 #+ignore
 (defmethod layout-graph-edges ((graph-output-record standard-graph-output-record)
                                stream arc-drawer arc-drawing-options)
@@ -399,6 +554,11 @@
 (defmethod layout-graph-edges ((graph standard-graph-output-record)
                                stream arc-drawer arc-drawing-options)
   (with-slots (orientation) graph
+   ;; We tranformed the position of the nodes when we inserted them into
+   ;; output history, so the bounding rectangles queried below will be
+   ;; transformed. Therefore, disable the transformation now, otherwise
+   ;; the transformation is effectively applied twice to the edges.
+   (with-identity-transformation (stream)
     (traverse-graph-nodes graph
                           (lambda (node children continuation)
                             (unless (eq node graph)
@@ -422,7 +582,7 @@
                                                 (/ (+ x1 x2) 2) from
                                                 (/ (+ u1 u2) 2) to
                                                 arc-drawing-options))))))))
-                            (map nil continuation children)))))
+                            (map nil continuation children))))))
 
 (defmethod layout-graph-edges :around ((graph-output-record tree-graph-output-record)
                                        stream arc-drawer arc-drawing-options)
