@@ -124,9 +124,9 @@ spatially organized data structure.
 	  ((and (symbolp map-test-func) (symbolp func)) ; not eq
 	   nil)
 	  ((and (symbolp map-test-func) (fboundp map-test-func))
-	   (eq (symbol-value map-test-func) func))
+	   (eq (symbol-function map-test-func) func))
 	  ((and (symbolp func) (fboundp func))
-	   (eq map-test-func (symbol-value func)))
+	   (eq map-test-func (symbol-function func)))
 	  (t nil))))
 
 (defun ensure-test (map test)
@@ -134,6 +134,15 @@ spatially organized data structure.
     (explode-map-hash map)
     (setf (tester-function map) :mismatch)))
 
+(defgeneric clear-map (map))
+
+(defmethod clear-map ((map updating-output-map-mixin))
+  (setf (id-map map) nil)
+  (setf (id-counter map) 0)
+  (setf (element-count map) 0))
+
+;;; Perhaps these should be generic functions, but in the name of premature
+;;; optimization they're not :)
 (defun get-from-map (map value test)
   (when (eq (tester-function map) 'none)
     (return-from get-from-map nil))
@@ -203,6 +212,9 @@ spatially organized data structure.
     (when deleted
       (decf (element-count map)))))
 
+;;; Reset the ID counter so that updating output records without explicit IDs
+;;; can be assigned one during a run of the code. I'm not sure about using
+;;; reinitialize-instance for this...
 (defmethod shared-initialize :after ((obj updating-output-map-mixin) slot-names
 				     &key)
   (declare (ignore slot-names))
@@ -233,8 +245,12 @@ spatially organized data structure.
 (defmethod pane-needs-redisplay :around ((pane updating-output-stream-mixin))
   (let ((redisplayp (call-next-method)))
     (values redisplayp (and (not (eq redisplayp :no-clear))
-			    (or (not (pane-incremental-redisplay pane))
-				(not *enable-updating-output*))))))
+			    (not (pane-incremental-redisplay pane))))))
+
+(defmethod window-clear :after ((pane updating-output-stream-mixin))
+  "Get rid of any updating output records stored in the stream; they're gone
+  from the screen."
+  (clear-map pane))
 
 ;;; INCREMENTAL-DISPLAY takes as input the difference set computed by
 ;;; COMPUTE-DIFFERENCE-SET and updates the screen. The 5 kinds of updates are
@@ -263,36 +279,33 @@ spatially organized data structure.
 (defgeneric incremental-redisplay
     (stream position erases moves draws erase-overlapping move-overlapping))
 
-(defmethod incremental-redisplay
-    ((stream updating-output-stream-mixin) position
-     erases moves draws erase-overlapping move-overlapping)
+(defmethod incremental-redisplay ((stream updating-output-stream-mixin) position
+                                  erases moves draws erase-overlapping move-overlapping)
   (declare (ignore position))
   (let ((history (stream-output-history stream)))
     (with-output-recording-options (stream :record nil :draw t)
       (loop
-	 for (nil br) in erases
-	 do (erase-rectangle stream br))
+          for (nil br) in erases
+          do (erase-rectangle stream br))
       (loop
-	 for (nil old-bounding) in moves
-	 do (erase-rectangle stream old-bounding))
+          for (nil old-bounding) in moves
+          do (erase-rectangle stream old-bounding))
       (loop
-	 for (nil br) in erase-overlapping
-	 do (erase-rectangle stream br))
+          for (nil br) in erase-overlapping
+          do (erase-rectangle stream br))
       (loop
-	 for (nil old-bounding) in move-overlapping
-	 do (erase-rectangle stream old-bounding)))
+          for (nil old-bounding) in move-overlapping
+          do (erase-rectangle stream old-bounding)))
     (loop
-       for (r) in moves
-       do (replay r stream))
+        for (r) in moves
+        do (replay r stream))
     (loop
-       for (r) in draws
-       do (replay r stream))
-    (loop
-       for (r) in erase-overlapping
-       do (replay history stream r))
-    (loop
-       for (r) in move-overlapping
-	do (replay history stream r) )))
+        for (r) in draws
+        do (replay r stream))
+    (let ((res +nowhere+))
+      (loop for (r) in erase-overlapping do (setf res (region-union res r)))
+      (loop for (r) in move-overlapping do (setf res (region-union res r)))
+      (replay history stream res))))
 
 (defclass updating-stream-state (complete-medium-state)
   ((cursor-x :accessor cursor-x :initarg :cursor-x :initform 0)
@@ -335,12 +348,10 @@ spatially organized data structure.
 ;;;programmer forcing all new output.
 
 (defun state-matches-stream-p (state stream)
-  (multiple-value-bind (cx cy)
-      (stream-cursor-position stream)
+  (multiple-value-bind (cx cy) (stream-cursor-position stream)
     (with-sheet-medium (medium stream)
-      (match-output-records state :cursor-x cx :cursor-y cy))))
-
-(define-protocol-class updating-output-record (output-record))
+      ;; Note: We don't match the y coordinate.
+      (match-output-records state :cursor-x cx))))
 
 (defclass updating-output-record-mixin (updating-output-map-mixin
 					standard-sequence-output-record)
@@ -385,6 +396,10 @@ that are children (but not necessarily direct) of this record.")
 updating-output-parent above this one in the tree.")
    ;; Results of (setf output-record-position) while updating
    (explicit-moves :accessor explicit-moves)
+   (old-bounds :accessor old-bounds
+	       :initform (make-bounding-rectangle 0.0d0 0.0d0 0.0d0 0.0d0)
+	       :documentation "Holds the old bounds of an updating output
+ record if that can no longer be determined from the old-children.")
    ;; on-screen state?
    ))
 
@@ -502,11 +517,15 @@ updating-output-parent above this one in the tree.")
 (defmethod compute-new-output-records ((record standard-updating-output-record)
 				       stream)
   (with-output-recording-options (stream :record t :draw nil)
-    (map-over-updating-output #'(lambda (r)
-				  (setf (old-children r) (sub-record r))
-				  (setf (output-record-dirty r) :updating))
-			      record
-			      nil)
+    (map-over-updating-output
+     #'(lambda (r)
+         (let ((sub-record (sub-record r)))
+           (setf (old-children r) sub-record)
+           (setf (output-record-dirty r) :updating)
+           (setf (rectangle-edges* (old-bounds r))
+                 (rectangle-edges* sub-record))))
+     record
+     nil)
     (finish-output stream)
     ;; Why is this binding here? We need the "environment" in this call that
     ;; computes the new records of an outer updating output record to resemble
@@ -528,8 +547,9 @@ updating-output-parent above this one in the tree.")
     ((record standard-updating-output-record) stream displayer)
   (multiple-value-bind (x y)
       (output-record-position record)
-    (when (sub-record record)
-      (delete-output-record (sub-record record) record))
+    (let ((sub-record (sub-record record)))
+      (when sub-record
+        (delete-output-record sub-record record)))
     ;; Don't add this record repeatedly to a parent updating-output-record.
     (unless (eq (output-record-parent record)
 		(stream-current-output-record stream))
@@ -543,45 +563,6 @@ updating-output-parent above this one in the tree.")
 				      &rest initargs
 				      &key unique-id unique-id-test))
 
-(defgeneric find-equal-display-record (root use-old-elements record))
-
-(defmethod find-equal-display-record ((root standard-updating-output-record)
-				      use-old-elements
-				      record)
-  (cond ((eq (output-record-dirty root) :clean)
-	 nil)
-	(use-old-elements
-	 (when (slot-boundp root 'old-children)
-	   (find-equal-display-record (old-children root)
-				      use-old-elements
-				      record)))
-	(t (find-equal-display-record (sub-record root)
-				      use-old-elements
-				      record))))
-
-(defmethod find-equal-display-record ((root compound-output-record)
-				      use-old-elements
-				      record)
-  (when (region-intersects-region-p root record)
-    (flet ((mapper (r)
-	     (let ((result (find-equal-display-record r
-						      use-old-elements
-						      record)))
-	       (when result
-		 (return-from find-equal-display-record result)))))
-      (declare (dynamic-extent #'mapper))
-      (map-over-output-records-overlapping-region #'mapper root record)))
-  nil)
-
-
-(defmethod find-equal-display-record ((root displayed-output-record)
-				      use-old-elements
-				      record)
-  (declare (ignore use-old-elements))
-  (if (output-record-equal root record)
-      root
-      nil))
-
 (defgeneric map-over-displayed-output-records
     (function root use-old-elements clean clip-region)
   (:documentation "Call function on all displayed-output-records in ROOT's
@@ -728,156 +709,167 @@ records. "))
 ;;; work in progress
 (defvar *existing-output-records* nil)
 
-;;; Helper functions for managing a hash table of records
+;;;
+(defgeneric output-record-hash (record)
+  (:documentation "Produce a value that can be used to hash the output record
+in an equalp hash table"))
 
-(defun get-record-hash (record hash)
-  (let ((bucket (gethash (slot-value record 'coordinates) hash)))
-    (if (null bucket)
-	(values nil nil)
-	(let ((rec (find record bucket :test #'output-record-equal)))
-	  (if rec
-	      (values rec t)
-	      (values nil nil))))))
+(defmethod output-record-hash ((record standard-bounding-rectangle))
+  (slot-value record 'coordinates))
 
-(defun add-record-hash (record hash)
-  (push record (gethash (slot-value record 'coordinates) hash nil)))
+(defconstant +fixnum-bits+ (integer-length most-positive-fixnum))
 
-(defun delete-record-hash (record hash)
-  (let ((bucket (gethash (slot-value record 'coordinates) hash)))
-    (if bucket
-	(multiple-value-bind (new-bucket deleted)
-	    (delete-1 record bucket :test #'output-record-equal)
-	  (if deleted
-	      (progn
-		(setf (gethash (slot-value record 'coordinates) hash)
-		      new-bucket)
-		t)
-	      nil))
-	nil)))
+(declaim (inline hash-coords))
+(defun hash-coords (x1 y1 x2 y2)
+  (declare (type coordinate x1 y1 x2 y2))
+  (let ((hash-val 0))
+      (declare (type fixnum hash-val))
+      (labels ((rot4 (val)
+		 (dpb (ldb (byte 4 0) val)
+		      (byte 4 (- +fixnum-bits+ 4 1))
+		      (ash val -4)))
+	       (mix-it-in (val)
+	       (let ((xval (sxhash val)))
+		 (declare (type fixnum xval))
+		 (when (minusp val)
+		   (setq xval (rot4 xval)))
+		 (setq hash-val (logxor (rot4 hash-val) xval)))))
+	(declare (inline rot4 mix-it-in))
+	(mix-it-in x1)
+	(mix-it-in y1)
+	(mix-it-in x2)
+	(mix-it-in y2)
+	hash-val)))
+
+(defmethod output-record-hash ((record output-record))
+  (with-bounding-rectangle* (x1 y1 x2 y2)
+      record
+    (hash-coords x1 y1 x2 y2)))
 
 (defmethod compute-difference-set ((record standard-updating-output-record)
 				   &optional (check-overlapping t)
-				   offset-x offset-y
-				   old-offset-x old-offset-y)
+                                             offset-x offset-y
+                                             old-offset-x old-offset-y)
   (declare (ignore offset-x offset-y old-offset-x old-offset-y))
-  (when (eq (output-record-dirty record) :clean)
-    (return-from compute-difference-set (values nil nil nil nil nil)))
-  (let* ((draws nil)
-	 (moves (explicit-moves record))
-	 (erases nil)
-	 (erase-overlapping nil)
-	 (move-overlapping nil)
-	 (stream (updating-output-stream record))
-	 (visible-region (pane-viewport-region stream))
-	 (old-children (if (slot-boundp record 'old-children)
-			   (old-children record)
-			   nil)))
-    (unless (or (null visible-region)
-		(region-intersects-region-p visible-region record)
-		(and old-children
-		     (region-intersects-region-p visible-region old-children)))
-      (return-from compute-difference-set (values nil nil nil nil nil)))
-    ;; XXX This means that compute-difference-set can't be called repeatedly on
-    ;; the same tree; ugh. On the other hand, if we don't clear explicit-moves,
-    ;; they can hang around in the tree for later passes and cause trouble.
-    (setf (explicit-moves record) nil)
-    (let ((existing-output-records (make-hash-table :test 'equalp)))
-      ;; Find output records in the new tree that match a record in the old
-      ;; tree i.e., already have a valid display on the screen.
-      (map-over-child-display
-       (if old-children
-	   #'(lambda (r)
-	       (add-record-hash r existing-output-records))
-	   #'(lambda (r) (push (list r r) draws)))
-       (sub-record record)
-       visible-region)
-      (when old-children
-	(map-over-child-display
-	 #'(lambda (r)
-	     (unless (delete-record-hash r existing-output-records)
-	       (push (list r (copy-bounding-rectange r)) erases)))
-	 old-children
-	 visible-region)
-	;; Any records left in the hash table do not have a counterpart
-	;; visible on the screen and need to be drawn.
-	(loop
-	   for bucket being the hash-values of existing-output-records
-	   do (loop
-		 for r in bucket
-		 do (push (list r r) draws)))))
-    (when check-overlapping
-      (setf erase-overlapping (nconc erases draws))
-      (setf move-overlapping moves)
-      (setf erases nil)
-      (setf moves nil)
-      (setf draws nil))
-    ;; Visit this record's updating-output children and merge in the
-    ;; difference set. We need to visit all updating-output records, not just
-    ;; ones in the visible region, because they might have old records that
-    ;; lie in the visible region and that need to be erased.
-    (map-over-child-updating-output
-     #'(lambda (r)
-	 (multiple-value-bind (e m d e-o m-o)
-	     (compute-difference-set r check-overlapping)
-	   (setf erases (nconc e erases))
-	   (setf moves (nconc m moves))
-	   (setf draws (nconc d draws))
-	   (setf erase-overlapping (nconc e-o erase-overlapping))
-	   (setf move-overlapping (nconc m-o move-overlapping))))
-     (sub-record record)
-     nil)
-    ;; Look for updating-output children that were not visited. Their
-    ;; display records need to be erased.
-    (when old-children
-      (flet ((erase-obsolete (dr)		;All of them
-	       (let ((erase-chunk (list dr (copy-bounding-rectange dr))))
-		 (if check-overlapping
-		     (push erase-chunk erase-overlapping)
-		     (push erase-chunk erases)))))
-	(declare (dynamic-extent #'erase-obsolete))
-	(map-over-child-updating-output
-	 #'(lambda (r)
-	     (when (eq (output-record-dirty r) :updating)
-	       (map-over-obsolete-display #'erase-obsolete
-					  (sub-record r)
-					  visible-region)))
-	 old-children
-	 visible-region)))
-    ;; Traverse all the display records for this updating output node and do
-    ;; the notes...
-    (flet ((note-got (r)
-	     (note-output-record-got-sheet r stream))
-	   (note-lost (r)
-	     (note-output-record-lost-sheet r stream)))
-      (declare (dynamic-extent #'note-got #'note-lost))
-      (map-over-child-display #'note-got (sub-record record) nil)
-      (when old-children
-	(map-over-child-display #'note-lost old-children nil)
-	(map-over-child-updating-output
-	 #'(lambda (r)
-	     (when (eq (output-record-dirty r) :updating)
-	       (map-over-obsolete-display #'note-lost
-					  (sub-record r)
-					  nil)))
-	 old-children
-	 nil)))
-    (values erases moves draws erase-overlapping move-overlapping)))
-
-(defparameter *enable-updating-output* t
-  "Switch to turn on incremental redisplay")
+  ;; (declare (values erases moves draws erase-overlapping move-overlapping))
+  (let (was
+        is
+        (everywhere (or +everywhere+
+                        (pane-viewport-region (updating-output-stream record)))))
+    ;; Collect what was there
+    (labels ((gather-was (record)
+               (cond ((displayed-output-record-p record)
+                      (push record was))
+                     ((updating-output-record-p record)
+                      (cond ((eq :clean (output-record-dirty record))
+                             (push record was))
+                            ((eq :moved (output-record-dirty record))
+                             (push (slot-value record 'old-bounds) was))
+                            (t
+                             (map-over-output-records-overlapping-region #'gather-was
+                                                                         (old-children record)
+                                                                         everywhere))))
+                     (t
+                      (map-over-output-records-overlapping-region #'gather-was record everywhere)) )))
+      (gather-was record))
+    ;; Collect what still is there
+    (labels ((gather-is (record)
+               (cond ((displayed-output-record-p record)
+                      (push record is))
+                     ((updating-output-record-p record)
+                      (cond ((eq :clean (output-record-dirty record))
+                             (push record is))
+                            ((eq :moved (output-record-dirty record))
+                             (push record is))
+                            (t
+                             (map-over-output-records-overlapping-region #'gather-is
+                                                                         (sub-record record)
+                                                                         everywhere))))
+                     (t
+                      (map-over-output-records-overlapping-region #'gather-is record everywhere) ))))
+      (gather-is record))
+    ;;
+    (let ((was-table (make-hash-table :test #'equalp))
+          (is-table (make-hash-table :test #'equalp))
+          gone
+          stay
+          come)
+      (loop for w in was do (push w (gethash (output-record-hash w) was-table)))
+      (loop for i in is do (push i (gethash (output-record-hash i) is-table)))
+      ;; gone = was \ is
+      (loop for w in was do
+            (cond ((updating-output-record-p w)
+                   (unless (eq :clean (output-record-dirty w))
+                     (push (old-children w) gone)))
+                  (t
+                   (let ((q (gethash (output-record-hash w) is-table)))
+                     (unless (some #'(lambda (x) (output-record-equal w x)) q)
+                       (push w gone))))))
+      ;; come = is \ was
+      ;; stay = is ^ was
+      (loop for i in is do
+            (cond ((updating-output-record-p i)
+                   (if (eq :clean (output-record-dirty i))
+                       (push i stay)
+                       (push i come)))
+                  (t
+                   (let ((q (gethash (output-record-hash i) was-table)))
+                     (if (some #'(lambda (x) (output-record-equal i x)) q)
+                         (push i stay)
+                         (push i come))))))
+      ;; Now we essentially want 'gone', 'stay', 'come'
+      (let ((gone-overlap nil)
+            (come-overlap nil))
+        (when check-overlapping
+          (setf (values gone gone-overlap)
+                (loop for k in gone
+                      if (some (lambda (x) (region-intersects-region-p k x))
+                               stay)
+                      collect k into gone-overlap*
+                      else collect k into gone*
+                      finally (return (values gone* gone-overlap*))))
+          (setf (values come come-overlap)
+                (loop for k in come
+                      if (some (lambda (x) (region-intersects-region-p k x))
+                               stay)
+                      collect k into come-overlap*
+                      else collect k into come*
+                      finally (return (values come* come-overlap*)))))
+        ;; Hmm, we somehow miss come-overlap ...
+        (values
+         ;; erases
+         (loop for k in gone collect (list k k))
+         ;; moves
+         nil
+         ;; draws
+         (loop for k in come collect (list k k))
+         ;; erase overlapping
+         (append (loop for k in gone-overlap collect (list k k))
+                 (loop for k in come-overlap collect (list k k)))
+         ;; move overlapping
+         nil)))))
 
 (defvar *trace-updating-output* nil)
 
 (defvar *no-unique-id* (cons nil nil))
 
+(defun move-output-record (record dx dy)
+  (multiple-value-bind (sx sy) (output-record-start-cursor-position record)
+    (multiple-value-bind (ex ey) (output-record-end-cursor-position record)
+      (setf (output-record-position record)
+            (values (+ (nth-value 0 (output-record-position record)) dx)
+                    (+ (nth-value 1 (output-record-position record)) dy)))
+      (setf (output-record-start-cursor-position record)
+            (values (+ sx dx) (+ sy dy)))
+      (setf (output-record-end-cursor-position record)
+            (values (+ ex dx) (+ ey dy))))))
+
 (defmethod invoke-updating-output ((stream updating-output-stream-mixin)
 				   continuation
 				   record-type
 				   unique-id id-test cache-value cache-test
 				   &key (fixed-position nil) (all-new nil)
 				   (parent-cache nil))
-  (unless *enable-updating-output*
-    (return-from invoke-updating-output (funcall continuation stream)))
   (finish-output stream)
   (let ((parent-cache (or parent-cache *current-updating-output* stream)))
     (when (eq unique-id *no-unique-id*)
@@ -909,16 +901,10 @@ records. "))
 	       (setf (end-graphics-state record)
 		     (medium-graphics-state stream))
 	       (add-to-map parent-cache record  unique-id id-test all-new)))
-	    ((or (setq state-mismatch
-		       (not (state-matches-stream-p (start-graphics-state
-						     record)
-						    stream)))
-		 (not (funcall cache-test
-			       cache-value
-			       (output-record-cache-value record))))
+	    ((or (setq state-mismatch (not (state-matches-stream-p (start-graphics-state record) stream)))
+		 (not (funcall cache-test cache-value (output-record-cache-value record))))
 	     (when *trace-updating-output*
-	       (format *trace-output* "~:[cache test~;stream state~] ~S~%"
-		       state-mismatch record))
+	       (format *trace-output* "~:[cache test~;stream state~] ~S~%" state-mismatch record))
 	     (let ((*current-updating-output* record))
 	       (setf (start-graphics-state record)
 		     (medium-graphics-state stream))
@@ -930,18 +916,36 @@ records. "))
 	    (t
 	     ;; It doesn't need to be updated, but it does go into the
 	     ;; parent's sequence of records
-	     (when *trace-updating-output*
-	       (format *trace-output* "clean ~S~%" record))
-	     (setf (output-record-dirty record) :clean)
-	     (setf (output-record-parent record) nil)
-	     (map-over-updating-output #'(lambda (r)
-					   (setf (output-record-dirty r)
-						 :clean))
-				       record
-				       nil)
-	     (add-output-record record (stream-current-output-record stream))
-	     (set-medium-graphics-state (end-graphics-state record) stream)
-	     (setf (parent-cache record) parent-cache)))
+             ;;
+             (multiple-value-bind (cx cy) (stream-cursor-position stream)
+               (multiple-value-bind (sx sy) (output-record-start-cursor-position record)
+                 (let ((dx (- cx sx))
+                       (dy (- cy sy)))
+                   (unless (zerop dy)
+                     (move-output-record record dx dy) )
+                   (let ((tag (cond
+				((= dx dy 0)
+				 (when *trace-updating-output*
+				   (format *trace-output* "clean ~S~%" record))
+				 :clean)
+				(t
+				 (when *trace-updating-output*
+				   (format *trace-output* "moved ~S~%" record))
+				 :moved))))
+                     (setf (output-record-dirty record) tag)
+                     (setf (output-record-parent record) nil)
+                     (map-over-updating-output #'(lambda (r)
+                                                   (unless (eq r record)
+                                                     (incf (slot-value (start-graphics-state r) 'cursor-x) dx)
+                                                     (incf (slot-value (start-graphics-state r) 'cursor-y) dy)
+                                                     (incf (slot-value (end-graphics-state r) 'cursor-x) dx)
+                                                     (incf (slot-value (end-graphics-state r) 'cursor-y) dy))
+                                                   (setf (output-record-dirty r) tag))
+                                               record
+                                               nil)
+                     (add-output-record record (stream-current-output-record stream))
+                     (set-medium-graphics-state (end-graphics-state record) stream)
+                     (setf (parent-cache record) parent-cache) )) ))))
       record)))
 
 ;;; The Franz user guide says that updating-output does
@@ -986,16 +990,6 @@ records. "))
 
 (defvar *dump-updating-output* nil)
 
-;;; Protocol for notifying that records have been added or deleted,
-;;; for real, during redisplay.
-
-(defgeneric redisplay-add-output-record (record stream)
-  (:documentation "Process an output record that has been added (i.e.,
-  was not in the output history of the stream) during redisplay. The
-  record has not been displayed yet."))
-
-(defgeneric redisplay-delete-output-record (record bounding-rectangle stream))
-
 (defgeneric redisplay-output-record (record stream
 				     &optional check-overlapping))
 
@@ -1027,30 +1021,6 @@ records. "))
 	     (delete-stale-updating-output record))
 	(set-medium-graphics-state current-graphics-state stream)))))
 
-(defmethod redisplay-add-output-record (record
-					(stream updating-output-stream-mixin))
-  (with-bounding-rectangle* (x1 y1 x2 y2)
-    record
-    (draw-rectangle* stream x1 y1 x2 y2
-		     :ink +background-ink+)))
-
-(defmethod redisplay-add-output-record :after
-    (record (stream updating-output-stream-mixin))
-  (note-output-record-got-sheet record stream))
-
-(defmethod redisplay-delete-output-record
-    (record bounding-rectangle (stream updating-output-stream-mixin))
-  (declare (ignore record))
-  (with-bounding-rectangle* (x1 y1 x2 y2)
-    bounding-rectangle
-    (draw-rectangle* stream x1 y1 x2 y2
-		     :ink +background-ink+)))
-
-(defmethod redisplay-delete-output-record :after
-    (record bounding-rectangle (stream updating-output-stream-mixin))
-  (declare (ignore bounding-rectangle))
-  (note-output-record-lost-sheet record stream))
-
 (defun erase-rectangle (stream bounding)
   (with-bounding-rectangle* (x1 y1 x2 y2)
       bounding
@@ -1239,7 +1209,7 @@ records. "))
     ((frame application-frame) (pane updating-output-stream-mixin) &key force-p)
   (setf (id-counter pane) 0)
   (let ((incremental-redisplay (pane-incremental-redisplay pane)))
-    (cond ((or (not incremental-redisplay) (not *enable-updating-output*))
+    (cond ((not incremental-redisplay)
 	   (call-next-method))
 	  ((or (null (updating-record pane))
 	       force-p)
