@@ -61,26 +61,16 @@
 
 (in-package :clim-internals)
 
-;; This must come early, because of implementation quirks:
-
-(define-protocol-class medium ()
-  ())
-
 ;;;;
 ;;;; 11 Text Styles
 ;;;;
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
 
-(define-protocol-class text-style ()
-  ())
-
 (defgeneric text-style-components (text-style))
 (defgeneric text-style-family (text-style))
 (defgeneric text-style-face (text-style))
 (defgeneric text-style-size (text-style))
-#+unicode
-(defgeneric text-style-language (text-style))
 (defgeneric merge-text-styles (text-style-1 text-style-2))
 (defgeneric text-style-ascent (text-style medium))
 (defgeneric text-style-descent (text-style medium))
@@ -89,6 +79,7 @@
 (defgeneric text-style-fixed-width-p (text-style medium))
 
 (defgeneric text-style-equalp (style1 style2))
+(defmethod text-style-equalp ((style1 text-style) (style2 text-style)) nil)
 
 (defclass standard-text-style (text-style)
   ((family   :initarg :text-family
@@ -99,24 +90,13 @@
 	     :reader text-style-face)
    (size     :initarg :text-size
 	     :initform :normal
-	     :reader text-style-size)
-   #+unicode
-   (language :initarg  :text-language
-	     :initform nil
-	     :reader   text-style-language)))
+	     :reader text-style-size)))
 
-#-unicode
 (defmethod make-load-form ((obj standard-text-style) &optional env)
   (declare (ignore env))
   (with-slots (family face size) obj
     `(make-text-style ',family ',face ',size)))
 
-#+unicode
-(defmethod make-load-form ((obj standard-text-style) &optional env)
-  (declare (ignore env))
-  (with-slots (family face size language) obj
-    `(make-text-style ',family ',face ',size ',language)))
-
 (defun family-key (family)
   (ecase family
     ((nil) 0)
@@ -148,29 +128,14 @@
 	((:smaller)    8)
 	((:larger)     9))))
 
-#+unicode
-(defun language-key (language)
-  (ecase language
-    ((:english nil) 0)
-    ((:korean)      1)))
-
-#-unicode
 (defun text-style-key (family face size)
   (+ (* 256 (size-key size))
      (* 16 (face-key face))
      (family-key family)))
 
-#+unicode
-(defun text-style-key (family face size &optional (language nil))
-  (+ (ash (size-key size)         12)
-     (ash (language-key language)  8)
-     (ash (face-key face)          4)
-     (ash (family-key family)      0)))
-
 (eval-when (:compile-toplevel :load-toplevel :execute)
   (defvar *text-style-hash-table* (make-hash-table :test #'eql)))
 
-#-unicode
 (defun make-text-style (family face size)
   (let ((key (text-style-key family face size)))
     (declare (type fixnum key))
@@ -181,20 +146,9 @@
 			     :text-face face
 			     :text-size size)))))
 
-#+unicode
-(defun make-text-style (family face size &optional language)
-  (let ((key (text-style-key family face size language)))
-    (declare (type fixnum key))
-    (or (gethash key *text-style-hash-table*)
-	(setf (gethash key *text-style-hash-table*)
-	      (make-instance 'standard-text-style
-			     :text-family family
-			     :text-face face
-			     :text-size size
-                             :text-language language)))))
 ) ; end eval-when
 
-(defmethod print-object ((self text-style) stream)
+(defmethod print-object ((self standard-text-style) stream)
   (print-unreadable-object (self stream :type t :identity nil)
     (format stream "~{~S~^ ~}" (multiple-value-list (text-style-components self)))))
 
@@ -202,9 +156,7 @@
 			      (style2 standard-text-style))
   (and (eql (text-style-family style1) (text-style-family style2))
        (eql (text-style-face style1) (text-style-face style2))
-       (eql (text-style-size style1) (text-style-size style2))
-       #+unicode (eql (text-style-language style1)
-		      (text-style-language style2))))
+       (eql (text-style-size style1) (text-style-size style2))))
 
 (defconstant *default-text-style* (make-text-style :fix :roman :normal))
 (defconstant *undefined-text-style* *default-text-style*)
@@ -232,22 +184,30 @@
 (defmethod text-style-components ((text-style standard-text-style))
   (values (text-style-family   text-style)
           (text-style-face     text-style)
-          (text-style-size     text-style)
-          #+unicode
-          (text-style-language text-style)))
+          (text-style-size     text-style)))
 
 ;;; Device-Font-Text-Style class
 
 (defclass device-font-text-style (text-style)
-  ())
+  ((display-device :initarg :display-device :accessor display-device)
+   (device-font-name :initarg :device-font-name :accessor device-font-name)))
+
+(defmethod print-object ((self device-font-text-style) stream)
+  (print-unreadable-object (self stream :type t :identity nil)
+    (format stream "~S on ~S" (device-font-name self) (display-device self))))
 
 (defun device-font-text-style-p (s)
   (typep s 'device-font-text-style))
 
+(defmethod text-style-equalp ((style1 device-font-text-style) (style2 device-font-text-style))
+  (eq style1 style2))
+
 (defmethod text-style-mapping ((port basic-port) text-style
                                &optional character-set)
   (declare (ignore character-set))
-  (gethash (parse-text-style text-style) (port-text-style-mappings port)))
+  (if (keywordp text-style)
+      (gethash (parse-text-style text-style) (port-text-style-mappings port))
+      (gethash text-style (port-text-style-mappings port))))
 
 (defmethod (setf text-style-mapping) (mapping (port basic-port)
                                       text-style
@@ -264,17 +224,17 @@
   (setf (gethash text-style (port-text-style-mappings port))
         mapping))
 
-(defun make-device-font-text-style (port font-name)
+(defgeneric make-device-font-text-style (port font-name))
+
+(defmethod make-device-font-text-style (port font-name)
   (let ((text-style (make-instance 'device-font-text-style
-                                   :text-family font-name
-                                   :text-face nil
-                                   :text-size nil)))
+				   :display-device port
+				   :device-font-name font-name)))
     (setf (text-style-mapping port text-style) font-name)
     text-style))
 
 ;;; Text-style utilities
 
-#-unicode
 (defmethod merge-text-styles (s1 s2)
   (setq s1 (parse-text-style s1))
   (setq s2 (parse-text-style s2))
@@ -296,31 +256,6 @@
         (make-text-style family face size))
       s1))
 
-#+unicode
-(defmethod merge-text-styles (s1 s2)
-  (setq s1 (parse-text-style s1))
-  (setq s2 (parse-text-style s2))
-  (if (and (not (device-font-text-style-p s1))
-	   (not (device-font-text-style-p s2)))
-      (let* ((family (or (text-style-family s1) (text-style-family s2)))
-             (face1 (text-style-face s1))
-             (face2 (text-style-face s2))
-             (face (if (subsetp '(:bold :italic) (list face1 face2))
-                       '(:bold :italic)
-                       (or face1 face2)))
-             (size1 (text-style-size s1))
-             (size2 (text-style-size s2))
-             (size (case size1
-                     ((nil) size2)
-                     (:smaller (find-smaller-size size2))
-                     (:larger (find-larger-size size2))
-                     (t size1)))
-             ; v- this is probably wrong, but it requires an idea of which
-             ; languages include which foreign language support.
-             (language (or (text-style-language s1) (text-style-language s2))))
-        (make-text-style family face size language))
-      s1))
-
 (defun parse-text-style (style)
   (cond ((text-style-p style) style)
         ((null style) (make-text-style nil nil nil)) ; ?
@@ -334,7 +269,7 @@
   (check-type medium symbol)
   (with-gensyms (cont)
     `(flet ((,cont (,medium)
-              (declare (ignorable ,medium))
+              ,(declare-ignorable-form* medium)
               ,@body))
        (declare (dynamic-extent #',cont))
        (invoke-with-text-style ,medium #',cont
@@ -363,7 +298,7 @@
     (setq medium '*standard-output*))
   (with-gensyms (cont)
     `(flet ((,cont (,medium)
-              (declare (ignorable ,medium))
+              ,(declare-ignorable-form* medium)
               ,@body))
        (declare (dynamic-extent #',cont))
        (invoke-with-text-style ,medium #',cont
@@ -375,7 +310,7 @@
     (setq medium '*standard-output*))
   (with-gensyms (cont)
     `(flet ((,cont (,medium)
-              (declare (ignorable ,medium))
+              ,(declare-ignorable-form* medium)
               ,@body))
        (declare (dynamic-extent #',cont))
        (invoke-with-text-style ,medium #',cont
@@ -386,24 +321,12 @@
   (when (eq medium t) (setq medium '*standard-output*))
   (with-gensyms (cont)
     `(flet ((,cont (,medium)
-              (declare (ignorable ,medium))
+              ,(declare-ignorable-form* medium)
               ,@body))
        (declare (dynamic-extent #',cont))
        (invoke-with-text-style ,medium #',cont
                                (make-text-style nil nil ,size)))))
 
-#+unicode
-(defmacro with-text-language ((medium language) &body body)
-  (declare (type symbol medium))
-  (when (eq medium t) (setq medium '*standard-output*))
-  (with-gensyms (cont)
-    `(flet ((,cont (,medium)
-              (declare (ignorable ,medium))
-              ,@body))
-       (declare (dynamic-extent #',cont))
-       (invoke-with-text-style ,medium #',cont
-                               (make-text-style nil nil nil ,language)))))
-
 
 ;;; MEDIUM class
 
@@ -516,8 +439,6 @@
 
 ;;; Line-Style class
 
-(define-protocol-class line-style ())
-
 (defgeneric line-style-equalp (arg1 arg2))
 
 (defclass standard-line-style (line-style)
@@ -585,7 +506,7 @@
        (eql (line-style-thickness style1) (line-style-thickness style2))
        (eql (line-style-joint-shape style1) (line-style-joint-shape style2))
        (eql (line-style-cap-shape style1) (line-style-cap-shape style2))
-       (eql (line-style-thickness style1) (line-style-thickness style2))))
+       (eql (line-style-dashes style1) (line-style-dashes style2))))
 
 ;;; Misc ops
 
@@ -828,7 +749,7 @@
   "Macro for optimizing drawing with graphical system dependant mechanisms."
   (with-gensyms (fn)
     `(flet ((,fn (,medium)
-              (declare (ignorable ,medium))
+              ,(declare-ignorable-form* medium)
               ,@body))
        (declare (dynamic-extent #',fn))
        (invoke-with-special-choices #',fn ,medium))))
