@@ -280,24 +280,23 @@
     (with-gensyms (record-arg continuation)
       (with-keywords-removed (key-args (:record-type
 					:allow-sensitive-inferiors))
-	`(let ((*allow-sensitive-inferiors*
-		(if *allow-sensitive-inferiors*
-		    ,allow-sensitive-inferiors
-		    nil)))
-	   (flet ((,continuation ()
-		    ,@decls
-		    ,@with-body))
-	     (declare (dynamic-extent #'continuation))
-	     (if (and (output-recording-stream-p ,stream)
-		      *allow-sensitive-inferiors*)
-		 (with-new-output-record
-		     (,stream ,record-type ,record-arg
-			      :object ,object
-			      :type (expand-presentation-type-abbreviation
-				     ,type)
-			      ,@key-args)
-		   (,continuation))
-		 (,continuation))))))))
+	`(flet ((,continuation ()
+		  ,@decls
+		  ,@with-body))
+	   (declare (dynamic-extent #'continuation))
+	   (if (and (output-recording-stream-p ,stream)
+		    *allow-sensitive-inferiors*)
+	       (with-new-output-record
+		   (,stream ,record-type ,record-arg
+			    :object ,object
+			    :type (expand-presentation-type-abbreviation
+				   ,type)
+			    ,@key-args)
+		 (let ((*allow-sensitive-inferiors*
+			,allow-sensitive-inferiors))
+		   (,continuation)))
+	       (,continuation)))))))
+
 
 (defun present (object &optional (type (presentation-type-of object))
 		&key
@@ -337,9 +336,10 @@
 			   (allow-sensitive-inferiors t)
 			   (sensitive t)
 			   (record-type 'standard-presentation))
-  (let ((*allow-sensitive-inferiors* (if *allow-sensitive-inferiors*
-					 sensitive
-					 nil)))
+  ;; *allow-sensitive-inferiors* controls whether or not
+  ;; with-output-as-presentation will emit a presentation
+  (let ((*allow-sensitive-inferiors* (and *allow-sensitive-inferiors*
+					  sensitive)))
     (with-output-as-presentation (stream object type
 				  :view view
 				  :modifier modifier
@@ -664,71 +664,74 @@ call-next-method to get the \"real\" answer based on the stream type."))
 		   display-default query-identifier
 		   activation-gestures additional-activation-gestures
 		   delimiter-gestures additional-delimiter-gestures))
-  (let* ((real-type (expand-presentation-type-abbreviation type))
-	 (real-default-type (cond (default-type-p
-				      (expand-presentation-type-abbreviation
-				       default-type))
-				  ((or defaultp provide-default)
-				   real-type)
-				  (t nil)))
-	 (real-history-type (cond ((null historyp) real-type)
-				  ((null history) nil)
-				  (t (expand-presentation-type-abbreviation
-				      history))))
-	 (*recursive-accept-p* *recursive-accept-1-p*)
-	 (*recursive-accept-1-p* t))
-    (with-keywords-removed (rest-args (:stream))
-      (when (or default-type-p defaultp)
-	(setf rest-args
-	      (list* :default-type real-default-type rest-args)))
-      (when historyp
-	(setf rest-args (list* :history real-history-type rest-args)))
-      (cond ((and viewp (symbolp view))
-	     (setf rest-args
-		   (list* :view (funcall #'make-instance view) rest-args)))
-	    ((consp view)
-	     (setf rest-args
-		   (list* :view (apply #'make-instance view) rest-args))))
-      ;; Presentation type history interaction. According to the spec,
-      ;; if provide-default is true, we take the default from the
-      ;; presentation history. In addition, we'll implement the Genera
-      ;; behavior of temporarily putting the default on the history
-      ;; stack so the user can conveniently suck it in.
-      (flet ((do-accept (args)
-	       (apply #'stream-accept stream real-type args))
-	     (get-history ()
-	       (when real-history-type
-		 (funcall-presentation-generic-function
-		  presentation-type-history-for-stream
-		  real-history-type stream))))
-	(let* ((default-from-history (and (not defaultp) provide-default))
-	       (history (get-history))
-	       (results
-		(multiple-value-list 
-		 (if history
-		     (let ((*active-history-type* real-history-type))
-		       (cond (defaultp
-			      (with-object-on-history
-				  (history default real-default-type)
-				(do-accept rest-args)))
-			     (default-from-history
-			      (multiple-value-bind
-				    (history-default history-type)
-				  (presentation-history-head history
-							     real-default-type)
-				(do-accept (if history-type
-					       (list* :default history-default
-						      :default-type history-type
-						      rest-args)
-					       rest-args))))
-			     (t (do-accept rest-args))))
-		     (do-accept rest-args))))
-	       (results-history (get-history)))
-	  (when results-history
-	    (presentation-history-add results-history
-				      (car results)
-				      (cadr results)))
-	  (values-list results))))))
+  (handler-bind ((abort-gesture (lambda (condition)
+                                  (signal condition) ;; to give outer handlers a chance to say "I know how to handle this"
+                                  (abort condition))))
+    (let* ((real-type (expand-presentation-type-abbreviation type))
+           (real-default-type (cond (default-type-p
+                                     (expand-presentation-type-abbreviation
+                                      default-type))
+                                    ((or defaultp provide-default)
+                                     real-type)
+                                    (t nil)))
+           (real-history-type (cond ((null historyp) real-type)
+                                    ((null history) nil)
+                                    (t (expand-presentation-type-abbreviation
+                                        history))))
+           (*recursive-accept-p* *recursive-accept-1-p*)
+           (*recursive-accept-1-p* t))
+      (with-keywords-removed (rest-args (:stream))
+        (when (or default-type-p defaultp)
+          (setf rest-args
+                (list* :default-type real-default-type rest-args)))
+        (when historyp
+          (setf rest-args (list* :history real-history-type rest-args)))
+        (cond ((and viewp (symbolp view))
+               (setf rest-args
+                     (list* :view (funcall #'make-instance view) rest-args)))
+              ((consp view)
+               (setf rest-args
+                     (list* :view (apply #'make-instance view) rest-args))))
+        ;; Presentation type history interaction. According to the spec,
+        ;; if provide-default is true, we take the default from the
+        ;; presentation history. In addition, we'll implement the Genera
+        ;; behavior of temporarily putting the default on the history
+        ;; stack so the user can conveniently suck it in.
+        (flet ((do-accept (args)
+                 (apply #'stream-accept stream real-type args))
+               (get-history ()
+                 (when real-history-type
+                   (funcall-presentation-generic-function
+                    presentation-type-history-for-stream
+                    real-history-type stream))))
+          (let* ((default-from-history (and (not defaultp) provide-default))
+                 (history (get-history))
+                 (results
+                  (multiple-value-list 
+                   (if history
+                       (let ((*active-history-type* real-history-type))
+                         (cond (defaultp
+                                (with-object-on-history
+                                    (history default real-default-type)
+                                  (do-accept rest-args)))
+                               (default-from-history
+                                (multiple-value-bind
+                                      (history-default history-type)
+                                    (presentation-history-head history
+                                                               real-default-type)
+                                  (do-accept (if history-type
+                                                 (list* :default history-default
+                                                        :default-type history-type
+                                                        rest-args)
+                                                 rest-args))))
+                               (t (do-accept rest-args))))
+                       (do-accept rest-args))))
+                 (results-history (get-history)))
+            (when results-history
+              (presentation-history-add results-history
+                                        (car results)
+                                        (cadr results)))
+            (values-list results)))))))
 
 (defgeneric stream-accept (stream type
 			   &key
@@ -807,7 +810,7 @@ call-next-method to get the \"real\" answer based on the stream type."))
 		 (additional-activation-gestures nil additional-activations-p)
 		 (delimiter-gestures nil delimitersp)
 		 (additional-delimiter-gestures nil  additional-delimiters-p))
-  (declare (ignore provide-default insert-default history active-p
+  (declare (ignore provide-default history active-p
 		   prompt prompt-mode
 		   display-default query-identifier))
   (when (and defaultp (not default-type-p))
@@ -827,6 +830,13 @@ call-next-method to get the \"real\" answer based on the stream type."))
 				 (declare (ignore stream))
 				 (funcall cont))))
       (with-input-position (stream)	; support for calls to replace-input
+        (when insert-default
+          ;; Insert the default value to the input stream. It should
+          ;; become fully keyboard-editable.
+          (presentation-replace-input stream
+                                       default
+                                       default-type
+                                       view))
 	(setf (values sensitizer-object sensitizer-type)
 	      (with-input-context (type)
 		  (object object-type event options)
@@ -843,10 +853,10 @@ call-next-method to get the \"real\" answer based on the stream type."))
 			  (setq accept-results
 				(multiple-value-list
 				 (if defaultp
-				     (funcall-presentation-generic-function
-				      accept type stream view
-				      :default default
-				      :default-type default-type)
+                                     (funcall-presentation-generic-function
+                                      accept type stream view
+                                      :default default
+                                      :default-type default-type)
 				     (funcall-presentation-generic-function
 				      accept type stream view))))
 			;; User entered activation or delimeter
@@ -894,6 +904,7 @@ call-next-method to get the \"real\" answer based on the stream type."))
 			    &key
 			    (default nil defaultp)
 			    (default-type type)
+                            (insert-default nil)
 			    (prompt t)
 			    (prompt-mode :normal)
 			    (display-default prompt)
@@ -911,9 +922,12 @@ call-next-method to get the \"real\" answer based on the stream type."))
 				     *recursive-accept-p*
 				     (describe-presentation-type type nil nil))
 			     prompt))
-	  (default-string (if (and defaultp display-default)
-			      (present-to-string default default-type)
-			      nil)))
+          ;; Don't display the default in the prompt if it is to be
+          ;; inserted into the input stream.
+	  (default-string (and defaultp
+				(not insert-default)
+				display-default
+				(present-to-string default default-type))))
       (cond ((null prompt)
 	   nil)
 	  (t
@@ -927,17 +941,49 @@ call-next-method to get the \"real\" answer based on the stream type."))
   (declare (ignore type view other-args))
   nil)
 
+;;; XXX This needs work! It needs to do everything that accept does for
+;;; expanding ptypes and setting up recursive call procesusing
 (defun accept-from-string (type string
 			   &rest args
 			   &key view
-			   default
-			   default-type
+			   (default nil defaultp)
+			   (default-type nil default-type-p)
+			   activation-gestures
+			   (additional-activation-gestures
+			    nil
+			    additional-activations-p)
+			   delimiter-gestures
+			   (additional-delimiter-gestures
+			    nil
+			    additional-delimiters-p)
 			   (start 0)
 			   (end (length string)))
-  (declare (ignore view default default-type))
-  (with-input-from-string (stream string :start start :end end)
-    (with-keywords-removed (args (:start :end))
-      (apply #'stream-accept stream type :view +textual-view+ args))))
+  (declare (ignore view))
+  ;;; XXX work in progress here.
+  (with-activation-gestures ((if additional-activations-p
+				 additional-activation-gestures
+				 activation-gestures)
+			     :override activationsp)
+    (with-delimiter-gestures ((if additional-delimiters-p
+				  additional-delimiter-gestures
+				  delimiter-gestures)
+			      :override delimitersp)))
+  (when (or (zerop (- end start))
+	    (let ((maybe-end))))
+    (if defaultp
+	(return-from accept-from-string (values default
+						(if default-type-p
+						    default-type
+						    type)
+						0))
+	(simple-parse-error "Empty string")))
+  (let ((index 0))
+    (multiple-value-bind (val ptype)
+	(with-input-from-string (stream string :start start :end end
+				 :index index)
+	  (with-keywords-removed (args (:start :end))
+	    (apply #'stream-accept stream type :view +textual-view+ args)))
+      (values val ptype index))))
 
 (define-presentation-generic-function %presentation-refined-position-test
     presentation-refined-position-test
@@ -966,7 +1012,7 @@ call-next-method to get the \"real\" answer based on the stream type."))
     (multiple-value-bind (min-x min-y max-x max-y)
 	(output-record-hit-detection-rectangle* record)
       (if (and (<= min-x x max-x) (<= min-y y max-y))
-	  (if (or (null single-box) (eq single-box :higlighting))
+	  (if (or (null single-box) (eq single-box :highlighting))
 	      (funcall-presentation-generic-function
 	       presentation-refined-position-test
 	       (presentation-type record) record x y)
@@ -1950,80 +1996,92 @@ call-next-method to get the \"real\" answer based on the stream type."))
 ;;; drag-n-drop fun
 
 (defclass drag-n-drop-translator (presentation-translator)
-  ((feedback :reader feedback :initarg :feedback)
-   (highlighting :reader highlighting :initarg :highlighting)))
+  ((destination-ptype :reader destination-ptype :initarg :destination-ptype)
+   (feedback :reader feedback :initarg :feedback)
+   (highlighting :reader highlighting :initarg :highlighting)
+   (destination-translator :reader destination-translator
+			   :initarg :destination-translator)))
 
+
+(defvar *dragged-presentation* nil
+  "Bound to the presentation dragged in a drag-and-drop context")
 (defvar *dragged-object* nil
   "Bound to the object dragged in a drag-and-drop context")
 
+()
 ;;; According to the Franz User's guide, the destination object is
 ;;; available in the tester, documentation, and translator function
 ;;; as destination-object. Therefore OBJECT is the dragged object. In
 ;;; our scheme the tester function, translator function etc. is
 ;;; really called on the destination object. So, we do a little
-;;; shuffling of arguments here.
+;;; shuffling of arguments here. We don't do that for the destination
+;;; translator because we can call that ourselves in frame-drag-and-drop.
+;;;
+;;; Also, in Classic CLIM the destination presentation is passed as a
+;;; destination-presentation keyword argument; hence the presentation argument
+;;; is the dragged presentation.
 
 (defmethod initialize-instance :after ((obj drag-n-drop-translator)
-				       &key tester documentation
+				       &key documentation
 				       pointer-documentation
-				       translator-function)
+				       destination-translator)
+  ;; This is starting to smell...
   (flet ((make-adapter (func)
-	   (lambda (object &rest args)
-	     (apply func *dragged-object* :destination-object object args))))
-    (setf (slot-value obj 'tester) (make-adapter tester))
+	   (lambda (object &rest args &key presentation &allow-other-keys)
+	     (if *dragged-presentation*
+		 (apply func
+			*dragged-object*
+			:presentation *dragged-presentation*
+			:destination-object object
+			:destination-presentation presentation
+			args)
+		 (apply func object args)))))
     (setf (slot-value obj 'documentation) (make-adapter documentation))
     (when pointer-documentation
       (setf (slot-value obj 'pointer-documentation)
-	    (make-adapter pointer-documentation)))
-    (setf (slot-value obj 'translator-function)
-	  (make-adapter translator-function))))
-
-(define-presentation-type drag-over (over context)
-  :inherit-from t
-  :parameters-are-types t)
-
-(define-presentation-method presentation-subtypep ((type drag-over)
-						   maybe-supertype)
-  (with-presentation-type-parameters (drag-over type)
-    (let ((subtype-over under)
-	  (subtype-context context))
-      (with-presentation-type-parameters (drag-over maybe-supertype)
-	(and (presentation-subtypep subtype-over over)
-	     (presentation-subtypep subtype-over over))))))
+	    (make-adapter pointer-documentation)))))
 
 (defmacro define-drag-and-drop-translator
-    (name
-     (from-type to-type destination-type command-table
-      &rest args
-      &key (gesture :select) tester documentation
-      pointer-documentation menu priority
-      (feedback 'frame-drag-and-drop-feedback)
-      (highlighting 'frame-drag-and-drop-highlighting))
+   (name (from-type to-type destination-type command-table
+           &rest args &key
+           (gesture :select)
+           (tester 'default-translator-tester)
+           documentation
+           (pointer-documentation nil pointer-doc-p)
+           (menu t)
+           (priority 0)
+           (feedback 'frame-drag-and-drop-feedback)
+           (highlighting 'frame-drag-and-drop-highlighting))
      arglist
      &body body)
-  (declare (ignore tester documentation pointer-documentation menu
-		   priority))
-  (let* ((real-from-type (expand-presentation-type-abbreviation from-type))
-	 (real-dest-type (expand-presentation-type-abbreviation
+  (declare (ignore tester gesture documentation pointer-documentation
+		   menu priority))
+  (let* ((real-dest-type (expand-presentation-type-abbreviation
 			  destination-type))
-	 (real-to-type (expand-presentation-type-abbreviation to-type))
-	 (action-name (gensym (format nil "~S-~S-DRAG-ACTION"
-				      from-type to-type))))
-    (with-keywords-removed (args (:feedback :highlighting))
+	 (name-string (command-name-from-symbol name))
+	 (drag-string (format nil "Drag to ~A" name-string))
+	 (pointer-doc (if pointer-doc-p
+			  nil
+			  `(:pointer-documentation
+			    ((object destination-object stream)
+			     (declare (ignore object))
+			     (write-string (if destination-object
+					       ,name-string
+					       ,drag-string)
+			                   stream))))))
+        (with-keywords-removed (args (:feedback :highlighting))
       `(progn
 	 (define-presentation-translator ,name
-	     (,real-dest-type (drag-over ,real-from-type ,real-to-type)
+	     (,from-type ,to-type ,command-table
+	      :tester-definitive t
 	      ,@args
+	      ,@pointer-doc
 	      :feedback #',feedback :highlighting #',highlighting
+	      :destination-ptype ',real-dest-type
+	      :destination-translator #',(make-translator-fun arglist body)
 	      :translator-class drag-n-drop-translator)
-	   ,arglist
-	   ,@body)
-	 (define-presentation-action ,action-name
-	     (,from-type ,to-type ,command-table :gesture ,gesture
-			 :priority -1)
-	   (object presentation context-type frame event window x y)
-	   (frame-drag ',name ',command-table object presentation context-type
-		       frame event window x y))))))
-
-
+	   (presentation context-type frame event window x y)
+	   (frame-drag-and-drop ',name ',command-table
+				presentation context-type
+				frame event window x y))))))
 
