@@ -25,9 +25,6 @@
 
 ;;; PRESENTATION class
 
-(define-protocol-class presentation ())
-
-
 (defvar *allow-sensitive-inferiors* t)
 
 (defclass presentation-mixin (presentation)
@@ -913,6 +910,7 @@ suitable for SUPER-NAME"))
 
 (defvar *standard-object-class* (find-class 'standard-object))
 
+#-scl
 (defmethod clim-mop:compute-applicable-methods-using-classes :around
     ((gf presentation-generic-function) classes)
   (multiple-value-bind (methods success)
@@ -927,7 +925,24 @@ suitable for SUPER-NAME"))
 				     *standard-object-class*))
 			     methods)
 		  t)))))
-  
+
+#+scl
+(defmethod clim-mop:compute-applicable-methods-using-classes :around
+    ((gf presentation-generic-function) classes)
+  (multiple-value-bind (methods success non-class-positions)
+      (call-next-method)
+    (let ((ptype-class (car classes)))
+      (if (or (null success)
+	      (not (typep ptype-class 'presentation-type-class)))
+	  (values methods non-class-positions non-class-positions)
+	  (values (remove-if #'(lambda (method)
+				 (eq (car (clim-mop:method-specializers
+					   method))
+				     *standard-object-class*))
+			     methods)
+		  t
+		  non-class-positions)))))
+
 (defun method-applicable (method arguments)
   (loop for arg in arguments
 	for specializer in (clim-mop:method-specializers method)
@@ -1213,6 +1228,9 @@ and used to ensure that presentation-translators-caches are up to date.")
     :writer (setf presentation-translators-cache)
     :initform (make-hash-table :test #'equal))))
 
+(defun invalidate-translator-caches ()
+  (incf *current-translator-cache-generation*))
+
 (defmethod presentation-translators-cache ((table translator-table))
   (with-slots ((cache presentation-translators-cache)
 	       (generation translator-cache-generation))
@@ -1254,10 +1272,11 @@ and used to ensure that presentation-translators-caches are up to date.")
 	      (remove old
 		      (gethash (presentation-type-name (from-type old))
 			       simple-type-translators))))
-      (incf *current-translator-cache-generation*)
+      (invalidate-translator-caches)
       (setf (gethash (name translator) translators) translator)
       (push translator
-	    (gethash (from-type translator) simple-type-translators)))))
+	    (gethash (from-type translator) simple-type-translators))
+      translator)))
 
 (defun make-translator-fun (args body)
   (multiple-value-bind (ll ignore)
@@ -1286,7 +1305,7 @@ and used to ensure that presentation-translators-caches are up to date.")
 	   (gesture :select)
 	   (tester 'default-translator-tester testerp)
 	   (tester-definitive (if testerp nil t))
-	   (documentation nil)
+	   (documentation nil documentationp)
 	   (pointer-documentation nil pointer-documentation-p)
 	   (menu t)
 	   (priority 0)
@@ -1320,7 +1339,10 @@ and used to ensure that presentation-translators-caches are up to date.")
 							      (cdr tester)))
 			:tester-definitive ',tester-definitive
 			:documentation #',(make-documentation-fun
-					   documentation)
+					   (if documentationp
+					       documentation
+					       (command-name-from-symbol
+						name)))
 			,@(when pointer-documentation-p
 				`(:pointer-documentation
 				  #',(make-documentation-fun
@@ -1335,7 +1357,7 @@ and used to ensure that presentation-translators-caches are up to date.")
     (name (from-type to-type command-table &key
 	   (gesture :select)
 	   (tester 'default-translator-tester)
-	   (documentation nil)
+	   (documentation nil documentationp)
 	   (pointer-documentation nil pointer-documentation-p)
 	   (menu t)
 	   (priority 0))
@@ -1357,7 +1379,11 @@ and used to ensure that presentation-translators-caches are up to date.")
 		    `',tester
 		    `#',(make-translator-fun (car tester)
 					     (cdr tester)))
-       :documentation #',(make-documentation-fun documentation)
+       :tester-definitive t
+       :documentation #',(make-documentation-fun (if documentationp
+						     documentation
+						     (command-name-from-symbol
+						      name)))
        ,@(when pointer-documentation-p
 	       `(:pointer-documentation
 		 #',(make-documentation-fun pointer-documentation)))
@@ -1499,17 +1525,20 @@ and used to ensure that presentation-translators-caches are up to date.")
 (defmethod call-presentation-translator
     ((translator presentation-translator) presentation context-type
      frame event window x y)
-  (multiple-value-bind (object ptype options)
-      (funcall (translator-function translator)
-	       (presentation-object presentation)
-	       :presentation presentation
-	       :context-type context-type
-	       :frame frame
-	       :event event
-	       :window window
-	       :x x
-	       :y y)
-    (values object (or ptype context-type) options)))
+  ;; Let the translator return an explict ptype of nil to, in effect, abort the
+  ;; presentation throw.
+  (multiple-value-call
+      #'(lambda (object &optional (ptype context-type) options)
+	  (values object ptype options))
+    (funcall (translator-function translator)
+	     (presentation-object presentation)
+	     :presentation presentation
+	     :context-type context-type
+	     :frame frame
+	     :event event
+	     :window window
+	     :x x
+	     :y y)))
 
 (defmethod call-presentation-translator
     ((translator presentation-action) presentation context-type
@@ -1597,8 +1626,6 @@ and used to ensure that presentation-translators-caches are up to date.")
 				       x y)
 				      context-type))
 	(return-from test-presentation-translator nil))))
-  
-  
   t)
 
 ;;; presentation-contains-position moved to presentation-defs.lisp
@@ -1660,9 +1687,14 @@ and used to ensure that presentation-translators-caches are up to date.")
 		  presentation
 		  x y)))))
 
+(defun window-modifier-state (window)
+  "Provides default modifier state for presentation translator functions."
+  (let ((pointer (port-pointer (port window))))
+    (pointer-modifier-state pointer)))
+
 (defun find-applicable-translators
     (presentation input-context frame window x y
-     &key event (modifier-state 0) for-menu fastp)
+     &key event (modifier-state (window-modifier-state window)) for-menu fastp)
   (let ((results nil))
     (flet ((fast-func (translator presentation context)
 	     (declare (ignore translator presentation context))
@@ -1750,7 +1782,9 @@ and used to ensure that presentation-translators-caches are up to date.")
 
 (defun find-innermost-applicable-presentation
     (input-context window x y
-     &key (frame *application-frame*) modifier-state event)
+     &key (frame *application-frame*)
+     (modifier-state (window-modifier-state window))
+     event)
   (values (find-innermost-presentation-match input-context
                                              (stream-output-history window)
                                              frame
@@ -1760,12 +1794,13 @@ and used to ensure that presentation-translators-caches are up to date.")
                                              modifier-state
 					     nil)))
 
-(defun find-innermost-presentation-context (input-context window x y
-					    &key
-					    (top-record
-					     (stream-output-history window))
-					    (frame *application-frame*)
-					    event modifier-state button)
+(defun find-innermost-presentation-context
+    (input-context window x y
+     &key (top-record (stream-output-history window))
+     (frame *application-frame*)
+     event
+     (modifier-state (window-modifier-state window))
+     button)
   (find-innermost-presentation-match input-context
 				     top-record
 				     frame
