@@ -69,12 +69,6 @@
 
 (in-package :clim-internals)
 
-(define-protocol-class output-record (bounding-rectangle)
-  ())
-
-(define-protocol-class displayed-output-record (output-record)
-  ())
-
 ;;; 16.2.1. The Basic Output Record Protocol
 #+:cmu(declaim (ftype (function (output-record) (values rational rational))
 		      output-record-position))
@@ -203,12 +197,7 @@ unspecified. "))
 (defgeneric tree-recompute-extent (record))
 
 ;;; 16.3. Types of Output Records
-(define-protocol-class graphics-displayed-output-record
-    (displayed-output-record)
-  ())
 
-(define-protocol-class text-displayed-output-record (displayed-output-record)
-  ())
 
 ;;; 16.3.3. Text Displayed Output Record
 (defgeneric add-character-output-to-text-record
@@ -220,8 +209,6 @@ unspecified. "))
 (defgeneric text-displayed-output-record-string (text-record))
 
 ;;; 16.4. Output Recording Streams
-(define-protocol-class output-recording-stream ()
-  ())
 
 ;;; 16.4.1. The Output Recording Stream Protocol
 (defgeneric stream-recording-p (stream))
@@ -374,7 +361,7 @@ unspecified. "))
   (setq stream (stream-designator-symbol stream '*standard-output*))
   (with-gensyms (continuation)
     `(flet ((,continuation  (,stream)
-	      (declare (ignorable ,stream))
+	      ,(declare-ignorable-form* stream)
 	      ,@body))
        (declare (dynamic-extent #',continuation))
        (invoke-with-output-recording-options
@@ -400,7 +387,7 @@ unspecified. "))
 	    (flet ((,constructor ()
 		     (make-instance ,record-type ,@m-i-args))
 		   (,continuation (,stream ,record)
-		     (declare (ignorable ,stream ,record))
+		     ,(declare-ignorable-form* stream record)
 		     ,@body))
 	      (declare (dynamic-extent #'constructor #'continuation))
 	      (,',func-name ,stream #',continuation ,record-type #',constructor
@@ -444,8 +431,7 @@ recording stream. If it is T, *STANDARD-OUTPUT* is used.")
 
 (defmethod initialize-instance :after ((record basic-output-record)
 				       &key (x-position 0.0d0)
-				       (y-position 0.0d0))
-  (declare (ignore args))
+                                            (y-position 0.0d0))
   (setf (rectangle-edges* record)
 	(values x-position y-position x-position y-position)))
 
@@ -935,7 +921,6 @@ the associated sheet can be determined."
 (defmethod map-over-output-records-1
     (function (record standard-sequence-output-record) function-args)
   "Applies FUNCTION to all children in the order they were added."
-  (declare (ignore x-offset y-offset))
   (if function-args
       (loop with children = (output-record-children record)
 	 for child across children
@@ -973,10 +958,118 @@ were added."
      when (region-intersects-region-p region child)
      do (apply function child function-args)))
 
-;;; XXX bogus for now.
-(defclass standard-tree-output-record (standard-sequence-output-record)
-  (
-   ))
+
+;;; tree output recording
+
+(defclass tree-output-record-entry ()
+     ((record :initarg :record :reader tree-output-record-entry-record)
+      (cached-rectangle :initform nil :accessor tree-output-record-entry-cached-rectangle)
+      (inserted-nr :initarg :inserted-nr :accessor tree-output-record-entry-inserted-nr)))
+
+(defun make-tree-output-record-entry (record inserted-nr)
+  (make-instance 'tree-output-record-entry :record record :inserted-nr inserted-nr))
+
+(defun %record-to-spatial-tree-rectangle (r)
+  (rectangles:make-rectangle
+   :lows `(,(bounding-rectangle-min-x r)
+            ,(bounding-rectangle-min-y r))
+   :highs `(,(bounding-rectangle-max-x r)
+             ,(bounding-rectangle-max-y r))))
+
+(defun %output-record-entry-to-spatial-tree-rectangle (r)
+  (when (null (tree-output-record-entry-cached-rectangle r))
+    (let* ((record (tree-output-record-entry-record r)))
+      (setf (tree-output-record-entry-cached-rectangle r) (%record-to-spatial-tree-rectangle record))))
+  (tree-output-record-entry-cached-rectangle r))
+
+(defun %make-tree-output-record-tree ()
+  (spatial-trees:make-spatial-tree :r
+                        :rectfun #'%output-record-entry-to-spatial-tree-rectangle))
+
+(defclass standard-tree-output-record (compound-output-record)
+  ((children :initform (%make-tree-output-record-tree)
+             :accessor %tree-record-children)
+   (children-hash :initform (make-hash-table :test #'eql) :reader %tree-record-children-cache)
+   (last-insertion-nr :initform 0 :accessor last-insertion-nr)))
+
+(defun %entry-in-children-cache (record entry)
+  (gethash entry (%tree-record-children-cache record)))
+
+(defun (setf %entry-in-children-cache) (new-val record entry)
+  (setf (gethash entry (%tree-record-children-cache record)) new-val))
+
+(defun %remove-entry-from-children-cache (record entry)
+  (remhash entry (%tree-record-children-cache record)))
+
+(defmethod output-record-children ((record standard-tree-output-record))
+  (map 'list
+       #'tree-output-record-entry-record
+       (spatial-trees:search (%record-to-spatial-tree-rectangle record)
+                             (%tree-record-children record))))
+
+(defmethod add-output-record (child (record standard-tree-output-record))
+  (let ((entry (make-tree-output-record-entry child (incf (last-insertion-nr record)))))
+    (spatial-trees:insert entry (%tree-record-children record))
+    (setf (output-record-parent child) record)
+    (setf (%entry-in-children-cache record child) entry)))
+
+(defmethod delete-output-record (child (record standard-tree-output-record) &optional (errorp t))
+  (let ((entry (find child (spatial-trees:search (%entry-in-children-cache record child)
+                                                 (%tree-record-children record))
+                     :key #'tree-output-record-entry-record)))
+   (cond
+     ((not (null entry))
+      (spatial-trees:delete entry (%tree-record-children record))
+      (%remove-entry-from-children-cache record child)
+      (setf (output-record-parent child) nil))
+     (errorp (error "~S is not a child of ~S" child record)))))
+
+(defmethod clear-output-record ((record standard-tree-output-record))
+  (dolist (child (output-record-children record))
+    (setf (output-record-parent child) nil)
+    (%remove-entry-from-children-cache record child))
+  (setf (%tree-record-children record) (%make-tree-output-record-tree)))
+
+(defun map-over-tree-output-records (function record rectangle sort-order function-args)
+  (dolist (child (sort (spatial-trees:search rectangle
+                                             (%tree-record-children record))
+                       (ecase sort-order
+                         (:most-recent-first #'>)
+                         (:most-recent-last #'<))
+                       :key #'tree-output-record-entry-inserted-nr))
+    (apply function (tree-output-record-entry-record child) function-args)))
+
+(defmethod map-over-output-records-1 (function (record standard-tree-output-record) function-args)
+  (map-over-tree-output-records function record (%record-to-spatial-tree-rectangle record) :most-recent-last
+                                function-args))
+
+(defmethod map-over-output-records-containing-position (function (record standard-tree-output-record) x y &optional x-offset y-offset &rest function-args)
+  (declare (ignore x-offset y-offset))
+  (map-over-tree-output-records function record (rectangles:make-rectangle :lows `(,x ,y) :highs `(,x ,y)) :most-recent-first
+                                function-args)) 
+
+(defmethod map-over-output-records-overlapping-region (function (record standard-tree-output-record) region &optional x-offset y-offset &rest function-args)
+  (declare (ignore x-offset y-offset))
+  (typecase region
+    (everywhere-region (map-over-output-records-1 function record function-args))
+    (nowhere-region nil)
+    (otherwise (map-over-tree-output-records
+                (lambda (child)
+                  (if (region-intersects-region-p (multiple-value-call 'make-rectangle* (bounding-rectangle* child))
+                                                     region)
+                       (apply function child function-args)))
+                record (%record-to-spatial-tree-rectangle (bounding-rectangle region)) :most-recent-last
+                nil))))
+
+(defmethod recompute-extent-for-changed-child :around ((record standard-tree-output-record) child old-min-x old-min-y old-max-x old-max-y)
+  (when (eql record (output-record-parent child))
+    (let ((entry (%entry-in-children-cache record child)))
+     (spatial-trees:delete entry (%tree-record-children record))
+     (setf (tree-output-record-entry-cached-rectangle entry) nil)
+     (spatial-trees:insert entry (%tree-record-children record))))
+  (call-next-method))
+
+;;;
 
 (defmethod match-output-records ((record t) &rest args)
   (apply #'match-output-records-1 record args))
@@ -1267,7 +1360,6 @@ were added."
                   ,class-vars)
                 (defmethod initialize-instance :after ((graphic ,class-name)
 						       &key)
-                  (declare (ignore args))
                   (with-slots (stream ink clipping-region
                                line-style text-style ,@args)
                       graphic
@@ -1644,7 +1736,7 @@ were added."
 (def-grecording draw-text ((gs-text-style-mixin) string point-x point-y start end
 			   align-x align-y toward-x toward-y transform-glyphs) ()
   ;; FIXME!!! Text direction.
-  ;; Multiple lines?
+  ;; FIXME: Multiple lines.
  (let* ((text-style (graphics-state-text-style graphic))
 	(width (if (characterp string)
 		   (stream-character-width stream string :text-style text-style)
@@ -1654,27 +1746,26 @@ were added."
         (ascent (text-style-ascent text-style (sheet-medium stream)))
         (descent (text-style-descent text-style (sheet-medium stream)))
         (height (+ ascent descent))
-	(transform (medium-transformation medium))
-        left top right bottom)
+	(transform (medium-transformation medium)))
    (setf (values point-x point-y)
 	 (transform-position transform point-x point-y))
-   (ecase align-x
-     (:left (setq left point-x
-                  right (+ point-x width)))
-     (:right (setq left (- point-x width)
-                   right point-x))
-     (:center (setq left (- point-x (round width 2))
-                    right (+ point-x (round width 2)))))
-   (ecase align-y
-     (:baseline (setq top (- point-y ascent)
-                      bottom (+ point-y descent)))
-     (:top (setq top point-y
-                 bottom (+ point-y height)))
-     (:bottom (setq top (- point-y height)
-                    bottom point-y))
-     (:center (setq top (- point-y (floor height 2))
-                    bottom (+ point-y (ceiling height 2)))))
-   (values left top right bottom)))
+   (multiple-value-bind (left top right bottom)
+       (text-bounding-rectangle* medium string 
+                                 :start start :end end :text-style text-style)
+     (ecase align-x
+       (:left (incf left point-x) (incf right point-x))
+       (:right (incf left (- point-x width)) (incf right (- point-x width)))
+       (:center (incf left (- point-x (round width 2)))
+                (incf right (- point-x (round width 2)))))
+     (ecase align-y
+       (:baseline (incf top point-y) (incf bottom point-y))
+       (:top (incf top (+ point-y ascent))
+             (incf bottom (+ point-y ascent)))
+       (:bottom (incf top (- point-y descent))
+                (incf bottom (- point-y descent)))
+       (:center (incf top (+ point-y (ceiling (- ascent descent) 2)))
+                (incf bottom (+ point-y (ceiling (- ascent descent) 2)))))
+     (values left top right bottom))))
 
 (defmethod* (setf output-record-position) :around
     (nx ny (record draw-text-output-record))
@@ -1736,6 +1827,15 @@ were added."
    (baseline :initform 0)
    (width :initform 0)
    (max-height :initform 0)
+   ;; FIXME (or rework this comment): CLIM does not separate the
+   ;; notions of the text width and the bounding box; however, we need
+   ;; to, because some fonts will render outside the logical
+   ;; coordinates defined by the start position and the width.  LEFT
+   ;; and RIGHT here (and below) deal with this in a manner completely
+   ;; hidden from the user.  (should we export
+   ;; TEXT-BOUNDING-RECTANGLE*?)
+   (left :initarg :start-x)
+   (right :initarg :start-x)
    (start-x :initarg :start-x)
    (start-y :initarg :start-y)
    (end-x :initarg :start-x)
@@ -1757,12 +1857,14 @@ were added."
     ((record standard-text-displayed-output-record)
      (record2 standard-text-displayed-output-record))
   (with-slots
-	(initial-x1 initial-y1 start-x start-y end-x end-y wrapped strings)
+	(initial-x1 initial-y1 start-x start-y left right end-x end-y wrapped strings)
       record2
     (and (coordinate= (slot-value record 'initial-x1) initial-x1)
 	 (coordinate= (slot-value record 'initial-y1) initial-y1)
 	 (coordinate= (slot-value record 'start-x) start-x)
 	 (coordinate= (slot-value record 'start-y) start-y)
+         (coordinate= (slot-value record 'left) left)
+         (coordinate= (slot-value record 'right) right)
 	 (coordinate= (slot-value record 'end-x) end-x)
 	 (coordinate= (slot-value record 'end-y) end-y)
 	 (eq (slot-value record 'wrapped) wrapped)
@@ -1835,20 +1937,21 @@ were added."
 
 (defmethod tree-recompute-extent
     ((text-record standard-text-displayed-output-record))
-  (with-standard-rectangle* (:x1 x1 :y1 y1)
+  (with-standard-rectangle* (:y1 y1)
       text-record
-    (with-slots (width max-height)
+    (with-slots (max-height left right)
 	text-record
       (setf (rectangle-edges* text-record)
-	    (values x1 y1
-		    (coordinate (+ x1 width))
-		    (coordinate (+ y1 max-height))))))
+	    (values (coordinate left)
+                    y1 
+                    (coordinate right)
+                    (coordinate (+ y1 max-height))))))
   text-record)
 
 (defmethod add-character-output-to-text-record ; XXX OAOO with ADD-STRING-...
     ((text-record standard-text-displayed-output-record)
      character text-style char-width height new-baseline)
-  (with-slots (strings baseline width max-height start-y end-x end-y medium)
+  (with-slots (strings baseline width max-height left right start-y end-x end-y medium)
       text-record
     (if (and strings
 	     (let ((string (last1 strings)))
@@ -1868,11 +1971,18 @@ were added."
 					   :element-type 'character
 					   :adjustable t
 					   :fill-pointer t)))))
-    (setq baseline (max baseline new-baseline)
-	  end-x (+ end-x char-width)
-	  max-height (max max-height height)
-	  end-y (max end-y (+ start-y max-height))
-	  width (+ width char-width)))
+    (multiple-value-bind (minx miny maxx maxy)
+        (text-bounding-rectangle* medium character :text-style text-style)
+      (declare (ignore miny maxy))
+      (setq baseline (max baseline new-baseline)
+            ;; KLUDGE: note END-X here is really START-X of the new
+            ;; string
+            left (min left (+ end-x minx))
+            end-x (+ end-x char-width)
+            right (+ end-x (max 0 (- maxx char-width)))
+            max-height (max max-height height)
+            end-y (max end-y (+ start-y max-height))
+            width (+ width char-width))))
   (tree-recompute-extent text-record))
 
 (defmethod add-string-output-to-text-record
@@ -1886,7 +1996,7 @@ were added."
                                             (aref string start)
                                             text-style
                                             string-width height new-baseline))
-      (t (with-slots (strings baseline width max-height start-y end-x end-y
+      (t (with-slots (strings baseline width max-height left right start-y end-x end-y
 		      medium)
 	     text-record
 	   (let ((styled-string (make-instance
@@ -1901,12 +2011,21 @@ were added."
 	     (nconcf strings (list styled-string))
 	     (replace (styled-string-string styled-string) string
 		      :start2 start :end2 end))
-	   (setq baseline (max baseline new-baseline)
-		 end-x (+ end-x string-width)
-		 max-height (max max-height height)
-		 end-y (max end-y (+ start-y max-height))
-		 width (+ width string-width)))
-       (tree-recompute-extent text-record)))))
+           (multiple-value-bind (minx miny maxx maxy)
+               (text-bounding-rectangle* medium string
+                                         :text-style text-style
+                                         :start start :end end)
+             (declare (ignore miny maxy))
+             (setq baseline (max baseline new-baseline)
+                   ;; KLUDGE: note that END-X here really means
+                   ;; START-X of the new string.
+                   left (min left (+ end-x minx))
+                   end-x (+ end-x string-width)
+                   right (+ end-x (max 0 (- maxx string-width)))
+                   max-height (max max-height height)
+                   end-y (max end-y (+ start-y max-height))
+                   width (+ width string-width))))
+         (tree-recompute-extent text-record)))))
 
 (defmethod text-displayed-output-record-string
     ((record standard-text-displayed-output-record))
@@ -2212,11 +2331,14 @@ according to the flags RECORD and DRAW."
 
 (defun %handle-repaint (stream region)
   (when (output-recording-stream-p stream)
-    (let ((region (bounding-rectangle region)))
-      (with-bounding-rectangle* (x1 y1 x2 y2) region
-        (with-output-recording-options (stream :record nil)
-          (draw-rectangle* stream x1 y1 x2 y2 :filled T :ink +background-ink+)))
-      (stream-replay stream region))))
+    (unless (region-equal region +nowhere+)                    ; ignore repaint requests for +nowhere+
+      (let ((region (if (region-equal region +everywhere+)
+			(sheet-region stream)                  ; fallback to the sheet's region for +everwhere+
+		      (bounding-rectangle region))))
+	(with-bounding-rectangle* (x1 y1 x2 y2) region
+	  (with-output-recording-options (stream :record nil)
+	    (draw-rectangle* stream x1 y1 x2 y2 :filled T :ink +background-ink+)))
+	(stream-replay stream region)))))
 
 (defmethod handle-repaint ((stream output-recording-stream) region)
   (%handle-repaint stream region))
