@@ -144,8 +144,6 @@ sheet-supports-only-one-child error to be signalled."))
 ;;;;
 ;;;; sheet protocol class
 
-(define-protocol-class sheet (bounding-rectangle))
-
 (defclass basic-sheet (sheet)
   ((region :type region
 	   :initarg :region
@@ -503,14 +501,16 @@ sheet-supports-only-one-child error to be signalled."))
   (setf (sheet-parent child) nil))
 
 (defmethod raise-sheet ((sheet sheet-parent-mixin))
-  (when (not (sheet-parent sheet))
-    (error 'sheet-is-not-child))
-  (raise-sheet-internal sheet (sheet-parent sheet)))
+  (when (sheet-parent sheet)
+    (raise-sheet-internal sheet (sheet-parent sheet)))
+  (when (sheet-direct-mirror sheet)
+    (raise-mirror (port sheet) sheet)))
 
 (defmethod bury-sheet ((sheet sheet-parent-mixin))
-  (when (not (sheet-parent sheet))
-    (error 'sheet-is-not-child))
-  (bury-sheet-internal sheet (sheet-parent sheet)))
+  (when (sheet-parent sheet)
+    (bury-sheet-internal sheet (sheet-parent sheet)))
+  (when (sheet-direct-mirror sheet)
+    (bury-mirror (port sheet) sheet)))
 
 (defmethod graft ((sheet sheet-parent-mixin))
   (graft (sheet-parent sheet)))
