@@ -117,6 +117,17 @@
 	else
 	  do (handle-event (event-sheet event) event))))
 
+(defmethod stream-clear-input ((pane standard-input-stream))
+  (setf (stream-unread-chars pane) nil)
+  (loop for event = (event-read-no-hang pane)
+	if (null event)
+	   return nil
+	else
+	do (handle-event (event-sheet event) event))
+  nil)
+
+;;; XXX The should be moved to protocol-classes.lisp and the
+;;; standard-sheet-input-mixin superclass should be removed.
 (define-protocol-class extended-input-stream (fundamental-character-input-stream ;Gray stream
 					      standard-sheet-input-mixin)
   ())
@@ -382,6 +393,18 @@
 	 do (stream-read-gesture estream) ; consume pointer gesture
 	 finally (return (characterp char)))))
 
+(defmethod stream-clear-input ((stream standard-extended-input-stream))
+  (with-encapsulating-stream (estream stream)
+    (loop
+       with char and reason
+	 do (setf (values char reason) (stream-read-gesture estream
+							    :timeout 0
+							    :peek-p t))
+	 until (or (eq reason :eof) (eq reason :timeout))
+	 do (stream-read-gesture estream) ; consume pointer gesture
+	 ))
+  nil)
+
 ;;; stream-read-line returns a second value of t if terminated by eof.
 (defmethod stream-read-line ((stream standard-extended-input-stream))
   (with-encapsulating-stream (estream stream)
@@ -637,6 +660,16 @@
 (define-protocol-class pointer ()
   ((port :reader port :initarg :port)))
 
+;;; FIXME: I think the standard-pointer should absorb some of the
+;;; common methods that are currently entirely provided by the
+;;; backends.
+
+(defclass standard-pointer (pointer)
+  ((port :reader port :initarg :port)
+   (state-lock :reader state-lock :initform (make-lock "pointer lock"))
+   (button-state :initform 0 )
+   (modifier-state :initform 0)))
+
 (defgeneric pointer-sheet (pointer))
 
 (defmethod pointer-sheet ((pointer pointer))
@@ -671,8 +704,37 @@
   (with-accessors ((port-pointer-sheet port-pointer-sheet))
       (port sheet)
     (when (eq port-pointer-sheet sheet)
+
       (setq port-pointer-sheet nil))))
 
+(defmethod pointer-button-state ((pointer standard-pointer))
+  (with-lock-held ((state-lock pointer))
+    (slot-value pointer 'button-state)))
+
+(defmethod pointer-modifier-state ((pointer standard-pointer))
+  (with-lock-held ((state-lock pointer))
+    (slot-value pointer 'modifier-state)))
+
+(defmethod pointer-update-state
+    ((pointer standard-pointer) (event keyboard-event))
+  (with-lock-held ((state-lock pointer))
+    (setf (slot-value pointer 'modifier-state) (event-modifier-state event))))
+
+(defmethod pointer-update-state
+    ((pointer standard-pointer) (event pointer-button-press-event))
+  (with-lock-held ((state-lock pointer))
+    (setf (slot-value pointer 'button-state)
+	  (logior (slot-value pointer 'button-state)
+		  (pointer-event-button event)))))
+
+(defmethod pointer-update-state
+    ((pointer standard-pointer) (event pointer-button-release-event))
+  (with-lock-held ((state-lock pointer))
+    (setf (slot-value pointer 'button-state)
+	  (logandc2 (slot-value pointer 'button-state)
+		    (pointer-event-button event)))))
+
+(defmethod pointer-butt)
 (defgeneric stream-pointer-position (stream &key pointer))
 
 (defmethod stream-pointer-position ((stream standard-extended-input-stream)
