@@ -52,7 +52,6 @@
 
 
 ;;; Cursor class
-(define-protocol-class cursor ())
 
 (defgeneric cursor-sheet (cursor))
 
@@ -78,6 +77,9 @@
    (x :initform 0 :initarg :x-position)
    (y :initform 0 :initarg :y-position)
    (width :initform 8)
+   (appearance :type (member :solid :hollow) 
+               :initarg :appearance :initform :hollow
+               :accessor cursor-appearance)
    ;; XXX what does "cursor is active" mean?
    ;; It means that the sheet (stream) updates the cursor, though
    ;; currently the cursor appears to be always updated after stream
@@ -142,7 +144,8 @@
 	(draw-rectangle* (sheet-medium (cursor-sheet cursor))
 			 x y
 			 (+ x width) (+ y height)
-			 :filled t
+			 :filled (ecase (cursor-appearance cursor)
+                                   (:solid t) (:hollow nil))
 			 :ink +flipping-ink+)))))
 
 (defmethod display-cursor ((cursor cursor-mixin) state)
@@ -154,7 +157,8 @@
 	(:draw (draw-rectangle* (sheet-medium (cursor-sheet cursor))
 				x y
 				(+ x width) (+ y height)
-				:filled t
+				:filled (ecase (cursor-appearance cursor)
+                                   (:solid t) (:hollow nil))
 				:ink +foreground-ink+
 				))       
         (:erase
@@ -168,7 +172,8 @@
                 (draw-rectangle* (sheet-medium (cursor-sheet cursor))
                                  x y
                                  (+ x width) (+ y height)
-                                 :filled t
+                                 :filled (ecase (cursor-appearance cursor)
+                                   (:solid t) (:hollow nil))
                                  :ink +background-ink+))))))
 
 ;;; Standard-Text-Cursor class
@@ -182,13 +187,6 @@
 
 ;;; Extended-Output-Stream class
 
-(define-protocol-class extended-output-stream
-    (fundamental-character-output-stream)
-  ;; CLIM Specification says that E-O-S is a subclass of
-  ;; OUTPUT-STREAM, but it does not says what is it.
-  ()
-  )
-
 ;;; Stream text cursor protocol
 (defgeneric stream-text-cursor (stream))
 (defgeneric (setf stream-text-cursor) (cursor stream))
@@ -358,9 +356,7 @@
 	    (unless (= start split)
 	      (stream-write-output stream
 				   string
-				   (if (eql end split)
-				       width
-				       nil)
+                                   nil
 				   start split)
 	      (setq cx (+ cx width))	      
           (with-slots (x y) (stream-text-cursor stream)
@@ -487,15 +483,15 @@ STREAM-STRING-WIDTH will be called."))
       (declare (ignore total-height final-y baseline))
       (values final-x total-width))))
 
-;;; XXX where does "6" come from?  Magic fudge factor? -- moore
 (defmethod stream-text-margin ((stream standard-extended-output-stream))
   (with-slots (margin) stream
     (or margin
 	(- (bounding-rectangle-width (or (pane-viewport stream)
                                          stream))
-	   6))))
+	   (text-size stream "O")))))
 
-(defmethod stream-line-height ((stream standard-extended-output-stream) &key (text-style nil))
+(defmethod stream-line-height ((stream standard-extended-output-stream)
+                               &key (text-style nil))
   (+ (text-style-height (or text-style (medium-text-style (sheet-medium stream)))
 			(sheet-medium stream))
      (stream-vertical-spacing stream)))
@@ -540,9 +536,10 @@ STREAM-STRING-WIDTH will be called."))
      ,@body))
 
 (defmethod beep (&optional medium)
-  (when (null medium)
-    (setq medium (sheet-medium *standard-output*)))
-  (medium-beep medium))
-
+  (if medium
+      (medium-beep medium)
+      (when (sheetp *standard-output*)
+        (medium-beep (sheet-medium *standard-output*)))))
+  
 (defmethod scroll-quantum ((sheet standard-extended-output-stream))
   (stream-line-height sheet))
