@@ -21,12 +21,12 @@
 
 (defun get-environment-variable (string)
   #+excl (sys:getenv string)
-  #+cmu (cdr (assoc string ext:*environment-list* :test #'string=))
+  #+(or cmu scl) (cdr (assoc string ext:*environment-list* :test #'string=))
   #+clisp (ext:getenv (string string))
   #+sbcl (sb-ext::posix-getenv string)
   #+openmcl (ccl::getenv string)
   #+lispworks (lw:environment-variable string)
-  #-(or excl cmu clisp sbcl openmcl lispworks)
+  #-(or excl cmu scl clisp sbcl openmcl lispworks)
   (error "GET-ENVIRONMENT-VARIABLE not implemented"))
 
 ;;; It would be nice to define this macro in terms of letf, but that
@@ -265,33 +265,6 @@ by the number of variables in VARS."
   "Clamps the value 'value' into the range [min,max]."
   (max min (min max value)))
   
-;;;;
-;;;; Protocol Classes
-;;;;
-
-(defmacro define-protocol-class (name super-classes &optional slots &rest options)
-  (let ((protocol-predicate
-         (intern (concatenate 'string (symbol-name name) (if (find #\- (symbol-name name)) "-" "") "P")))
-	(predicate-docstring
-	 (concatenate 'string "Protocol predicate checking for class " (symbol-name name))))
-    `(progn
-       (defclass ,name ,super-classes ,slots ,@options)
-
-       (let ((the-class (find-class ',name)))
-	 (setf (documentation the-class 'type) "CLIM protocol class")
-         (defmethod initialize-instance :after ((object ,name) &key &allow-other-keys)
-           (when (eq (class-of object) the-class)
-             (error "~S is a protocol class and thus can't be instantiated" ',name))))
-
-       (defgeneric ,protocol-predicate (object)
-	 (:method ((object t))
-	   nil)
-	 (:method ((object ,name))
-	   t)
-	 (:documentation ,predicate-docstring))
-
-       ',name)))
-
 ;;;;
 ;;;; meta functions
 ;;;;
@@ -461,21 +434,27 @@ in KEYWORDS removed."
         (t
          (error "~S Can not be a stream designator for ~S" symbol default))))
 
+(defun declare-ignorable-form (variables)
+  #+CMU
+  ;; CMUCL barfs if you declare a special variable ignorable, work
+  ;; around that.
+  `(declare (ignorable
+             ,@(remove-if (lambda (symbol)
+                            (eq :special (lisp::info lisp::variable lisp::kind symbol)))
+                          variables)))
+  #-CMU
+  `(declare (ignorable ,@variables)))
+
+;; spread version:
+
+(defun declare-ignorable-form* (&rest variables)
+  (declare-ignorable-form variables))
+  
 (defun gen-invoke-trampoline (fun to-bind to-pass body)
   "Macro helper function, generates the LABELS / INVOKE-WITH-... ideom."
   (let ((cont (gensym ".CONT.")))
     `(labels ((,cont (,@to-bind)
-               #+CMU
-               ;; for some reason CMUCL barfs if we declare a special
-               ;; variable to be ignored. so we take an alternate
-               ;; route.
-               ;; --GB 2003-06-05
-               (progn
-                 ,@to-bind
-                 (locally ,@body))
-               #-CMU
-               (declare (ignorable ,@to-bind))
-               #-CMU
+               ,(declare-ignorable-form to-bind)
                ,@body))
       (declare (dynamic-extent #',cont))
       (,fun ,@to-bind #',cont ,@to-pass))))
@@ -487,7 +466,7 @@ in KEYWORDS removed."
 STREAM in the direction DIRECTION."
   ;; This implementation lives unter the assumption that an
   ;; extended-output stream is also a sheet and has a graft. 
-  ;; --GB2002-08-14
+  ;; --GB 2002-08-14
   (etypecase specification
     (integer specification)
     ((or string character) (multiple-value-bind (width height)
@@ -585,3 +564,28 @@ STREAM in the direction DIRECTION."
        and collect var into new-arg-list
      end
      finally (return (values bindings new-arg-list))))
+
+(defun make-keyword (obj)
+  "Turn OBJ into a keyword"
+  (etypecase obj
+    (keyword
+     obj)
+    (symbol
+     (intern (symbol-name obj) :keyword))
+    (string
+     (intern (string-upcase obj) :keyword))))
+
+;;; Command name utilities that are useful elsewhere.
+
+(defun command-name-from-symbol (symbol)
+  (let ((name (symbol-name symbol)))
+    (string-capitalize
+     (substitute
+      #\Space #\-
+      (subseq name (if (string= '#:com- name :end2 (min (length name) 4))
+		       4
+		       0))))))
+
+(defun keyword-arg-name-from-symbol (symbol)
+  (let ((name (symbol-name symbol)))
+    (string-capitalize (substitute #\Space #\- name))))
